@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2020 Intel Corporation
+* Copyright 2016-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -80,13 +80,17 @@ const alg_kind_t eltwise_linear = dnnl_eltwise_linear;
 const alg_kind_t eltwise_bounded_relu = dnnl_eltwise_bounded_relu;
 const alg_kind_t eltwise_soft_relu = dnnl_eltwise_soft_relu;
 const alg_kind_t eltwise_logistic = dnnl_eltwise_logistic;
+const alg_kind_t eltwise_logsigmoid = dnnl_eltwise_logsigmoid;
+const alg_kind_t eltwise_mish = dnnl_eltwise_mish;
 const alg_kind_t eltwise_exp = dnnl_eltwise_exp;
 const alg_kind_t eltwise_gelu = dnnl_eltwise_gelu;
 const alg_kind_t eltwise_log = dnnl_eltwise_log;
 const alg_kind_t eltwise_clip = dnnl_eltwise_clip;
+const alg_kind_t eltwise_clip_v2 = dnnl_eltwise_clip_v2;
 const alg_kind_t eltwise_pow = dnnl_eltwise_pow;
 const alg_kind_t eltwise_gelu_tanh = dnnl_eltwise_gelu_tanh;
 const alg_kind_t eltwise_gelu_erf = dnnl_eltwise_gelu_erf;
+const alg_kind_t eltwise_hardswish = dnnl_eltwise_hardswish;
 const alg_kind_t eltwise_relu_use_dst_for_bwd
         = dnnl_eltwise_relu_use_dst_for_bwd;
 const alg_kind_t eltwise_tanh_use_dst_for_bwd
@@ -97,6 +101,8 @@ const alg_kind_t eltwise_sqrt_use_dst_for_bwd
 const alg_kind_t eltwise_logistic_use_dst_for_bwd
         = dnnl_eltwise_logistic_use_dst_for_bwd;
 const alg_kind_t eltwise_exp_use_dst_for_bwd = dnnl_eltwise_exp_use_dst_for_bwd;
+const alg_kind_t eltwise_clip_v2_use_dst_for_bwd
+        = dnnl_eltwise_clip_v2_use_dst_for_bwd;
 const alg_kind_t eltwise_round = dnnl_eltwise_round;
 const alg_kind_t pooling_max = dnnl_pooling_max;
 const alg_kind_t pooling_avg = dnnl_pooling_avg;
@@ -113,6 +119,13 @@ const alg_kind_t binary_mul = dnnl_binary_mul;
 const alg_kind_t binary_max = dnnl_binary_max;
 const alg_kind_t binary_min = dnnl_binary_min;
 const alg_kind_t binary_div = dnnl_binary_div;
+const alg_kind_t binary_sub = dnnl_binary_sub;
+const alg_kind_t binary_ge = dnnl_binary_ge;
+const alg_kind_t binary_gt = dnnl_binary_gt;
+const alg_kind_t binary_le = dnnl_binary_le;
+const alg_kind_t binary_lt = dnnl_binary_lt;
+const alg_kind_t binary_eq = dnnl_binary_eq;
+const alg_kind_t binary_ne = dnnl_binary_ne;
 const alg_kind_t resampling_nearest = dnnl_resampling_nearest;
 const alg_kind_t resampling_linear = dnnl_resampling_linear;
 const alg_kind_t reduction_max = dnnl_reduction_max;
@@ -150,6 +163,7 @@ namespace rnn_packed_format {
 const rnn_packed_format_t undef = dnnl_packed_format_undef;
 const rnn_packed_format_t ldigo_p = dnnl_ldigo_p;
 const rnn_packed_format_t ldgoi_p = dnnl_ldgoi_p;
+const rnn_packed_format_t ldio_p = dnnl_ldio_p;
 } // namespace rnn_packed_format
 
 using format_kind_t = dnnl_format_kind_t;
@@ -188,9 +202,11 @@ const format_tag_t abced = dnnl_abced;
 const format_tag_t abdc = dnnl_abdc;
 const format_tag_t acbd = dnnl_acbd;
 const format_tag_t abdec = dnnl_abdec;
+const format_tag_t abdfce = dnnl_abdfce;
 const format_tag_t acb = dnnl_acb;
 const format_tag_t acbde = dnnl_acbde;
 const format_tag_t acbdef = dnnl_acbdef;
+const format_tag_t abdefc = dnnl_abdefc;
 const format_tag_t acdb = dnnl_acdb;
 const format_tag_t acdeb = dnnl_acdeb;
 const format_tag_t ba = dnnl_ba;
@@ -206,18 +222,46 @@ const format_tag_t dcab = dnnl_dcab;
 const format_tag_t cdeba = dnnl_cdeba;
 const format_tag_t decab = dnnl_decab;
 const format_tag_t defcab = dnnl_defcab;
+const format_tag_t AB16b16a = dnnl_AB16b16a;
+const format_tag_t AB16b32a = dnnl_AB16b32a;
+const format_tag_t AB16b64a = dnnl_AB16b64a;
+const format_tag_t AB8b16a2b = dnnl_AB8b16a2b;
+const format_tag_t AB8b32a2b = dnnl_AB8b32a2b;
+const format_tag_t AB8b64a2b = dnnl_AB8b64a2b;
+const format_tag_t AB4b16a4b = dnnl_AB4b16a4b;
+const format_tag_t AB4b32a4b = dnnl_AB4b32a4b;
+const format_tag_t AB4b64a4b = dnnl_AB4b64a4b;
+const format_tag_t AB16b16a4b = dnnl_AB16b16a4b;
+const format_tag_t AB16b32a4b = dnnl_AB16b32a4b;
+const format_tag_t AB16b48a4b = dnnl_AB16b48a4b;
+const format_tag_t AB16b64a4b = dnnl_AB16b64a4b;
+const format_tag_t AB16b16a2b = dnnl_AB16b16a2b;
+const format_tag_t AB16b32a2b = dnnl_AB16b32a2b;
+const format_tag_t AB16b48a2b = dnnl_AB16b48a2b;
+const format_tag_t AB16b64a2b = dnnl_AB16b64a2b;
 const format_tag_t Abc16a = dnnl_Abc16a;
 const format_tag_t ABc16a16b = dnnl_ABc16a16b;
 const format_tag_t ABc4a4b = dnnl_ABc4a4b;
 const format_tag_t aBc16b = dnnl_aBc16b;
 const format_tag_t aBc32b = dnnl_aBc32b;
 const format_tag_t ABc16b16a = dnnl_ABc16b16a;
+const format_tag_t ABc16b32a = dnnl_ABc16b32a;
+const format_tag_t ABc16b64a = dnnl_ABc16b64a;
 const format_tag_t Abc4a = dnnl_Abc4a;
 const format_tag_t aBc4b = dnnl_aBc4b;
 const format_tag_t ABc4b16a4b = dnnl_ABc4b16a4b;
+const format_tag_t ABc4b32a4b = dnnl_ABc4b32a4b;
+const format_tag_t ABc4b64a4b = dnnl_ABc4b64a4b;
 const format_tag_t ABc2b8a4b = dnnl_ABc2b8a4b;
 const format_tag_t ABc16b16a4b = dnnl_ABc16b16a4b;
+const format_tag_t ABc16b32a4b = dnnl_ABc16b32a4b;
+const format_tag_t ABc16b48a4b = dnnl_ABc16b48a4b;
+const format_tag_t ABc16b64a4b = dnnl_ABc16b64a4b;
 const format_tag_t ABc16b16a2b = dnnl_ABc16b16a2b;
+const format_tag_t ABc16b32a2b = dnnl_ABc16b32a2b;
+const format_tag_t ABc16b48a2b = dnnl_ABc16b48a2b;
+const format_tag_t ABc16b64a2b = dnnl_ABc16b64a2b;
+const format_tag_t ABc16a16b2a = dnnl_ABc16a16b2a;
 const format_tag_t ABc4b4a = dnnl_ABc4b4a;
 const format_tag_t ABc8a16b2a = dnnl_ABc8a16b2a;
 const format_tag_t BAc8a16b2a = dnnl_BAc8a16b2a;
@@ -225,6 +269,8 @@ const format_tag_t ABc8a8b = dnnl_ABc8a8b;
 const format_tag_t ABc8a4b = dnnl_ABc8a4b;
 const format_tag_t aBc8b = dnnl_aBc8b;
 const format_tag_t ABc8b16a2b = dnnl_ABc8b16a2b;
+const format_tag_t ABc8b32a2b = dnnl_ABc8b32a2b;
+const format_tag_t ABc8b64a2b = dnnl_ABc8b64a2b;
 const format_tag_t ABc8b8a = dnnl_ABc8b8a;
 const format_tag_t Abcd16a = dnnl_Abcd16a;
 const format_tag_t Abcd8a = dnnl_Abcd8a;
@@ -233,19 +279,31 @@ const format_tag_t ABcd16a16b = dnnl_ABcd16a16b;
 const format_tag_t aBcd16b = dnnl_aBcd16b;
 const format_tag_t aBcd32b = dnnl_aBcd32b;
 const format_tag_t ABcd16b16a = dnnl_ABcd16b16a;
+const format_tag_t ABcd16b32a = dnnl_ABcd16b32a;
+const format_tag_t ABcd16b64a = dnnl_ABcd16b64a;
 const format_tag_t aBCd16b16c = dnnl_aBCd16b16c;
 const format_tag_t aBCd16c16b = dnnl_aBCd16c16b;
 const format_tag_t Abcd4a = dnnl_Abcd4a;
 const format_tag_t aBcd4b = dnnl_aBcd4b;
 const format_tag_t ABcd4b16a4b = dnnl_ABcd4b16a4b;
+const format_tag_t ABcd4b32a4b = dnnl_ABcd4b32a4b;
+const format_tag_t ABcd4b64a4b = dnnl_ABcd4b64a4b;
 const format_tag_t ABcd16b16a4b = dnnl_ABcd16b16a4b;
+const format_tag_t ABcd16b32a4b = dnnl_ABcd16b32a4b;
+const format_tag_t ABcd16b48a4b = dnnl_ABcd16b48a4b;
+const format_tag_t ABcd16b64a4b = dnnl_ABcd16b64a4b;
 const format_tag_t ABcd16b16a2b = dnnl_ABcd16b16a2b;
+const format_tag_t ABcd16b32a2b = dnnl_ABcd16b32a2b;
+const format_tag_t ABcd16b48a2b = dnnl_ABcd16b48a2b;
+const format_tag_t ABcd16b64a2b = dnnl_ABcd16b64a2b;
+const format_tag_t ABcd16a16b2a = dnnl_ABcd16a16b2a;
 const format_tag_t ABcd4b4a = dnnl_ABcd4b4a;
 const format_tag_t ABcd4a4b = dnnl_ABcd4a4b;
 const format_tag_t aBCd4c16b4c = dnnl_aBCd4c16b4c;
 const format_tag_t aBCd2c8b4c = dnnl_aBCd2c8b4c;
 const format_tag_t aBCd16c16b4c = dnnl_aBCd16c16b4c;
 const format_tag_t aBCd16c16b2c = dnnl_aBCd16c16b2c;
+const format_tag_t aBCd16b16c2b = dnnl_aBCd16b16c2b;
 const format_tag_t aBCd4c4b = dnnl_aBCd4c4b;
 const format_tag_t aBCd4b4c = dnnl_aBCd4b4c;
 const format_tag_t ABcd8a16b2a = dnnl_ABcd8a16b2a;
@@ -254,6 +312,8 @@ const format_tag_t ABcd8a8b = dnnl_ABcd8a8b;
 const format_tag_t ABcd8a4b = dnnl_ABcd8a4b;
 const format_tag_t aBcd8b = dnnl_aBcd8b;
 const format_tag_t ABcd8b16a2b = dnnl_ABcd8b16a2b;
+const format_tag_t ABcd8b32a2b = dnnl_ABcd8b32a2b;
+const format_tag_t ABcd8b64a2b = dnnl_ABcd8b64a2b;
 const format_tag_t ABcd2b8a4b = dnnl_ABcd2b8a4b;
 const format_tag_t aBCd8b16c2b = dnnl_aBCd8b16c2b;
 const format_tag_t aCBd8b16c2b = dnnl_aCBd8b16c2b;
@@ -268,6 +328,8 @@ const format_tag_t ABcde16a16b = dnnl_ABcde16a16b;
 const format_tag_t aBcde16b = dnnl_aBcde16b;
 const format_tag_t aBcde32b = dnnl_aBcde32b;
 const format_tag_t ABcde16b16a = dnnl_ABcde16b16a;
+const format_tag_t ABcde16b32a = dnnl_ABcde16b32a;
+const format_tag_t ABcde16b64a = dnnl_ABcde16b64a;
 const format_tag_t aBCde16b16c = dnnl_aBCde16b16c;
 const format_tag_t aBCde16c16b = dnnl_aBCde16c16b;
 const format_tag_t aBCde2c8b4c = dnnl_aBCde2c8b4c;
@@ -279,15 +341,24 @@ const format_tag_t aBCde4b4c = dnnl_aBCde4b4c;
 const format_tag_t aBCde4c16b4c = dnnl_aBCde4c16b4c;
 const format_tag_t aBCde16c16b4c = dnnl_aBCde16c16b4c;
 const format_tag_t aBCde16c16b2c = dnnl_aBCde16c16b2c;
+const format_tag_t aBCde16b16c2b = dnnl_aBCde16b16c2b;
 const format_tag_t aBCde4c4b = dnnl_aBCde4c4b;
 const format_tag_t Abcde8a = dnnl_Abcde8a;
 const format_tag_t ABcde8a8b = dnnl_ABcde8a8b;
 const format_tag_t ABcde8a4b = dnnl_ABcde8a4b;
 const format_tag_t aBcde8b = dnnl_aBcde8b;
 const format_tag_t ABcde8b16a2b = dnnl_ABcde8b16a2b;
+const format_tag_t ABcde8b32a2b = dnnl_ABcde8b32a2b;
+const format_tag_t ABcde8b64a2b = dnnl_ABcde8b64a2b;
 const format_tag_t ABcde8a16b2a = dnnl_ABcde8a16b2a;
 const format_tag_t BAcde8a16b2a = dnnl_BAcde8a16b2a;
 const format_tag_t ABcde4b16a4b = dnnl_ABcde4b16a4b;
+const format_tag_t ABcde4b32a4b = dnnl_ABcde4b32a4b;
+const format_tag_t ABcde4b64a4b = dnnl_ABcde4b64a4b;
+const format_tag_t ABcde16b16a4b = dnnl_ABcde16b16a4b;
+const format_tag_t ABcde16b32a4b = dnnl_ABcde16b32a4b;
+const format_tag_t ABcde16b48a4b = dnnl_ABcde16b48a4b;
+const format_tag_t ABcde16b64a4b = dnnl_ABcde16b64a4b;
 const format_tag_t ABcde2b8a4b = dnnl_ABcde2b8a4b;
 const format_tag_t aBCde8b16c2b = dnnl_aBCde8b16c2b;
 const format_tag_t aCBde8b16c2b = dnnl_aCBde8b16c2b;
@@ -336,6 +407,7 @@ const format_tag_t aBdec4b = dnnl_aBdec4b;
 const format_tag_t aBdec8b = dnnl_aBdec8b;
 const format_tag_t aBdefc16b = dnnl_aBdefc16b;
 const format_tag_t aBdefC16b2c = dnnl_aBdefC16b2c;
+const format_tag_t aBdefC16b4c = dnnl_aBdefC16b4c;
 const format_tag_t aCBdef16c16b = dnnl_aCBdef16c16b;
 const format_tag_t aCBdef16b16c = dnnl_aCBdef16b16c;
 const format_tag_t aBdefc4b = dnnl_aBdefc4b;
@@ -357,6 +429,7 @@ const format_tag_t Acdb4a = dnnl_Acdb4a;
 const format_tag_t Acdb8a = dnnl_Acdb8a;
 const format_tag_t Acdeb16a = dnnl_Acdeb16a;
 const format_tag_t AcdeB16a2b = dnnl_AcdeB16a2b;
+const format_tag_t AcdeB16a4b = dnnl_AcdeB16a4b;
 const format_tag_t Acdeb4a = dnnl_Acdeb4a;
 const format_tag_t Acdeb8a = dnnl_Acdeb8a;
 const format_tag_t Adcb16a = dnnl_Adcb16a;
@@ -385,6 +458,89 @@ const format_tag_t aBCdef4b8c2b = dnnl_aBCdef4b8c2b;
 const format_tag_t aBCd4c8b2c = dnnl_aBCd4c8b2c;
 const format_tag_t aBCde4c8b2c = dnnl_aBCde4c8b2c;
 const format_tag_t aBCdef4c8b2c = dnnl_aBCdef4c8b2c;
+const format_tag_t AB32a32b8a4b = dnnl_AB32a32b8a4b;
+const format_tag_t AB8a4b = dnnl_AB8a4b;
+const format_tag_t AB32a32b8a2b = dnnl_AB32a32b8a2b;
+const format_tag_t AB8a2b = dnnl_AB8a2b;
+const format_tag_t abDc32d = dnnl_abDc32d;
+const format_tag_t abDC32d4c = dnnl_abDC32d4c;
+const format_tag_t abdEc32e = dnnl_abdEc32e;
+const format_tag_t abdEC32e2c = dnnl_abdEC32e2c;
+const format_tag_t abdEC32e4c = dnnl_abdEC32e4c;
+const format_tag_t aBCdef16c16b4c = dnnl_aBCdef16c16b4c;
+const format_tag_t ABcde16b16a2b = dnnl_ABcde16b16a2b;
+const format_tag_t ABcde16b32a2b = dnnl_ABcde16b32a2b;
+const format_tag_t ABcde16b48a2b = dnnl_ABcde16b48a2b;
+const format_tag_t ABcde16b64a2b = dnnl_ABcde16b64a2b;
+const format_tag_t aBCdef16c16b2c = dnnl_aBCdef16c16b2c;
+const format_tag_t cBa2b = dnnl_cBa2b;
+const format_tag_t cBa4b = dnnl_cBa4b;
+const format_tag_t adcb = dnnl_adcb;
+const format_tag_t adCb2c = dnnl_adCb2c;
+const format_tag_t adCb4c = dnnl_adCb4c;
+const format_tag_t cdBa2b = dnnl_cdBa2b;
+const format_tag_t cdBa4b = dnnl_cdBa4b;
+const format_tag_t adecb = dnnl_adecb;
+const format_tag_t adeCb2c = dnnl_adeCb2c;
+const format_tag_t adeCb4c = dnnl_adeCb4c;
+const format_tag_t cdeBa2b = dnnl_cdeBa2b;
+const format_tag_t cdeBa4b = dnnl_cdeBa4b;
+const format_tag_t adefcb = dnnl_adefcb;
+const format_tag_t adefCb2c = dnnl_adefCb2c;
+const format_tag_t adefCb4c = dnnl_adefCb4c;
+const format_tag_t Acb32a = dnnl_Acb32a;
+const format_tag_t AcB32a2b = dnnl_AcB32a2b;
+const format_tag_t AcB32a4b = dnnl_AcB32a4b;
+const format_tag_t Acb48a = dnnl_Acb48a;
+const format_tag_t AcB48a2b = dnnl_AcB48a2b;
+const format_tag_t AcB48a4b = dnnl_AcB48a4b;
+const format_tag_t Acb64a = dnnl_Acb64a;
+const format_tag_t AcB64a2b = dnnl_AcB64a2b;
+const format_tag_t AcB64a4b = dnnl_AcB64a4b;
+const format_tag_t aBdc32b = dnnl_aBdc32b;
+const format_tag_t aBdC32b2c = dnnl_aBdC32b2c;
+const format_tag_t aBdC32b4c = dnnl_aBdC32b4c;
+const format_tag_t aBdc48b = dnnl_aBdc48b;
+const format_tag_t aBdC48b2c = dnnl_aBdC48b2c;
+const format_tag_t aBdC48b4c = dnnl_aBdC48b4c;
+const format_tag_t aBdc64b = dnnl_aBdc64b;
+const format_tag_t aBdC64b2c = dnnl_aBdC64b2c;
+const format_tag_t aBdC64b4c = dnnl_aBdC64b4c;
+const format_tag_t AcdB32a2b = dnnl_AcdB32a2b;
+const format_tag_t AcdB32a4b = dnnl_AcdB32a4b;
+const format_tag_t Acdb48a = dnnl_Acdb48a;
+const format_tag_t AcdB48a2b = dnnl_AcdB48a2b;
+const format_tag_t AcdB48a4b = dnnl_AcdB48a4b;
+const format_tag_t Acdb64a = dnnl_Acdb64a;
+const format_tag_t AcdB64a2b = dnnl_AcdB64a2b;
+const format_tag_t AcdB64a4b = dnnl_AcdB64a4b;
+const format_tag_t aBdeC32b2c = dnnl_aBdeC32b2c;
+const format_tag_t aBdeC32b4c = dnnl_aBdeC32b4c;
+const format_tag_t aBdec48b = dnnl_aBdec48b;
+const format_tag_t aBdeC48b2c = dnnl_aBdeC48b2c;
+const format_tag_t aBdeC48b4c = dnnl_aBdeC48b4c;
+const format_tag_t aBdec64b = dnnl_aBdec64b;
+const format_tag_t aBdeC64b2c = dnnl_aBdeC64b2c;
+const format_tag_t aBdeC64b4c = dnnl_aBdeC64b4c;
+const format_tag_t Acdeb32a = dnnl_Acdeb32a;
+const format_tag_t AcdeB32a2b = dnnl_AcdeB32a2b;
+const format_tag_t AcdeB32a4b = dnnl_AcdeB32a4b;
+const format_tag_t Acdeb48a = dnnl_Acdeb48a;
+const format_tag_t AcdeB48a2b = dnnl_AcdeB48a2b;
+const format_tag_t AcdeB48a4b = dnnl_AcdeB48a4b;
+const format_tag_t Acdeb64a = dnnl_Acdeb64a;
+const format_tag_t AcdeB64a2b = dnnl_AcdeB64a2b;
+const format_tag_t AcdeB64a4b = dnnl_AcdeB64a4b;
+const format_tag_t aBdefc32b = dnnl_aBdefc32b;
+const format_tag_t aBdefC32b2c = dnnl_aBdefC32b2c;
+const format_tag_t aBdefC32b4c = dnnl_aBdefC32b4c;
+const format_tag_t aBdefc48b = dnnl_aBdefc48b;
+const format_tag_t aBdefC48b2c = dnnl_aBdefC48b2c;
+const format_tag_t aBdefC48b4c = dnnl_aBdefC48b4c;
+const format_tag_t aBdefc64b = dnnl_aBdefc64b;
+const format_tag_t aBdefC64b2c = dnnl_aBdefC64b2c;
+const format_tag_t aBdefC64b4c = dnnl_aBdefC64b4c;
+
 const format_tag_t last = dnnl_format_tag_last;
 
 const format_tag_t x = dnnl_x;
@@ -421,6 +577,9 @@ const format_tag_t dhwigo = dnnl_dhwigo;
 const format_tag_t giohw = dnnl_giohw;
 const format_tag_t goidhw = dnnl_goidhw;
 const format_tag_t giodhw = dnnl_giodhw;
+const format_tag_t gowi = dnnl_gowi;
+const format_tag_t gohwi = dnnl_gohwi;
+const format_tag_t godhwi = dnnl_godhwi;
 const format_tag_t tnc = dnnl_tnc;
 const format_tag_t ntc = dnnl_ntc;
 const format_tag_t ldnc = dnnl_ldnc;
@@ -447,6 +606,23 @@ const format_tag_t NCdhw16n16c = dnnl_NCdhw16n16c;
 const format_tag_t NCw32n32c = dnnl_NCw32n32c;
 const format_tag_t NChw32n32c = dnnl_NChw32n32c;
 const format_tag_t NCdhw32n32c = dnnl_NCdhw32n32c;
+const format_tag_t OI16i16o = dnnl_OI16i16o;
+const format_tag_t OI16i32o = dnnl_OI16i32o;
+const format_tag_t OI16i64o = dnnl_OI16i64o;
+const format_tag_t OI8i16o2i = dnnl_OI8i16o2i;
+const format_tag_t OI8i32o2i = dnnl_OI8i32o2i;
+const format_tag_t OI8i64o2i = dnnl_OI8i64o2i;
+const format_tag_t OI4i16o4i = dnnl_OI4i16o4i;
+const format_tag_t OI4i32o4i = dnnl_OI4i32o4i;
+const format_tag_t OI4i64o4i = dnnl_OI4i64o4i;
+const format_tag_t OI16i16o4i = dnnl_OI16i16o4i;
+const format_tag_t OI16i32o4i = dnnl_OI16i32o4i;
+const format_tag_t OI16i48o4i = dnnl_OI16i48o4i;
+const format_tag_t OI16i64o4i = dnnl_OI16i64o4i;
+const format_tag_t OI16i16o2i = dnnl_OI16i16o2i;
+const format_tag_t OI16i32o2i = dnnl_OI16i32o2i;
+const format_tag_t OI16i48o2i = dnnl_OI16i48o2i;
+const format_tag_t OI16i64o2i = dnnl_OI16i64o2i;
 const format_tag_t IOdhw16i16o = dnnl_IOdhw16i16o;
 const format_tag_t IOhw16i16o = dnnl_IOhw16i16o;
 const format_tag_t Ohwi32o = dnnl_Ohwi32o;
@@ -457,16 +633,29 @@ const format_tag_t IOw16o16i = dnnl_IOw16o16i;
 const format_tag_t IOw16i16o = dnnl_IOw16i16o;
 const format_tag_t gIOw16i16o = dnnl_gIOw16i16o;
 const format_tag_t OIw16i16o = dnnl_OIw16i16o;
+const format_tag_t OIw16i32o = dnnl_OIw16i32o;
+const format_tag_t OIw16i64o = dnnl_OIw16i64o;
 const format_tag_t OIw16o16i = dnnl_OIw16o16i;
 const format_tag_t Oiw16o = dnnl_Oiw16o;
 const format_tag_t OIw4i16o4i = dnnl_OIw4i16o4i;
+const format_tag_t OIw4i32o4i = dnnl_OIw4i32o4i;
+const format_tag_t OIw4i64o4i = dnnl_OIw4i64o4i;
 const format_tag_t OIw2i8o4i = dnnl_OIw2i8o4i;
 const format_tag_t OIw16i16o4i = dnnl_OIw16i16o4i;
+const format_tag_t OIw16i32o4i = dnnl_OIw16i32o4i;
+const format_tag_t OIw16i48o4i = dnnl_OIw16i48o4i;
+const format_tag_t OIw16i64o4i = dnnl_OIw16i64o4i;
 const format_tag_t OIw16i16o2i = dnnl_OIw16i16o2i;
+const format_tag_t OIw16i32o2i = dnnl_OIw16i32o2i;
+const format_tag_t OIw16i48o2i = dnnl_OIw16i48o2i;
+const format_tag_t OIw16i64o2i = dnnl_OIw16i64o2i;
+const format_tag_t OIw16o16i2o = dnnl_OIw16o16i2o;
 const format_tag_t OIw4i4o = dnnl_OIw4i4o;
 const format_tag_t OIw4o4i = dnnl_OIw4o4i;
 const format_tag_t Oiw4o = dnnl_Oiw4o;
 const format_tag_t OIw8i16o2i = dnnl_OIw8i16o2i;
+const format_tag_t OIw8i32o2i = dnnl_OIw8i32o2i;
+const format_tag_t OIw8i64o2i = dnnl_OIw8i64o2i;
 const format_tag_t OIw8i8o = dnnl_OIw8i8o;
 const format_tag_t OIw8o16i2o = dnnl_OIw8o16i2o;
 const format_tag_t IOw8o16i2o = dnnl_IOw8o16i2o;
@@ -485,15 +674,28 @@ const format_tag_t OhwI16o4i = dnnl_OhwI16o4i;
 const format_tag_t Ohwi4o = dnnl_Ohwi4o;
 const format_tag_t Ohwi8o = dnnl_Ohwi8o;
 const format_tag_t OIhw16i16o = dnnl_OIhw16i16o;
+const format_tag_t OIhw16i32o = dnnl_OIhw16i32o;
+const format_tag_t OIhw16i64o = dnnl_OIhw16i64o;
 const format_tag_t OIhw16o16i = dnnl_OIhw16o16i;
 const format_tag_t Oihw16o = dnnl_Oihw16o;
 const format_tag_t OIhw4i16o4i = dnnl_OIhw4i16o4i;
+const format_tag_t OIhw4i32o4i = dnnl_OIhw4i32o4i;
+const format_tag_t OIhw4i64o4i = dnnl_OIhw4i64o4i;
 const format_tag_t OIhw16i16o4i = dnnl_OIhw16i16o4i;
+const format_tag_t OIhw16i32o4i = dnnl_OIhw16i32o4i;
+const format_tag_t OIhw16i48o4i = dnnl_OIhw16i48o4i;
+const format_tag_t OIhw16i64o4i = dnnl_OIhw16i64o4i;
 const format_tag_t OIhw16i16o2i = dnnl_OIhw16i16o2i;
+const format_tag_t OIhw16i32o2i = dnnl_OIhw16i32o2i;
+const format_tag_t OIhw16i48o2i = dnnl_OIhw16i48o2i;
+const format_tag_t OIhw16i64o2i = dnnl_OIhw16i64o2i;
+const format_tag_t OIhw16o16i2o = dnnl_OIhw16o16i2o;
 const format_tag_t OIhw4i4o = dnnl_OIhw4i4o;
 const format_tag_t OIhw4o4i = dnnl_OIhw4o4i;
 const format_tag_t Oihw4o = dnnl_Oihw4o;
 const format_tag_t OIhw8i16o2i = dnnl_OIhw8i16o2i;
+const format_tag_t OIhw8i32o2i = dnnl_OIhw8i32o2i;
+const format_tag_t OIhw8i64o2i = dnnl_OIhw8i64o2i;
 const format_tag_t OIhw2i8o4i = dnnl_OIhw2i8o4i;
 const format_tag_t OIhw8i8o = dnnl_OIhw8i8o;
 const format_tag_t OIhw8o16i2o = dnnl_OIhw8o16i2o;
@@ -503,16 +705,31 @@ const format_tag_t OIhw8o4i = dnnl_OIhw8o4i;
 const format_tag_t Owhi16o = dnnl_Owhi16o;
 const format_tag_t Odhwi16o = dnnl_Odhwi16o;
 const format_tag_t OdhwI16o2i = dnnl_OdhwI16o2i;
+const format_tag_t OdhwI16o4i = dnnl_OdhwI16o4i;
 const format_tag_t Odhwi4o = dnnl_Odhwi4o;
 const format_tag_t Odhwi8o = dnnl_Odhwi8o;
 const format_tag_t OIdhw16i16o = dnnl_OIdhw16i16o;
+const format_tag_t OIdhw16i32o = dnnl_OIdhw16i32o;
+const format_tag_t OIdhw16i64o = dnnl_OIdhw16i64o;
 const format_tag_t OIdhw16o16i = dnnl_OIdhw16o16i;
 const format_tag_t Oidhw16o = dnnl_Oidhw16o;
 const format_tag_t OIdhw4i4o = dnnl_OIdhw4i4o;
 const format_tag_t OIdhw4o4i = dnnl_OIdhw4o4i;
 const format_tag_t Oidhw4o = dnnl_Oidhw4o;
 const format_tag_t OIdhw8i16o2i = dnnl_OIdhw8i16o2i;
+const format_tag_t OIdhw8i32o2i = dnnl_OIdhw8i32o2i;
+const format_tag_t OIdhw8i64o2i = dnnl_OIdhw8i64o2i;
 const format_tag_t OIdhw4i16o4i = dnnl_OIdhw4i16o4i;
+const format_tag_t OIdhw4i32o4i = dnnl_OIdhw4i32o4i;
+const format_tag_t OIdhw4i64o4i = dnnl_OIdhw4i64o4i;
+const format_tag_t OIdhw16i16o4i = dnnl_OIdhw16i16o4i;
+const format_tag_t OIdhw16i32o4i = dnnl_OIdhw16i32o4i;
+const format_tag_t OIdhw16i48o4i = dnnl_OIdhw16i48o4i;
+const format_tag_t OIdhw16i64o4i = dnnl_OIdhw16i64o4i;
+const format_tag_t OIdhw16i16o2i = dnnl_OIdhw16i16o2i;
+const format_tag_t OIdhw16i32o2i = dnnl_OIdhw16i32o2i;
+const format_tag_t OIdhw16i48o2i = dnnl_OIdhw16i48o2i;
+const format_tag_t OIdhw16i64o2i = dnnl_OIdhw16i64o2i;
 const format_tag_t OIdhw2i8o4i = dnnl_OIdhw2i8o4i;
 const format_tag_t OIdhw8o16i2o = dnnl_OIdhw8o16i2o;
 const format_tag_t IOdhw8o16i2o = dnnl_IOdhw8o16i2o;
@@ -530,6 +747,7 @@ const format_tag_t gOIw4i16o4i = dnnl_gOIw4i16o4i;
 const format_tag_t gOIw2i8o4i = dnnl_gOIw2i8o4i;
 const format_tag_t gOIw16i16o4i = dnnl_gOIw16i16o4i;
 const format_tag_t gOIw16i16o2i = dnnl_gOIw16i16o2i;
+const format_tag_t gOIw16o16i2o = dnnl_gOIw16o16i2o;
 const format_tag_t gOIw4i4o = dnnl_gOIw4i4o;
 const format_tag_t gOIw4o4i = dnnl_gOIw4o4i;
 const format_tag_t gOiw4o = dnnl_gOiw4o;
@@ -559,6 +777,7 @@ const format_tag_t gOIhw2i8o4i = dnnl_gOIhw2i8o4i;
 const format_tag_t gOIhw4i16o4i = dnnl_gOIhw4i16o4i;
 const format_tag_t gOIhw16i16o4i = dnnl_gOIhw16i16o4i;
 const format_tag_t gOIhw16i16o2i = dnnl_gOIhw16i16o2i;
+const format_tag_t gOIhw16o16i2o = dnnl_gOIhw16o16i2o;
 const format_tag_t gOIhw4i4o = dnnl_gOIhw4i4o;
 const format_tag_t gOIhw4o4i = dnnl_gOIhw4o4i;
 const format_tag_t gOihw4o = dnnl_gOihw4o;
@@ -588,6 +807,7 @@ const format_tag_t gOwhi16o = dnnl_gOwhi16o;
 const format_tag_t gIOdhw16i16o = dnnl_gIOdhw16i16o;
 const format_tag_t gOdhwi16o = dnnl_gOdhwi16o;
 const format_tag_t gOdhwI16o2i = dnnl_gOdhwI16o2i;
+const format_tag_t gOdhwI16o4i = dnnl_gOdhwI16o4i;
 const format_tag_t gOdhwi4o = dnnl_gOdhwi4o;
 const format_tag_t gOdhwi8o = dnnl_gOdhwi8o;
 const format_tag_t gOIdhw16i16o = dnnl_gOIdhw16i16o;
@@ -598,7 +818,9 @@ const format_tag_t gOIdhw4o4i = dnnl_gOIdhw4o4i;
 const format_tag_t gOidhw4o = dnnl_gOidhw4o;
 const format_tag_t gOIdhw8i16o2i = dnnl_gOIdhw8i16o2i;
 const format_tag_t gOIdhw4i16o4i = dnnl_gOIdhw4i16o4i;
+const format_tag_t gOIdhw16i16o4i = dnnl_gOIdhw16i16o4i;
 const format_tag_t gOIdhw2i8o4i = dnnl_gOIdhw2i8o4i;
+const format_tag_t gOIdhw16i16o2i = dnnl_gOIdhw16i16o2i;
 const format_tag_t gOIdhw8o16i2o = dnnl_gOIdhw8o16i2o;
 const format_tag_t gIOdhw8o16i2o = dnnl_gIOdhw8o16i2o;
 const format_tag_t gOIdhw8i8o = dnnl_gOIdhw8i8o;
@@ -619,6 +841,79 @@ const format_tag_t gOIdhw4i8o2i = dnnl_gOIdhw4i8o2i;
 const format_tag_t gOIw4o8i2o = dnnl_gOIw4o8i2o;
 const format_tag_t gOIhw4o8i2o = dnnl_gOIhw4o8i2o;
 const format_tag_t gOIdhw4o8i2o = dnnl_gOIdhw4o8i2o;
+const format_tag_t ldOi32o = dnnl_ldOi32o;
+const format_tag_t ldOI32o4i = dnnl_ldOI32o4i;
+const format_tag_t ldgOi32o = dnnl_ldgOi32o;
+const format_tag_t ldgOI32o2i = dnnl_ldgOI32o2i;
+const format_tag_t ldgOI32o4i = dnnl_ldgOI32o4i;
+
+const format_tag_t wIo2i = dnnl_wIo2i;
+const format_tag_t wIo4i = dnnl_wIo4i;
+const format_tag_t gwio = dnnl_gwio;
+const format_tag_t gwIo2i = dnnl_gwIo2i;
+const format_tag_t gwIo4i = dnnl_gwIo4i;
+const format_tag_t hwIo2i = dnnl_hwIo2i;
+const format_tag_t hwIo4i = dnnl_hwIo4i;
+const format_tag_t ghwio = dnnl_ghwio;
+const format_tag_t ghwIo2i = dnnl_ghwIo2i;
+const format_tag_t ghwIo4i = dnnl_ghwIo4i;
+const format_tag_t dhwIo2i = dnnl_dhwIo2i;
+const format_tag_t dhwIo4i = dnnl_dhwIo4i;
+const format_tag_t gdhwio = dnnl_gdhwio;
+const format_tag_t gdhwIo2i = dnnl_gdhwIo2i;
+const format_tag_t gdhwIo4i = dnnl_gdhwIo4i;
+const format_tag_t Owi32o = dnnl_Owi32o;
+const format_tag_t OwI32o2i = dnnl_OwI32o2i;
+const format_tag_t OwI32o4i = dnnl_OwI32o4i;
+const format_tag_t Owi48o = dnnl_Owi48o;
+const format_tag_t OwI48o2i = dnnl_OwI48o2i;
+const format_tag_t OwI48o4i = dnnl_OwI48o4i;
+const format_tag_t Owi64o = dnnl_Owi64o;
+const format_tag_t OwI64o2i = dnnl_OwI64o2i;
+const format_tag_t OwI64o4i = dnnl_OwI64o4i;
+const format_tag_t OhwI32o2i = dnnl_OhwI32o2i;
+const format_tag_t OhwI32o4i = dnnl_OhwI32o4i;
+const format_tag_t Ohwi48o = dnnl_Ohwi48o;
+const format_tag_t OhwI48o2i = dnnl_OhwI48o2i;
+const format_tag_t OhwI48o4i = dnnl_OhwI48o4i;
+const format_tag_t Ohwi64o = dnnl_Ohwi64o;
+const format_tag_t OhwI64o2i = dnnl_OhwI64o2i;
+const format_tag_t OhwI64o4i = dnnl_OhwI64o4i;
+const format_tag_t Odhwi32o = dnnl_Odhwi32o;
+const format_tag_t OdhwI32o2i = dnnl_OdhwI32o2i;
+const format_tag_t OdhwI32o4i = dnnl_OdhwI32o4i;
+const format_tag_t Odhwi48o = dnnl_Odhwi48o;
+const format_tag_t OdhwI48o2i = dnnl_OdhwI48o2i;
+const format_tag_t OdhwI48o4i = dnnl_OdhwI48o4i;
+const format_tag_t Odhwi64o = dnnl_Odhwi64o;
+const format_tag_t OdhwI64o2i = dnnl_OdhwI64o2i;
+const format_tag_t OdhwI64o4i = dnnl_OdhwI64o4i;
+const format_tag_t gOwi32o = dnnl_gOwi32o;
+const format_tag_t gOwI32o2i = dnnl_gOwI32o2i;
+const format_tag_t gOwI32o4i = dnnl_gOwI32o4i;
+const format_tag_t gOwi48o = dnnl_gOwi48o;
+const format_tag_t gOwI48o2i = dnnl_gOwI48o2i;
+const format_tag_t gOwI48o4i = dnnl_gOwI48o4i;
+const format_tag_t gOwi64o = dnnl_gOwi64o;
+const format_tag_t gOwI64o2i = dnnl_gOwI64o2i;
+const format_tag_t gOwI64o4i = dnnl_gOwI64o4i;
+const format_tag_t gOhwI32o2i = dnnl_gOhwI32o2i;
+const format_tag_t gOhwI32o4i = dnnl_gOhwI32o4i;
+const format_tag_t gOhwi48o = dnnl_gOhwi48o;
+const format_tag_t gOhwI48o2i = dnnl_gOhwI48o2i;
+const format_tag_t gOhwI48o4i = dnnl_gOhwI48o4i;
+const format_tag_t gOhwi64o = dnnl_gOhwi64o;
+const format_tag_t gOhwI64o2i = dnnl_gOhwI64o2i;
+const format_tag_t gOhwI64o4i = dnnl_gOhwI64o4i;
+const format_tag_t gOdhwi32o = dnnl_gOdhwi32o;
+const format_tag_t gOdhwI32o2i = dnnl_gOdhwI32o2i;
+const format_tag_t gOdhwI32o4i = dnnl_gOdhwI32o4i;
+const format_tag_t gOdhwi48o = dnnl_gOdhwi48o;
+const format_tag_t gOdhwI48o2i = dnnl_gOdhwI48o2i;
+const format_tag_t gOdhwI48o4i = dnnl_gOdhwI48o4i;
+const format_tag_t gOdhwi64o = dnnl_gOdhwi64o;
+const format_tag_t gOdhwI64o2i = dnnl_gOdhwI64o2i;
+const format_tag_t gOdhwI64o4i = dnnl_gOdhwI64o4i;
 } // namespace format_tag
 
 using memory_extra_flags_t = dnnl_memory_extra_flags_t;
@@ -627,8 +922,8 @@ const memory_extra_flags_t none = dnnl_memory_extra_flag_none;
 const memory_extra_flags_t compensation_conv_s8s8
         = dnnl_memory_extra_flag_compensation_conv_s8s8;
 const memory_extra_flags_t scale_adjust = dnnl_memory_extra_flag_scale_adjust;
-const memory_extra_flags_t gpu_rnn_u8s8_compensation
-        = dnnl_memory_extra_flag_gpu_rnn_u8s8_compensation;
+const memory_extra_flags_t rnn_u8s8_compensation
+        = dnnl_memory_extra_flag_rnn_u8s8_compensation;
 const memory_extra_flags_t compensation_conv_asymmetric_src
         = dnnl_memory_extra_flag_compensation_conv_asymmetric_src;
 } // namespace memory_extra_flags
@@ -673,6 +968,7 @@ const primitive_kind_t eltwise = dnnl_eltwise;
 const primitive_kind_t softmax = dnnl_softmax;
 const primitive_kind_t pooling = dnnl_pooling;
 const primitive_kind_t pooling_v2 = dnnl_pooling_v2;
+const primitive_kind_t prelu = dnnl_prelu;
 const primitive_kind_t lrn = dnnl_lrn;
 const primitive_kind_t batch_normalization = dnnl_batch_normalization;
 const primitive_kind_t layer_normalization = dnnl_layer_normalization;
@@ -721,6 +1017,7 @@ const query_t eltwise_d = dnnl_query_eltwise_d;
 const query_t softmax_d = dnnl_query_softmax_d;
 const query_t pooling_d = dnnl_query_pooling_d;
 const query_t pooling_v2_d = dnnl_query_pooling_v2_d;
+const query_t prelu_d = dnnl_query_prelu_d;
 const query_t lrn_d = dnnl_query_lrn_d;
 const query_t batch_normalization_d = dnnl_query_batch_normalization_d;
 const query_t layer_normalization_d = dnnl_query_layer_normalization_d;
@@ -760,6 +1057,7 @@ using deconvolution_desc_t = dnnl_deconvolution_desc_t;
 using shuffle_desc_t = dnnl_shuffle_desc_t;
 using pooling_desc_t = dnnl_pooling_desc_t;
 using pooling_v2_desc_t = dnnl_pooling_v2_desc_t;
+using prelu_desc_t = dnnl_prelu_desc_t;
 using eltwise_desc_t = dnnl_eltwise_desc_t;
 using softmax_desc_t = dnnl_softmax_desc_t;
 using lrn_desc_t = dnnl_lrn_desc_t;
@@ -794,8 +1092,8 @@ struct op_desc_t {
         convolution_desc_t convolution;
         deconvolution_desc_t deconvolution;
         shuffle_desc_t shuffle;
-        pooling_desc_t pooling;
         pooling_v2_desc_t pooling_v2;
+        prelu_desc_t prelu;
         eltwise_desc_t eltwise;
         softmax_desc_t softmax;
         lrn_desc_t lrn;
@@ -825,8 +1123,8 @@ struct op_desc_t {
 
     DECL_CTOR_AND_CONVERTERS(convolution_desc_t);
     DECL_CTOR_AND_CONVERTERS(shuffle_desc_t);
-    DECL_CTOR_AND_CONVERTERS(pooling_desc_t);
     DECL_CTOR_AND_CONVERTERS(pooling_v2_desc_t);
+    DECL_CTOR_AND_CONVERTERS(prelu_desc_t);
     DECL_CTOR_AND_CONVERTERS(eltwise_desc_t);
     DECL_CTOR_AND_CONVERTERS(softmax_desc_t);
     DECL_CTOR_AND_CONVERTERS(lrn_desc_t);
@@ -901,6 +1199,7 @@ struct matmul_pd_t;
 struct pooling_bwd_pd_t;
 struct pooling_fwd_pd_t;
 struct pooling_pd_t;
+struct prelu_pd_t;
 struct reduction_pd_t;
 struct reorder_pd_t;
 struct resampling_pd_t;
