@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020 Intel Corporation
+* Copyright 2020-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -16,7 +16,6 @@
 
 #include "common/memory_storage.hpp"
 #include "common/memory.hpp"
-#include "common/stream.hpp"
 
 #include <assert.h>
 
@@ -35,21 +34,5 @@ status_t memory_storage_t::init(unsigned flags, size_t size, void *handle) {
     return set_data_handle(handle);
 }
 
-status_t memory_storage_t::map_data(
-        void **mapped_ptr, stream_t *stream, size_t size) const {
-    UNUSED(size);
-    if (stream != nullptr && stream->engine() != engine_)
-        return status::invalid_arguments;
-    return get_data_handle(mapped_ptr);
-}
-
-status_t memory_storage_t::unmap_data(
-        void *mapped_ptr, stream_t *stream) const {
-    UNUSED(mapped_ptr);
-    if (stream != nullptr && stream->engine() != engine_)
-        return status::invalid_arguments;
-    return status::success;
-}
-
 } // namespace impl
 } // namespace dnnl
