@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2020 Intel Corporation
+* Copyright 2018-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -159,6 +159,11 @@ enum {
     key_bnorm_tmp_diff_ss,
     key_bnorm_tmp_stats,
     key_bnorm_reduction,
+    key_brgemm_primitive_batch,
+    key_brgemm_primitive_buffer,
+    key_brgemm_primitive_buffer_a,
+    key_brgemm_primitive_buffer_b,
+    key_brgemm_primitive_buffer_comp,
     key_concat_iptrs,
     key_concat_istrides,
     key_concat_nelems,
@@ -172,9 +177,23 @@ enum {
     key_conv_amx_wsp_buffer,
     key_conv_bia_reduction,
     key_conv_bias_bf16_convert_wsp,
+    key_conv_cudnn,
+    key_conv_cudnn_algo,
+    key_conv_cudnn_filter,
+    key_conv_cudnn_temp,
+    key_conv_dst_bf16_convert_wsp,
+    key_conv_brgemm_addr_a,
+    key_conv_brgemm_addr_b,
+    key_conv_brgemm_batch,
+    key_conv_brgemm_buffer,
+    key_conv_brgemm_inp_buffer,
+    key_conv_brgemm_inp_buffer_mask,
+    key_conv_bwd_w_1st_bia_reorder,
+    key_conv_bwd_w_1st_wei_reorder,
     key_conv_gemm_acc,
     key_conv_gemm_col,
     key_conv_gemm_imtr,
+    key_conv_gemm_zp_src_comp,
     key_conv_int_dat_in_acc_dt,
     key_conv_padded_bias,
     key_conv_rtus_space,
@@ -187,8 +206,11 @@ enum {
     key_conv_wei_reduction,
     key_conv_wei_bia_reduction,
     key_conv_wei_bia_reduction_bctx,
+    key_conv_zero_point_flag,
+    key_conv_zero_point_pad,
     key_deconv_bias,
     key_deconv_sum,
+    key_deconv_zp,
     key_eltwise_diff_dst,
     key_eltwise_src,
     key_fusion_forward_scratchpad,
@@ -200,6 +222,7 @@ enum {
     key_iprod_dst_bf16_convert_wsp,
     key_iprod_dst_reorder,
     key_iprod_int_dat_in_acc_dt,
+    key_lnorm_inv_sqrtvar,
     key_lnorm_tmp_mean,
     key_lnorm_tmp_var,
     key_lnorm_tmp_diff_ss,
@@ -210,8 +233,10 @@ enum {
     key_pool_ind_plain2blocked_cvt,
     key_pool_src_bf16cvt,
     key_pool_src_plain2blocked_cvt,
+    key_prelu_reduction,
     key_reducer_space,
     key_reducer_space_bctx,
+    key_reduction,
     key_reorder_cross_space,
     key_reorder_space,
     key_reorder_scales,
