@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2020 Intel Corporation
+* Copyright 2016-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -17,7 +17,9 @@
 #ifndef COMMON_UTILS_HPP
 #define COMMON_UTILS_HPP
 
+#include <atomic>
 #include <cassert>
+#include <cmath>
 #include <cstddef>
 #include <cstdint>
 #include <cstdio>
@@ -238,6 +240,18 @@ inline R array_product(const T *arr, size_t size) {
     return prod;
 }
 
+template <typename T, typename R = T>
+inline R array_min(const T *arr, size_t size) {
+    R min = std::numeric_limits<R>::max();
+    for (size_t i = 0; i < size; ++i)
+        min = std::min(min, arr[i]);
+    return min;
+}
+
+inline bool equal_with_nan(float v1, float v2) {
+    return (v1 == v2) || (std::isnan(v1) && std::isnan(v2));
+}
+
 /* Sorts an array of @p vals using @p comparator. Uses @p vals_2nd_level as a
  * second level comparing criteria in case comparator returns 0 (equal values)
  * for @p vals elements.
@@ -569,6 +583,19 @@ public:
     DNNL_DISALLOW_COPY_AND_ASSIGN(setting_t);
 };
 
+// The following code is derived from Boost C++ library
+// Copyright 2005-2014 Daniel James.
+// Distributed under the Boost Software License, Version 1.0. (See accompanying
+// file LICENSE or copy at http://www.boost.org/LICENSE_1_0.txt)
+template <typename T>
+static size_t hash_combine(size_t seed, const T &v) {
+    return seed ^= std::hash<T> {}(v) + 0x9e3779b9 + (seed << 6) + (seed >> 2);
+}
+
+inline int float2int(float x) {
+    return utils::bit_cast<int>(x);
+}
+
 // XXX: Currently SYCL doesn't provide an API to get device UUID but
 // we need to be able to distinguish OpenCL device from Level0 device.
 // As a temporary solution the compound ID will be used for that.
@@ -579,11 +606,65 @@ public:
 // -------------------------------------------------------------
 //  Host         | <backend_t::host, 0, 0>
 //  OpenCL       | <backend_t::opencl, cl_device, 0>
+//  NVIDIA       | <backend_t::nvidia, cuDevice, 0>
 //  Level0       | <backend_t::level0, uuid[0-63], uuid[64-127]>
 //  Pure CPU     | <0, 0, 0>
 //  Pure GPU     | <0, cl_device, 0>
 using device_id_t = std::tuple<int, uint64_t, uint64_t>;
 
+struct device_id_hash_t {
+    size_t operator()(const device_id_t &id) const {
+        size_t result = 0;
+        result = hash_combine(result, std::get<0>(id));
+        result = hash_combine(result, std::get<1>(id));
+        result = hash_combine(result, std::get<2>(id));
+        return result;
+    }
+};
+
+// A setting (basically a value) that can be set() multiple times until the
+// time first time the get() method is called. The set() method is expected to
+// be as expensive as a busy-waiting spinlock. The get() method is expected to
+// be asymptotically as expensive as a single lock-prefixed memory read. The
+// get() method also has a 'soft' mode when the setting is not locked for
+// re-setting. This is used for testing purposes.
+template <typename T>
+struct set_before_first_get_setting_t {
+private:
+    T value_;
+    std::atomic<unsigned> state_;
+    enum : unsigned { idle = 0, busy_setting = 1, locked_after_a_get = 2 };
+
+public:
+    set_before_first_get_setting_t(T init) : value_ {init}, state_ {idle} {}
+
+    bool set(T new_value) {
+        if (state_.load() == locked_after_a_get) return false;
+
+        while (true) {
+            unsigned expected = idle;
+            if (state_.compare_exchange_weak(expected, busy_setting)) break;
+            if (expected == locked_after_a_get) return false;
+        }
+
+        value_ = new_value;
+        state_.store(idle);
+        return true;
+    }
+
+    T get(bool soft = false) {
+        if (!soft && state_.load() != locked_after_a_get) {
+            while (true) {
+                unsigned expected = idle;
+                if (state_.compare_exchange_weak(expected, locked_after_a_get))
+                    break;
+                if (expected == locked_after_a_get) break;
+            }
+        }
+        return value_;
+    }
+};
+
 } // namespace impl
 } // namespace dnnl
 
