@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2020 Intel Corporation
+* Copyright 2018-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -41,8 +41,10 @@
             gemm_acc_t *diff_dst_layer_, gemm_acc_t *diff_dst_iter_, \
             gemm_acc_t *diff_dst_iter_c_, const float *weights_peephole_, \
             float *bias_, gates_t *ws_grid_, scratch_t *scratch_cell_, \
-            dst_iter_t *dst_iter_) const
+            dst_iter_t *dst_iter_, float *weights_scales_, int block_step) \
+            const
 
+#if DNNL_X64
 #define rnn_cell_execution_sig(f) \
     dnnl_status_t f(const rnn_utils::rnn_conf_t &rnn, \
             rnn_utils::cell_position_t cell_position, dst_layer_t *dst_layer_, \
@@ -50,33 +52,75 @@
             gemm_acc_t *diff_src_iter_, gemm_acc_t *diff_src_iter_c_, \
             weights_t **w_layer_, weights_t **w_iter_, \
             weights_t **w_projection_, const float *weights_peephole_, \
-            float **bias_, const src_layer_t *src_layer_, \
-            const src_iter_t *src_iter_, const float *src_iter_c_, \
-            gemm_acc_t *diff_dst_layer_, gemm_acc_t *diff_dst_iter_, \
-            gemm_acc_t *diff_dst_iter_c_, gemm_acc_t *diff_w_layer_, \
-            gemm_acc_t *diff_w_iter_, float *diff_weights_projection_, \
-            float *diff_weights_peephole_, float *diff_bias_, \
-            gates_t *ws_gates_, scratch_t *scratch_gates_, ht_t *proj_ht_, \
-            gemm_acc_t *scratch_diff_ht_, gates_t *ws_grid_, \
-            scratch_t *scratch_cell_, dst_iter_t *dst_iter_) const
+            const float *w_proj_comp, float **bias_, \
+            const src_layer_t *src_layer_, const src_iter_t *src_iter_, \
+            const float *src_iter_c_, gemm_acc_t *diff_dst_layer_, \
+            gemm_acc_t *diff_dst_iter_, gemm_acc_t *diff_dst_iter_c_, \
+            gemm_acc_t *diff_w_layer_, gemm_acc_t *diff_w_iter_, \
+            float *diff_weights_projection_, float *diff_weights_peephole_, \
+            float *diff_bias_, gates_t *ws_gates_, scratch_t *scratch_gates_, \
+            ht_t *proj_ht_, gemm_acc_t *scratch_diff_ht_, gates_t *ws_grid_, \
+            scratch_t *scratch_cell_, dst_iter_t *dst_iter_, \
+            gemm_acc_t *amx_scratchpad, \
+            x64::brgemm_batch_element_t *addr_batch_global) const
 
 #define rnn_grid_execution_sig(f) \
     dnnl_status_t f(const rnn_utils::rnn_conf_t &rnn, \
             weights_t **weights_layer_, weights_t **weights_iter_, \
             weights_t **weights_projection_, const float *weights_peephole_, \
-            float **bias_, const src_layer_t *src_layer_, \
-            const src_iter_t *src_iter_, const float *src_iter_c_, \
-            dst_layer_t *dst_layer_, dst_iter_t *dst_iter_, \
-            float *dst_iter_c_, src_layer_t *ws_states_layer_, \
-            src_iter_t *ws_states_iter_, float *ws_states_iter_c_, \
-            gemm_acc_t *ws_diff_states_layer_, \
+            const float *w_proj_comp, float **bias_, \
+            const src_layer_t *src_layer_, const src_iter_t *src_iter_, \
+            const float *src_iter_c_, dst_layer_t *dst_layer_, \
+            dst_iter_t *dst_iter_, float *dst_iter_c_, \
+            src_layer_t *ws_states_layer_, src_iter_t *ws_states_iter_, \
+            float *ws_states_iter_c_, gemm_acc_t *ws_diff_states_layer_, \
             gemm_acc_t *ws_diff_states_iter_, \
             gemm_acc_t *ws_diff_states_iter_c_, gates_t *ws_gates_, \
             ht_t *ws_ht_, gates_t *ws_grid_, scratch_t *scratch_gates_, \
             ht_t *scratch_ht_, gemm_acc_t *scratch_diff_ht_, \
             scratch_t *scratch_cell_, gemm_acc_t *diff_weights_layer_, \
             gemm_acc_t *diff_weights_iter_, float *diff_weights_projection_, \
-            float *diff_weights_peephole_, float *diff_bias_) const
+            float *diff_weights_peephole_, float *diff_bias_, \
+            gemm_acc_t *amx_scratchpad, \
+            x64::brgemm_batch_element_t *addr_batch_global) const
+#else
+#define rnn_cell_execution_sig(f) \
+    dnnl_status_t f(const rnn_utils::rnn_conf_t &rnn, \
+            rnn_utils::cell_position_t cell_position, dst_layer_t *dst_layer_, \
+            float *dst_iter_c_, gemm_acc_t *diff_src_layer_, \
+            gemm_acc_t *diff_src_iter_, gemm_acc_t *diff_src_iter_c_, \
+            weights_t **w_layer_, weights_t **w_iter_, \
+            weights_t **w_projection_, const float *weights_peephole_, \
+            const float *w_proj_comp, float **bias_, \
+            const src_layer_t *src_layer_, const src_iter_t *src_iter_, \
+            const float *src_iter_c_, gemm_acc_t *diff_dst_layer_, \
+            gemm_acc_t *diff_dst_iter_, gemm_acc_t *diff_dst_iter_c_, \
+            gemm_acc_t *diff_w_layer_, gemm_acc_t *diff_w_iter_, \
+            float *diff_weights_projection_, float *diff_weights_peephole_, \
+            float *diff_bias_, gates_t *ws_gates_, scratch_t *scratch_gates_, \
+            ht_t *proj_ht_, gemm_acc_t *scratch_diff_ht_, gates_t *ws_grid_, \
+            scratch_t *scratch_cell_, dst_iter_t *dst_iter_, \
+            gemm_acc_t *amx_scratchpad) const
+
+#define rnn_grid_execution_sig(f) \
+    dnnl_status_t f(const rnn_utils::rnn_conf_t &rnn, \
+            weights_t **weights_layer_, weights_t **weights_iter_, \
+            weights_t **weights_projection_, const float *weights_peephole_, \
+            const float *w_proj_comp, float **bias_, \
+            const src_layer_t *src_layer_, const src_iter_t *src_iter_, \
+            const float *src_iter_c_, dst_layer_t *dst_layer_, \
+            dst_iter_t *dst_iter_, float *dst_iter_c_, \
+            src_layer_t *ws_states_layer_, src_iter_t *ws_states_iter_, \
+            float *ws_states_iter_c_, gemm_acc_t *ws_diff_states_layer_, \
+            gemm_acc_t *ws_diff_states_iter_, \
+            gemm_acc_t *ws_diff_states_iter_c_, gates_t *ws_gates_, \
+            ht_t *ws_ht_, gates_t *ws_grid_, scratch_t *scratch_gates_, \
+            ht_t *scratch_ht_, gemm_acc_t *scratch_diff_ht_, \
+            scratch_t *scratch_cell_, gemm_acc_t *diff_weights_layer_, \
+            gemm_acc_t *diff_weights_iter_, float *diff_weights_projection_, \
+            float *diff_weights_peephole_, float *diff_bias_, \
+            gemm_acc_t *amx_scratchpad) const
+#endif
 
 #define rnn_gemm_sig(f) \
     dnnl_status_t f(const char transA, const char transB, dim_t m, dim_t n, \
@@ -236,6 +280,22 @@ struct rnn_conf_t {
         return utils::one_of(
                 dt_conf, u8u8u8f32, f32u8f32f32, u8u8u8u8, f32u8f32u8);
     }
+    inline bool is_int8_amx() const {
+#if DNNL_X64
+        return brgemm_isa == x64::avx512_core_bf16_amx_int8 && is_int8();
+#else
+        return false;
+#endif
+    }
+    inline bool is_bf16() const { return dt_conf == all_bf16; }
+    inline bool is_bf16_amx() const {
+#if DNNL_X64
+        return brgemm_isa == x64::avx512_core_bf16_amx_bf16 && is_bf16();
+#else
+        return false;
+#endif
+    }
+    inline bool is_f32() const { return dt_conf == all_f32; }
 
     inline bool skip_src_layer_copy() const {
         // Note: this currently always returns true
@@ -249,14 +309,12 @@ struct rnn_conf_t {
                         dt_conf, u8u8u8u8, u8u8u8f32, all_f32, all_bf16);
     }
     inline bool skip_dst_layer_copy() const {
-        // TODO: enable skip copy with lstm_projection
-        return (exec_dir == l2r) && !is_lstm_projection
+        return (exec_dir == l2r)
                 && utils::one_of(
                         dt_conf, u8u8u8u8, f32u8f32u8, all_f32, all_bf16);
     }
     inline bool skip_dst_iter_copy() const {
-        // TODO: enable skip copy with lstm_projection
-        return (exec_dir == l2r) && (dst_iter_ld_ > 0) && !is_lstm_projection
+        return (exec_dir == l2r) && (dst_iter_ld_ > 0)
                 && utils::one_of(
                         dt_conf, u8u8u8u8, u8u8u8f32, all_f32, all_bf16);
     }
@@ -278,6 +336,21 @@ struct rnn_conf_t {
                                 : ws_states_iter_ld);
     }
 
+    inline dim_t layer_brgemm_desc(cell_position_t cell_position) const {
+        return ((cell_position & first_layer) && skip_src_layer_copy())
+                ? 0
+                : ((cell_position & last_iter) && skip_dst_iter_copy()) ? 1 : 2;
+    }
+
+    inline dim_t iter_brgemm_desc(cell_position_t cell_position) const {
+        return ((cell_position & first_iter) && skip_src_iter_copy())
+                ? 0
+                : ((cell_position & last_layer) && skip_dst_layer_copy()
+                          && !(cell_position & first_iter))
+                        ? 1
+                        : 2;
+    }
+
     inline dim_t src_iter_c_ld(cell_position_t cell_position) const {
         return (cell_position & c_state_first_iter) ? src_iter_c_ld_
                                                     : ws_states_iter_c_ld;
@@ -295,6 +368,16 @@ struct rnn_conf_t {
                         : ws_states_layer_ld;
     }
 
+    inline dim_t dst_brgemm_desc(
+            cell_position_t cell_position, bool after_proj = false) const {
+        // We use scratch_ht and not dst_layer for lstmp
+        if (is_lstm_projection && !after_proj) return 0;
+
+        return (cell_position & last_layer) && skip_dst_layer_copy()
+                ? 1
+                : (cell_position & last_iter) && skip_dst_iter_copy() ? 2 : 3;
+    }
+
     inline dim_t dst_iter_ld(cell_position_t cell_position) const {
         return (cell_position & last_iter) && skip_dst_iter_copy()
                 ? dst_iter_ld_
@@ -326,12 +409,41 @@ struct rnn_conf_t {
                 skip_dst_iter_copy() && (cell_position & last_iter)
                         && !(cell_position & first_layer));
     }
+    bool is_brgemm;
+
+    dim_t M, N, K1, K2;
+
+    dim_t LDB1, LDB2;
+    dim_t LDA1[3];
+    dim_t LDA2[3];
+    dim_t LDC;
+
+    dim_t m_block, M_blocks;
+    dim_t n_block, N_blocks, n_tail;
+
+    dim_t k2_block, k1_block, k1_tail, k2_tail;
+    dim_t KB1_blocks, KB2_blocks;
+    dim_t K1padded, K2padded;
+
+    dim_t Kproj, Kprojpadded;
+    dim_t kproj_block, KBproj_blocks, kproj_tail;
+
+    dim_t Nproj, Nproj_blocks, nproj_tail;
+    dim_t LDAproj, LDBproj, LDCproj[4];
+
+    dim_t nthr;
+#if DNNL_X64
+    x64::cpu_isa_t brgemm_isa;
+#endif
+    bool unfused_post_gemm;
 };
 
 bool is_ldigo(const memory_desc_wrapper &md);
 bool is_ldgoi(const memory_desc_wrapper &md);
 bool is_ldio(const memory_desc_wrapper &md);
 bool is_ldoi(const memory_desc_wrapper &md);
+bool is_ldigo_blocked(const memory_desc_wrapper &md);
+bool is_ldio_blocked(const memory_desc_wrapper &md);
 
 int get_good_ld(int dim, int sizeof_dt);
 
@@ -450,9 +562,13 @@ bool init_conf(rnn_conf_t &rnn, const rnn_desc_t &rd,
     rnn.ws_diff_states_iter_c_ld = rnn.dhc;
 
     // set scratch (not)leading dimensions
+    // scratch gates is used to store intermediate gates before postgemm operation
+    // temporary: we also use it in lstmp as temporary scratchpad
+    // between projection and downconversion, hence the max with dlc
     rnn.scratch_gates_nld = rnn.mb;
     rnn.scratch_gates_ld
-            = get_good_ld(rnn.n_gates * rnn.dhc, sizeof(typename T::scratch_t));
+            = get_good_ld(nstl::max(rnn.dlc, rnn.n_gates * rnn.dhc),
+                    sizeof(typename T::scratch_t));
     rnn.scratch_ht_nld = rnn.proj_ht_nld;
     rnn.scratch_ht_ld = rnn.proj_ht_ld;
 
@@ -507,12 +623,16 @@ bool init_conf(rnn_conf_t &rnn, const rnn_desc_t &rd,
     auto dst_layer_is_trivial_stride = dst_layer_d.blocking_desc().strides[0]
             == (rnn.dst_layer_ld_ * rnn.mb);
 
-    rnn.merge_gemm_layer = ((rnn.is_fwd && src_layer_is_trivial_stride)
-                                   || ((rd.prop_kind == prop_kind::backward)
-                                           && dst_layer_is_trivial_stride))
-            && (((rnn.is_fwd && rnn.mb < 128) || !rnn.is_fwd) || rnn.is_int8());
-    rnn.merge_gemm_iter
-            = dst_layer_is_trivial_stride && !(rnn.is_fwd || is_gru);
+    rnn.merge_gemm_layer = (!rnn.is_brgemm)
+            ? ((rnn.is_fwd && src_layer_is_trivial_stride)
+                      || ((rd.prop_kind == prop_kind::backward)
+                              && dst_layer_is_trivial_stride))
+                    && (((rnn.is_fwd && rnn.mb < 128) || !rnn.is_fwd)
+                            || rnn.is_int8())
+            : false;
+    rnn.merge_gemm_iter = (!rnn.is_brgemm)
+            ? dst_layer_is_trivial_stride && !(rnn.is_fwd || is_gru)
+            : false;
     rnn.force_nocopy = false;
 #if DNNL_X64
     rnn.force_nocopy = !x64::mayiuse(x64::avx512_mic) && x64::mayiuse(x64::avx)
@@ -523,32 +643,40 @@ bool init_conf(rnn_conf_t &rnn, const rnn_desc_t &rd,
     /* Decide to copy bias */
     rnn.copy_bias = rnn.is_int8();
 
-    rnn.use_layer_packed_gemm
-            = utils::one_of(weights_layer_d.format_kind(), format_kind::any,
+    rnn.use_layer_packed_gemm = !rnn.is_brgemm
+            ? utils::one_of(weights_layer_d.format_kind(), format_kind::any,
                       format_kind::rnn_packed)
-            && is_inference
-            && ((is_f32 && pack_sgemm_supported() && rnn.n_iter == 1)
-                    || rnn.is_int8() || is_bf16);
-    rnn.use_iter_packed_gemm
-            = utils::one_of(weights_iter_d.format_kind(), format_kind::any,
+                    && is_inference
+                    && ((is_f32 && pack_sgemm_supported() && rnn.n_iter == 1)
+                            || rnn.is_int8() || is_bf16)
+            : false;
+    rnn.use_iter_packed_gemm = !rnn.is_brgemm
+            ? utils::one_of(weights_iter_d.format_kind(), format_kind::any,
                       format_kind::rnn_packed)
-            && is_inference
-            && ((is_f32 && pack_sgemm_supported() && rnn.mb >= 16)
-                    || rnn.is_int8() || is_bf16);
-    rnn.use_projection_packed_gemm = false;
+                    && is_inference
+                    && ((is_f32 && pack_sgemm_supported() && rnn.mb >= 16)
+                            || rnn.is_int8() || is_bf16)
+            : false;
+    rnn.use_projection_packed_gemm = !rnn.is_brgemm
+            ? utils::one_of(weights_projection_d.format_kind(),
+                      format_kind::any, format_kind::rnn_packed)
+                    && is_inference
+                    && ((is_f32 && pack_sgemm_supported() && rnn.n_iter == 1)
+                            || rnn.is_int8() || is_bf16)
+            : false;
 
     /* Set packed gemm sizes */
     /* TODO: investigate the benefit of mixing packed and non-packed weights parts */
     auto set_pack_sizes
             = [&](bool merge, bool &do_pack, size_t &weights_pack_size,
                       int &n_parts, int *parts, size_t *parts_pack_size,
-                      size_t &comp_offset, int feature_size, dim_t weights_oc,
+                      size_t &comp_offset, int ic, int oc, int weights_oc,
                       dim_t data_ld) -> bool {
         bool pack = true;
         weights_pack_size = 0;
         for (int p = 0; p < n_parts; p++) {
-            dim_t m_p = rnn.is_fwd ? (parts[p] * rnn.dhc) : feature_size;
-            dim_t k_p = rnn.is_fwd ? feature_size : (parts[p] * rnn.dhc);
+            dim_t m_p = rnn.is_fwd ? (parts[p] * oc) : ic;
+            dim_t k_p = rnn.is_fwd ? ic : (parts[p] * oc);
             dim_t n_p = merge ? rnn.mb * rnn.n_iter : rnn.mb;
             bool pack_part = true;
 
@@ -594,7 +722,8 @@ bool init_conf(rnn_conf_t &rnn, const rnn_desc_t &rd,
                 rnn.use_layer_packed_gemm, rnn.weights_layer_pack_size,
                 rnn.n_parts_weights_layer, rnn.parts_weights_layer,
                 rnn.part_weights_layer_pack_size, rnn.weights_layer_comp_offset,
-                rnn.slc, rnn.n_gates * rnn.dhc, rnn.ws_states_layer_ld);
+                rnn.slc, rnn.dhc, rnn.n_gates * rnn.dhc,
+                rnn.ws_states_layer_ld);
         if (!ok) return false;
     }
 
@@ -602,8 +731,18 @@ bool init_conf(rnn_conf_t &rnn, const rnn_desc_t &rd,
         bool ok = set_pack_sizes(rnn.merge_gemm_iter, rnn.use_iter_packed_gemm,
                 rnn.weights_iter_pack_size, rnn.n_parts_weights_iter,
                 rnn.parts_weights_iter, rnn.part_weights_iter_pack_size,
-                rnn.weights_iter_comp_offset, rnn.sic, rnn.n_gates * rnn.dhc,
-                rnn.ws_states_iter_ld);
+                rnn.weights_iter_comp_offset, rnn.sic, rnn.dhc,
+                rnn.n_gates * rnn.dhc, rnn.ws_states_iter_ld);
+        if (!ok) return false;
+    }
+
+    if (rnn.use_projection_packed_gemm) {
+        bool ok = set_pack_sizes(false, rnn.use_projection_packed_gemm,
+                rnn.weights_projection_pack_size,
+                rnn.n_parts_weights_projection, rnn.parts_weights_projection,
+                rnn.part_weights_projection_pack_size,
+                rnn.weights_projection_comp_offset, rnn.dhc, rnn.dic, rnn.dic,
+                rnn.scratch_ht_ld);
         if (!ok) return false;
     }
 
