@@ -97,7 +97,7 @@ void reduce_balancer_t::balance() {
 using namespace Xbyak;
 
 template <impl::data_type_t data_type>
-struct reducer_2d_driver_t : public c_compatible {
+struct reducer_2d_driver_t : public jit_generator {
     using data_t = typename prec_traits<data_type>::type;
 
     reducer_2d_driver_t(int n_src, size_t src_ld, size_t src_step,
@@ -107,11 +107,9 @@ struct reducer_2d_driver_t : public c_compatible {
         , src_step_(src_step)
         , dst_step_(dst_step)
         , nullify_dst_(nullify_dst) {}
-    virtual ~reducer_2d_driver_t() = default;
     virtual void operator()(
             data_t *dst, const data_t *srcs, size_t ny, size_t nx)
             = 0;
-    virtual status_t create_kernel() = 0;
 
 protected:
     int n_src_;
@@ -120,8 +118,7 @@ protected:
 };
 
 template <impl::data_type_t data_type, cpu_isa_t isa>
-struct reducer_2d_driver_f_s_32_t : public reducer_2d_driver_t<data_type>,
-                                    public jit_generator {
+struct reducer_2d_driver_f_s_32_t : public reducer_2d_driver_t<data_type> {
     DECLARE_CPU_JIT_AUX_FUNCTIONS(reducer_2d_driver_f_s_32_t)
 
     using data_t = typename prec_traits<data_type>::type;
@@ -131,22 +128,20 @@ struct reducer_2d_driver_f_s_32_t : public reducer_2d_driver_t<data_type>,
         jit_generator::operator()(dst, srcs, ny, nx);
     }
 
-    status_t create_kernel() override { return jit_generator::create_kernel(); }
-
     /* cpu specific part */
     using Vmm = typename utils::conditional<isa == avx2, Ymm, Zmm>::type;
-    const AddressFrame &vmmword = (isa == avx2) ? yword : zword;
+    const AddressFrame &vmmword = (isa == avx2) ? this->yword : this->zword;
     void uni_vadd(const Xmm &x1, const Xmm &x2, const Operand &op) {
         if (data_type == data_type::f32)
-            vaddps(x1, x2, op);
+            this->vaddps(x1, x2, op);
         else
-            vpaddd(x1, x2, op);
+            this->vpaddd(x1, x2, op);
     }
     void uni_add(const Xmm &x1, const Operand &op) {
         if (data_type == data_type::f32)
-            addss(x1, op);
+            this->addss(x1, op);
         else
-            paddd(x1, op);
+            this->paddd(x1, op);
     }
 
     const int vlen = cpu_isa_traits<isa>::vlen;
@@ -157,9 +152,9 @@ struct reducer_2d_driver_f_s_32_t : public reducer_2d_driver_t<data_type>,
     Xbyak::Reg64 reg_ny = abi_param3;
     Xbyak::Reg64 reg_nx = abi_param4;
 
-    Xbyak::Reg64 reg_x = rax;
-    Xbyak::Reg64 reg_src_id = r10;
-    Xbyak::Reg64 reg_long_offt = r11;
+    Xbyak::Reg64 reg_x = this->rax;
+    Xbyak::Reg64 reg_src_id = this->r10;
+    Xbyak::Reg64 reg_long_offt = this->r11;
 
     reducer_2d_driver_f_s_32_t(int n_src, size_t src_ld, size_t src_step,
             size_t dst_step, bool nullify_dst)
@@ -169,16 +164,16 @@ struct reducer_2d_driver_f_s_32_t : public reducer_2d_driver_t<data_type>,
     void nullify_dst(int nloads, int load_len) {
         UNUSED(load_len);
         for (int i = 0; i < nloads; ++i)
-            uni_vpxor(Vmm(i), Vmm(i), Vmm(i));
+            this->uni_vpxor(Vmm(i), Vmm(i), Vmm(i));
         /* prefetches[dst] ? */
     }
 
     void load_dst(int nloads, int load_len) {
         for (int i = 0; i < nloads; ++i) {
             if (load_len == typesize)
-                movd(Xmm(i), ptr[reg_dst + i * load_len]);
+                this->movd(Xmm(i), this->ptr[reg_dst + i * load_len]);
             else if (load_len == vlen)
-                vmovups(Vmm(i), ptr[reg_dst + i * load_len]);
+                this->vmovups(Vmm(i), this->ptr[reg_dst + i * load_len]);
             else
                 assert(!"unsupported");
         }
@@ -187,9 +182,9 @@ struct reducer_2d_driver_f_s_32_t : public reducer_2d_driver_t<data_type>,
     void store_dst(int nloads, int load_len) {
         for (int i = 0; i < nloads; ++i) {
             if (load_len == typesize)
-                movd(ptr[reg_dst + i * load_len], Xmm(i));
+                this->movd(this->ptr[reg_dst + i * load_len], Xmm(i));
             else if (load_len == vlen)
-                vmovups(ptr[reg_dst + i * load_len], Vmm(i));
+                this->vmovups(this->ptr[reg_dst + i * load_len], Vmm(i));
             else
                 assert(!"unsupported");
         }
@@ -200,9 +195,9 @@ struct reducer_2d_driver_f_s_32_t : public reducer_2d_driver_t<data_type>,
             size_t off = base_off + i * load_len;
 
             if (load_len == typesize)
-                uni_add(Xmm(i), ptr[reg_src + off]);
+                this->uni_add(Xmm(i), this->ptr[reg_src + off]);
             else if (load_len == vlen)
-                uni_vadd(Vmm(i), Vmm(i), vmmword[reg_src + off]);
+                this->uni_vadd(Vmm(i), Vmm(i), vmmword[reg_src + off]);
             else
                 assert(!"unsupported");
         }
@@ -215,13 +210,13 @@ struct reducer_2d_driver_f_s_32_t : public reducer_2d_driver_t<data_type>,
         const int load_len[nbranches] = {vlen, vlen, typesize};
         Label loop_x_label[nbranches + 1];
 
-        mov(reg_x, reg_nx);
+        this->mov(reg_x, reg_nx);
 
         for (int id = 0; id < nbranches; ++id) {
-            L(loop_x_label[id]);
+            this->L(loop_x_label[id]);
 
-            cmp(reg_x, nloads[id] * load_len[id]);
-            jl(loop_x_label[id + 1], T_NEAR);
+            this->cmp(reg_x, nloads[id] * load_len[id]);
+            this->jl(loop_x_label[id + 1], this->T_NEAR);
 
             if (this->nullify_dst_)
                 nullify_dst(nloads[id], load_len[id]);
@@ -230,18 +225,18 @@ struct reducer_2d_driver_f_s_32_t : public reducer_2d_driver_t<data_type>,
 
             if (nloads[id] > 1) {
                 Label loop_srcs;
-                mov(reg_src_id, this->n_src_);
-                L(loop_srcs);
+                this->mov(reg_src_id, this->n_src_);
+                this->L(loop_srcs);
 
                 accumulate(nloads[id], load_len[id], 0);
-                add(reg_src, this->src_ld_ * typesize);
+                this->add(reg_src, this->src_ld_ * typesize);
 
-                dec(reg_src_id);
-                jnz(loop_srcs, T_NEAR);
+                this->dec(reg_src_id);
+                this->jnz(loop_srcs, this->T_NEAR);
 
                 size_t base_off
                         = (size_t)this->n_src_ * this->src_ld_ * typesize;
-                safe_sub(reg_src, base_off, reg_long_offt);
+                this->safe_sub(reg_src, base_off, reg_long_offt);
             } else {
                 for (int src_id = 0; src_id < this->n_src_; ++src_id) {
                     const size_t base_off
@@ -252,40 +247,40 @@ struct reducer_2d_driver_f_s_32_t : public reducer_2d_driver_t<data_type>,
 
             store_dst(nloads[id], load_len[id]);
 
-            add(reg_src, nloads[id] * load_len[id]);
-            add(reg_dst, nloads[id] * load_len[id]);
+            this->add(reg_src, nloads[id] * load_len[id]);
+            this->add(reg_dst, nloads[id] * load_len[id]);
 
-            sub(reg_x, nloads[id] * load_len[id]);
+            this->sub(reg_x, nloads[id] * load_len[id]);
 
-            jmp(loop_x_label[id], T_NEAR);
+            this->jmp(loop_x_label[id], this->T_NEAR);
         }
 
-        L(loop_x_label[nbranches]);
+        this->L(loop_x_label[nbranches]);
 
         /* restore address registers */
-        sub(reg_src, reg_nx);
-        sub(reg_dst, reg_nx);
+        this->sub(reg_src, reg_nx);
+        this->sub(reg_dst, reg_nx);
     }
 
     void generate() override {
         assert(isa == avx2 || isa == avx512_common || isa == avx512_mic);
 
-        preamble();
+        this->preamble();
 
-        shl(reg_nx, 2);
+        this->shl(reg_nx, 2);
 
         Label ny_loop;
-        L(ny_loop);
+        this->L(ny_loop);
 
         loop_x();
 
-        add(reg_dst, this->dst_step_ * typesize);
-        add(reg_src, this->src_step_ * typesize);
+        this->add(reg_dst, this->dst_step_ * typesize);
+        this->add(reg_src, this->src_step_ * typesize);
 
-        dec(reg_ny);
-        jnz(ny_loop, T_NEAR);
+        this->dec(reg_ny);
+        this->jnz(ny_loop, this->T_NEAR);
 
-        postamble();
+        this->postamble();
     }
 };
 
