@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020 Intel Corporation
+* Copyright 2020-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -47,7 +47,7 @@ struct xbyak_gemm_smalln_tn_t : public jit_generator {
             size_t code_size = 80 * Xbyak::DEFAULT_MAX_CODE_SIZE)
         : jit_generator(code_ptr, code_size), N(N), beta(beta), alpha(alpha) {}
 
-    void generate() override {
+    void generate() override ATTRIBUTE_OPTIMIZE {
         using namespace Xbyak;
         /**
          * numN = 1 : 16 rows of A, 1x16 accumulators
@@ -800,7 +800,7 @@ dnnl_status_t jit_avx512_core_gemm_smalln_tn_f32(const char *transa,
         const float *p_beta, float *C, const dim_t *p_ldc) {
     using namespace avx512_core_gemm_smalln_tn_f32;
 
-    int max_num_threads = (dnnl_in_parallel()) ? 1 : dnnl_get_max_threads();
+    int max_num_threads = dnnl_get_current_num_threads();
 
     dim_t m = *p_m;
     dim_t n = *p_n;
