@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2020 Intel Corporation
+* Copyright 2019-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -42,8 +42,9 @@ void jit_uni_eltwise_injector_f32<isa>::injector_preamble(
     // For avx we need a register to save the upper part of Ymm
     preserve_vec_for_avx = isa == avx
             && utils::one_of(alg_, eltwise_tanh, eltwise_elu, eltwise_abs,
-                    eltwise_soft_relu, eltwise_logistic, eltwise_exp,
-                    eltwise_gelu_tanh, eltwise_swish, eltwise_gelu_erf,
+                    eltwise_soft_relu, eltwise_logsigmoid, eltwise_mish,
+                    eltwise_logistic, eltwise_exp, eltwise_gelu_tanh,
+                    eltwise_swish, eltwise_gelu_erf,
                     eltwise_tanh_use_dst_for_bwd, eltwise_elu_use_dst_for_bwd,
                     eltwise_logistic_use_dst_for_bwd,
                     eltwise_exp_use_dst_for_bwd);
@@ -223,12 +224,17 @@ void jit_uni_eltwise_injector_f32<isa>::test_mask() {
 template <cpu_isa_t isa>
 void jit_uni_eltwise_injector_f32<isa>::exp_compute_vector_fwd(
         const Vmm &vmm_src) {
+    // exp(x) =
+    // = exp(n * ln(2) + r) // divide x by ln(2) and get quot and rem
+    // = 2^n * exp(r) // simplify the exp(n*ln(2)) expression
+
     // get mask of values lower than log(FLT_MIN) to zero them in the output
     compute_cmp_mask(vmm_src, table_val(exp_ln_flt_min_f), _cmp_lt_os);
 
     h->uni_vminps(vmm_src, vmm_src, table_val(exp_ln_flt_max_f));
     h->uni_vmaxps(vmm_src, vmm_src, table_val(exp_ln_flt_min_f));
     h->uni_vmovups(vmm_aux1, vmm_src);
+
     // calculate exp(x)
     // fx = x * log2ef + 0.5
     h->uni_vmulps(vmm_src, vmm_src, table_val(exp_log2ef));
@@ -243,7 +249,13 @@ void jit_uni_eltwise_injector_f32<isa>::exp_compute_vector_fwd(
     // x = x - fx * ln2
     h->uni_vfnmadd231ps(vmm_aux1, vmm_aux2, table_val(ln2f));
 
-    // compute 2^n
+    // We do not count 2^n here, because n can reach 128 and 2^128 is not
+    // representable by fp32, so to get around this problem, instead of computing
+    // 2^n * exp(r) will be counted 2*2^(n-1)*exp(r), because 2^127
+    // and 2 are numbers representable in fp32.
+
+    // compute 2^(n-1)
+    h->uni_vsubps(vmm_src, vmm_src, table_val(one));
     h->uni_vcvtps2dq(vmm_aux2, vmm_src);
     if (isa != avx)
         h->uni_vpaddd(vmm_aux2, vmm_aux2, table_val(exponent_bias));
@@ -255,7 +267,7 @@ void jit_uni_eltwise_injector_f32<isa>::exp_compute_vector_fwd(
         h->vpaddd(xmm_aux2, xmm_aux2, table_val(exponent_bias));
         h->vinsertf128(ymm_aux2, ymm_aux2, xmm_tmp, 1);
     }
-    vec_shift(vmm_aux2, vmm_aux2, true, n_mantissa_bits); //Vmm(6) = 2^-fx
+    vec_shift(vmm_aux2, vmm_aux2, true /*shift_left*/, n_mantissa_bits);
     // use vmm_src as tmp vmm_zero when applying mask
     h->uni_vxorps(vmm_src, vmm_src, vmm_src);
     // set zeroes at those points which were < log(FLT_MIN)
@@ -270,6 +282,7 @@ void jit_uni_eltwise_injector_f32<isa>::exp_compute_vector_fwd(
     h->uni_vfmadd213ps(vmm_src, vmm_aux1, table_val(one));
     // y = y * 2^n
     h->uni_vmulps(vmm_src, vmm_src, vmm_aux2);
+    h->uni_vmulps(vmm_src, vmm_src, table_val(two));
 }
 
 template <cpu_isa_t isa>
@@ -320,8 +333,6 @@ void jit_uni_eltwise_injector_f32<isa>::tanh_compute_vector_fwd(
         vmm_pol = vmm_aux2, vmm_indices = vmm_aux3, vmm_src_original = vmm_aux4,
         vmm_sign = vmm_aux4;
     Reg64 gpr_idx[XMM_float_lanes_count];
-    // tmp_reg to save the state of regs64 (for avx only).
-    Reg64 tmp_reg;
 
     if (isa == sse41 || isa == avx) {
         assert(aux_gprs_count() >= XMM_float_lanes_count);
@@ -570,15 +581,78 @@ void jit_uni_eltwise_injector_f32<isa>::clip_compute_vector_fwd(
     h->uni_vminps(vmm_src, vmm_src, table_val(beta));
 }
 
+template <cpu_isa_t isa>
+void jit_uni_eltwise_injector_f32<isa>::logsigmoid_compute_vector_fwd(
+        const Vmm &vmm_src) {
+    h->uni_vmulps(vmm_src, vmm_src, table_val(minus_one));
+    soft_relu_compute_vector_fwd(vmm_src);
+    h->uni_vmulps(vmm_src, vmm_src, table_val(minus_one));
+}
+
+template <cpu_isa_t isa>
+void jit_uni_eltwise_injector_f32<isa>::mish_compute_vector_fwd(
+        const Vmm &vmm_src) {
+    // An equation other than mish(x) = x*tanh(srelu(x)) was used
+    // to calculate mish, but it should be remembered that it is equivalent
+    // equation, it uses the following rule:
+    // tanh(x) = (e^x - e^-x) / (e^x + e^-x),
+    // hence the equation for mish can take the form:
+    // mish(x) = x * ((e^x + 1)^2 - 1)/((e^x + 1)^2 + 1).
+    // This option was chosen because computing tanh requires more registers
+    // than exp, and also requires more constants to be stored in memory,
+    // making the algorithm slower.
+
+    // IMPORTANT: we use vmm_aux3 to save src as exp does not use it.
+    h->uni_vmovups(vmm_aux3, vmm_src); // vmm_aux3 = x
+
+    h->uni_vminps(vmm_src, vmm_src, table_val(fwd_mish_max_x_for_equation_f));
+    exp_compute_vector_fwd(vmm_src);
+
+    // (e^x+1)^2
+    h->uni_vaddps(vmm_src, vmm_src, table_val(one));
+    h->uni_vmulps(vmm_src, vmm_src, vmm_src);
+
+    // save (e^x+1)^2 as it appears in both the denominator and the numerator
+    h->uni_vmovups(vmm_aux1, vmm_src);
+
+    // x * ((e^x + 1)^2 - 1) / ((e^x + 1)^2 + 1)
+    h->uni_vsubps(vmm_src, vmm_src, table_val(one));
+    h->uni_vaddps(vmm_aux1, vmm_aux1, table_val(one));
+    h->uni_vdivps(vmm_src, vmm_src, vmm_aux1);
+    h->uni_vmulps(vmm_src, vmm_src, vmm_aux3);
+}
+
+template <cpu_isa_t isa>
+void jit_uni_eltwise_injector_f32<isa>::hardswish_compute_vector_fwd(
+        const Vmm &vmm_src) {
+    // result = x * relu6(x + 3) / 6
+    h->uni_vmovups(vmm_aux0, vmm_src);
+
+    h->uni_vaddps(vmm_aux0, vmm_aux0, table_val(three));
+    h->uni_vmaxps(vmm_aux0, vmm_aux0, table_val(zero));
+    h->uni_vminps(vmm_aux0, vmm_aux0, table_val(six));
+    h->uni_vdivps(vmm_aux0, vmm_aux0, table_val(six));
+    h->uni_vmulps(vmm_src, vmm_src, vmm_aux0);
+}
+
 template <cpu_isa_t isa>
 void jit_uni_eltwise_injector_f32<isa>::soft_relu_compute_vector_fwd(
         const Vmm &vmm_src) {
+    // ln(1 + exp(x)) =
+    // = ln(1 + exp(n * ln(2) + r)) // divide x by ln(2) and get quot and rem
+    // = ln(1 + 2^n * exp(r)) // simplify the exp(n*ln(2)) expression
+    // = ln(2 ^ 0 + 2^n * exp(r)) // note 1 = 2^0
+    // = ln(2 ^ (n - n) + 2^n * exp(r)) // 2^0 = 2^(n-n)
+    // = ln(2 ^ n * (2^-n + exp(r))) // factorize with 2^n
+    // = n * ln(2) + ln(2^-n + exp(r)) // take the 2^n factor out of the ln
+
     // keep src for further computations
     h->uni_vmovups(vmm_aux2, vmm_src);
 
     h->uni_vminps(vmm_src, vmm_src, table_val(exp_ln_flt_max_f));
     h->uni_vmaxps(vmm_src, vmm_src, table_val(exp_ln_flt_min_f));
     h->uni_vmovups(vmm_aux1, vmm_src);
+
     // calculate exp(x)
     // fx = x * log2ef + 0.5
     h->uni_vmulps(vmm_src, vmm_src, table_val(exp_log2ef));
@@ -601,7 +675,14 @@ void jit_uni_eltwise_injector_f32<isa>::soft_relu_compute_vector_fwd(
     h->uni_vfmadd213ps(vmm_aux3, vmm_aux1, table_val(exp_pol, 0));
     h->uni_vfmadd213ps(vmm_aux3, vmm_aux1, table_val(one));
 
-    // compute 2^(-n)
+    // We do not count 2^-n here, because n can reach 128 and 2^(-128) is not
+    // representable by fp32, so to get around this problem, instead of computing
+    // 2^-n + exp(r) will be counted (2^-(n-1) + 2*exp(r))/2, because 2^(-127)
+    // and 2 are numbers representable in fp32.
+
+    // compute 2^-(n-1)
+    // vmm_src now represents n-1
+    h->uni_vsubps(vmm_src, vmm_src, table_val(one));
     if (has_avx512()) {
         h->vmulps(vmm_aux1, vmm_src, table_val(minus_one));
         h->vcvtps2dq(vmm_aux1, vmm_aux1);
@@ -612,6 +693,8 @@ void jit_uni_eltwise_injector_f32<isa>::soft_relu_compute_vector_fwd(
         h->uni_vcvtps2dq(vmm_aux1, vmm_src);
         h->uni_vpsignd(vmm_aux1, vmm_aux1, table_val(minus_one));
     }
+    // restore vmm_src to n
+    h->uni_vaddps(vmm_src, vmm_src, table_val(one));
 
     if (isa != avx)
         h->uni_vpaddd(vmm_aux1, vmm_aux1, table_val(exponent_bias));
@@ -623,11 +706,14 @@ void jit_uni_eltwise_injector_f32<isa>::soft_relu_compute_vector_fwd(
         h->vpaddd(xmm_aux1, xmm_aux1, table_val(exponent_bias));
         h->vinsertf128(ymm_aux1, ymm_aux1, xmm_tmp, 1);
     }
-    vec_shift(vmm_aux1, vmm_aux1, true, n_mantissa_bits); //vmm_aux1 = 2^-fx
+    vec_shift(vmm_aux1, vmm_aux1, true /*shift_left*/, n_mantissa_bits);
     // calculate ln(1 + y)
-    h->uni_vaddps(vmm_aux3, vmm_aux3, vmm_aux1);
+    h->uni_vmulps(vmm_aux3, vmm_aux3, table_val(two)); // 2*exp(r)
+    h->uni_vaddps(vmm_aux3, vmm_aux3, vmm_aux1); // 2^-(n-1) + 2*exp(r)
+    h->uni_vdivps(
+            vmm_aux3, vmm_aux3, table_val(two)); // (2^-(n-1) + 2*exp(r))/2
     // frexp()
-    vec_shift(vmm_src, vmm_aux3, false, n_mantissa_bits);
+    vec_shift(vmm_src, vmm_aux3, false /*shift_left*/, n_mantissa_bits);
     h->uni_vcvtdq2ps(vmm_src, vmm_src);
     // got n. where n is x = 2^n * y. y = 0.5 .. 1
     h->uni_vsubps(vmm_src, vmm_src, table_val(soft_relu_one_twenty_six));
@@ -1179,6 +1265,54 @@ void jit_uni_eltwise_injector_f32<isa>::soft_relu_compute_vector_bwd(
     logistic_compute_vector_fwd(vmm_src);
 }
 
+template <cpu_isa_t isa>
+void jit_uni_eltwise_injector_f32<isa>::logsigmoid_compute_vector_bwd(
+        const Vmm &vmm_src) {
+    h->uni_vmulps(vmm_src, vmm_src, table_val(minus_one));
+    soft_relu_compute_vector_bwd(vmm_src);
+}
+
+template <cpu_isa_t isa>
+void jit_uni_eltwise_injector_f32<isa>::mish_compute_vector_bwd(
+        const Vmm &vmm_src) {
+    // IMPORTANT: we use vmm_aux3 to save src as exp does not use it.
+    h->uni_vmovups(vmm_aux3, vmm_src); // vmm_aux3 = x
+
+    h->uni_vminps(vmm_src, vmm_src, table_val(bwd_mish_max_x_for_equation_f));
+    exp_compute_vector_fwd(vmm_src);
+    h->uni_vmovups(vmm_aux2, vmm_src); // vmm_aux2 = e^x
+
+    // e^3x + 4*e^2x
+    h->uni_vmulps(vmm_src, vmm_src, vmm_src); // e^2x
+    h->uni_vmovups(vmm_aux1, vmm_src);
+    h->uni_vmulps(vmm_aux1, vmm_aux1, table_val(two));
+    h->uni_vmulps(vmm_aux1, vmm_aux1, table_val(two)); // 4*e^2x
+    h->uni_vfmadd213ps(vmm_src, vmm_aux2, vmm_aux1);
+
+    // e^3x + 4*e^2x + 4*e^x*(x+1.5)
+    h->uni_vaddps(vmm_aux3, vmm_aux3, table_val(one)); // vmm_aux3 = x + 1
+    h->uni_vmovups(vmm_aux1, vmm_aux3);
+    h->uni_vaddps(vmm_aux1, vmm_aux1, table_val(half));
+    h->uni_vmulps(vmm_aux1, vmm_aux1, table_val(two));
+    h->uni_vmulps(vmm_aux1, vmm_aux1, table_val(two));
+    h->uni_vfmadd231ps(vmm_src, vmm_aux1, vmm_aux2);
+
+    // omega = e^3x + 4*e^2x + 4*e^x*(x+1.5) + 4*(x+1)
+    h->uni_vmulps(vmm_aux3, vmm_aux3, table_val(two));
+    h->uni_vfmadd231ps(vmm_src, vmm_aux3, table_val(two));
+
+    // delta = (e^x+1)^2 + 1
+    h->uni_vmovups(vmm_aux1, vmm_aux2);
+    h->uni_vaddps(vmm_aux1, vmm_aux1, table_val(one));
+    h->uni_vmulps(vmm_aux1, vmm_aux1, vmm_aux1);
+    h->uni_vaddps(vmm_aux1, vmm_aux1, table_val(one));
+    h->uni_vmulps(vmm_aux1, vmm_aux1, vmm_aux1);
+
+    // e^x * omega / delta^2
+    h->uni_vmulps(vmm_src, vmm_src, vmm_aux2);
+    h->uni_vdivps(vmm_src, vmm_src, vmm_aux1);
+}
+
 template <cpu_isa_t isa>
 void jit_uni_eltwise_injector_f32<isa>::logistic_compute_vector_bwd(
         const Vmm &vmm_src) {
@@ -1236,10 +1370,13 @@ void jit_uni_eltwise_injector_f32<isa>::log_compute_vector_bwd(
 template <cpu_isa_t isa>
 void jit_uni_eltwise_injector_f32<isa>::clip_compute_vector_bwd(
         const Vmm &vmm_src) {
+    using namespace alg_kind;
+
     // set result with 1.f
     h->uni_vmovups(vmm_aux1, table_val(one));
-    // get mask of values > beta and blend with 0.f
-    compute_cmp_mask(vmm_src, table_val(beta), _cmp_gt_os);
+    const auto cmp_flag = alg_ == eltwise_clip ? _cmp_gt_os : _cmp_ge_os;
+    // get mask of values > beta (or >= beta) and blend with 0.f
+    compute_cmp_mask(vmm_src, table_val(beta), cmp_flag);
     blend_with_mask(vmm_aux1, table_val(zero));
     // get mask of values <= alpha and blend with 0.f
     compute_cmp_mask(vmm_src, table_val(alpha), _cmp_le_os);
@@ -1338,6 +1475,21 @@ void jit_uni_eltwise_injector_f32<isa>::gelu_erf_compute_vector_bwd(
     h->uni_vmovups(vmm_src, vmm_aux2);
 }
 
+template <cpu_isa_t isa>
+void jit_uni_eltwise_injector_f32<isa>::hardswish_compute_vector_bwd(
+        const Vmm &vmm_src) {
+    h->uni_vmovups(vmm_aux1, vmm_src);
+
+    h->uni_vaddps(vmm_src, vmm_src, vmm_aux1);
+    h->uni_vaddps(vmm_src, vmm_src, table_val(three));
+    h->uni_vdivps(vmm_src, vmm_src, table_val(six));
+
+    compute_cmp_mask(vmm_aux1, table_val(minus_three), _cmp_le_os);
+    blend_with_mask(vmm_src, table_val(zero));
+    compute_cmp_mask(vmm_aux1, table_val(three), _cmp_ge_os);
+    blend_with_mask(vmm_src, table_val(one));
+}
+
 template <cpu_isa_t isa>
 size_t jit_uni_eltwise_injector_f32<isa>::aux_gprs_count() {
     using namespace alg_kind;
@@ -1373,7 +1525,9 @@ size_t jit_uni_eltwise_injector_f32<isa>::aux_vecs_count() {
             case eltwise_sqrt: return 0;
             case eltwise_linear: return 1;
             case eltwise_bounded_relu: return 0;
-            case eltwise_soft_relu: return 4;
+            case eltwise_soft_relu:
+            case eltwise_logsigmoid: return 4;
+            case eltwise_mish: return 4;
             case eltwise_logistic_use_dst_for_bwd:
             case eltwise_logistic: return 4;
             case eltwise_exp_use_dst_for_bwd:
@@ -1381,10 +1535,13 @@ size_t jit_uni_eltwise_injector_f32<isa>::aux_vecs_count() {
             case eltwise_gelu_tanh: return 5;
             case eltwise_swish: return 4;
             case eltwise_log: return 5;
-            case eltwise_clip: return 0;
+            case eltwise_clip:
+            case eltwise_clip_v2_use_dst_for_bwd:
+            case eltwise_clip_v2: return 0;
             case eltwise_pow: return 2;
             case eltwise_gelu_erf: return 5;
             case eltwise_round: return 0;
+            case eltwise_hardswish: return 1;
             default: assert(!"unsupported eltwise algorithm");
         }
     } else {
@@ -1401,7 +1558,9 @@ size_t jit_uni_eltwise_injector_f32<isa>::aux_vecs_count() {
             case eltwise_sqrt: return 1;
             case eltwise_linear: return 0;
             case eltwise_bounded_relu: return 1;
-            case eltwise_soft_relu: return 4;
+            case eltwise_soft_relu:
+            case eltwise_logsigmoid: return 4;
+            case eltwise_mish: return 4;
             case eltwise_logistic_use_dst_for_bwd: return 1;
             case eltwise_logistic: return 4;
             case eltwise_exp_use_dst_for_bwd: return 0;
@@ -1409,9 +1568,12 @@ size_t jit_uni_eltwise_injector_f32<isa>::aux_vecs_count() {
             case eltwise_gelu_tanh: return 5;
             case eltwise_swish: return 4;
             case eltwise_log: return 1;
-            case eltwise_clip: return 2;
+            case eltwise_clip:
+            case eltwise_clip_v2_use_dst_for_bwd:
+            case eltwise_clip_v2: return 2;
             case eltwise_pow: return 2;
             case eltwise_gelu_erf: return 5;
+            case eltwise_hardswish: return 2;
             default: assert(!"unsupported eltwise algorithm");
         }
     }
@@ -1450,6 +1612,10 @@ void jit_uni_eltwise_injector_f32<isa>::compute_body(
                 case eltwise_soft_relu:
                     soft_relu_compute_vector_fwd(Vmm(idx));
                     break;
+                case eltwise_logsigmoid:
+                    logsigmoid_compute_vector_fwd(Vmm(idx));
+                    break;
+                case eltwise_mish: mish_compute_vector_fwd(Vmm(idx)); break;
                 case eltwise_logistic_use_dst_for_bwd:
                 case eltwise_logistic:
                     logistic_compute_vector_fwd(Vmm(idx));
@@ -1460,12 +1626,17 @@ void jit_uni_eltwise_injector_f32<isa>::compute_body(
                     gelu_tanh_compute_vector_fwd(Vmm(idx));
                     break;
                 case eltwise_log: log_compute_vector_fwd(Vmm(idx)); break;
-                case eltwise_clip: clip_compute_vector_fwd(Vmm(idx)); break;
+                case eltwise_clip:
+                case eltwise_clip_v2_use_dst_for_bwd:
+                case eltwise_clip_v2: clip_compute_vector_fwd(Vmm(idx)); break;
                 case eltwise_pow: pow_compute_vector_fwd(Vmm(idx)); break;
                 case eltwise_gelu_erf:
                     gelu_erf_compute_vector_fwd(Vmm(idx));
                     break;
                 case eltwise_round: round_compute_vector_fwd(Vmm(idx)); break;
+                case eltwise_hardswish:
+                    hardswish_compute_vector_fwd(Vmm(idx));
+                    break;
                 default: assert(!"unsupported eltwise algorithm");
             }
         } else {
@@ -1487,6 +1658,10 @@ void jit_uni_eltwise_injector_f32<isa>::compute_body(
                 case eltwise_soft_relu:
                     soft_relu_compute_vector_bwd(Vmm(idx));
                     break;
+                case eltwise_logsigmoid:
+                    logsigmoid_compute_vector_bwd(Vmm(idx));
+                    break;
+                case eltwise_mish: mish_compute_vector_bwd(Vmm(idx)); break;
                 case eltwise_logistic_use_dst_for_bwd:
                 case eltwise_logistic:
                     logistic_compute_vector_bwd(Vmm(idx));
@@ -1498,11 +1673,16 @@ void jit_uni_eltwise_injector_f32<isa>::compute_body(
                     break;
                 case eltwise_swish: swish_compute_vector_bwd(Vmm(idx)); break;
                 case eltwise_log: log_compute_vector_bwd(Vmm(idx)); break;
-                case eltwise_clip: clip_compute_vector_bwd(Vmm(idx)); break;
+                case eltwise_clip:
+                case eltwise_clip_v2_use_dst_for_bwd:
+                case eltwise_clip_v2: clip_compute_vector_bwd(Vmm(idx)); break;
                 case eltwise_pow: pow_compute_vector_bwd(Vmm(idx)); break;
                 case eltwise_gelu_erf:
                     gelu_erf_compute_vector_bwd(Vmm(idx));
                     break;
+                case eltwise_hardswish:
+                    hardswish_compute_vector_bwd(Vmm(idx));
+                    break;
                 default: assert(!"unsupported eltwise algorithm");
             }
         }
@@ -1602,6 +1782,7 @@ void jit_uni_eltwise_injector_f32<isa>::register_table_entries() {
 
     // exp(x) polynomial approximation
     static const table_t exp_polynomial {
+            // p0 = 1.0f
             {exp_pol, {0x3f7ffffb, true}}, // p1 = 0.999999701f
             {exp_pol, {0x3efffee3, true}}, // p2 = 0.499991506f
             {exp_pol, {0x3e2aad40, true}}, // p3 = 0.166676521f
@@ -1609,6 +1790,11 @@ void jit_uni_eltwise_injector_f32<isa>::register_table_entries() {
             {exp_pol, {0x3c07cfce, true}} // p5 = 0.00828929059f
     };
 
+    // mish(x) constants
+    static const table_t mish_consts {
+            {fwd_mish_max_x_for_equation_f, {0x42317217, true}},
+            {bwd_mish_max_x_for_equation_f, {0x41b17217, true}}};
+
     // tanh(x) constants for four interval approximation
     static const table_t tanh_consts {{tanh_idx_bias, {0x39800000, true}},
             {tanh_idx_mask, {0xffc00000, true}},
@@ -2010,6 +2196,9 @@ void jit_uni_eltwise_injector_f32<isa>::register_table_entries() {
                     {0xc2b00f34, true}}, // 63: -88.029693603515625
     };
 
+    static const table_t hardswish_consts {{three, {0x40400000, true}},
+            {six, {0x40c00000, true}}, {minus_three, {0xc0400000, true}}};
+
     // This object takes care about which constants and polynomials to include.
     struct need_t {
         need_t(alg_kind_t alg) {
@@ -2026,25 +2215,32 @@ void jit_uni_eltwise_injector_f32<isa>::register_table_entries() {
                 case eltwise_gelu_tanh: gelu_tanh_ = true; break;
                 case eltwise_log: log_ = true; break;
                 case eltwise_soft_relu: soft_relu_ = true; break;
+                case eltwise_logsigmoid: soft_relu_ = true; break;
+                case eltwise_mish: mish_ = true; break;
                 case eltwise_tanh_use_dst_for_bwd:
                 case eltwise_tanh: tanh_ = true; break;
+                case eltwise_hardswish: hardswish_ = true; break;
                 default: break;
             }
         }
 
         bool exp_ = false;
+        bool mish_ = false;
         bool tanh_ = false;
         bool soft_relu_ = false;
         bool gelu_tanh_ = false;
         bool gelu_erf_ = false;
         bool log_ = false;
+        bool hardswish_ = false;
 
-        bool exp() const { return exp_ || soft_relu_ || gelu_erf_; }
+        bool exp() const { return exp_ || soft_relu_ || gelu_erf_ || mish_; }
+        bool mish() const { return mish_; }
         bool tanh() const { return tanh_ || gelu_tanh_; }
         bool soft_relu() const { return soft_relu_; }
         bool gelu_tanh() const { return gelu_tanh_; }
         bool gelu_erf() const { return gelu_erf_; }
         bool log() const { return log_; }
+        bool hardswish() const { return hardswish_; }
     };
 
     need_t need(alg_);
@@ -2069,6 +2265,7 @@ void jit_uni_eltwise_injector_f32<isa>::register_table_entries() {
     push_entries_of(common_values);
     if (need.exp()) push_entries_of(exp_consts);
     if (need.exp()) push_entries_of(exp_polynomial);
+    if (need.mish()) push_entries_of(mish_consts);
     if (need.tanh()) push_entries_of(tanh_consts);
     if (need.tanh()) push_entries_of(tanh_polynomial_table);
     if (need.soft_relu()) push_entries_of(soft_relu_consts);
@@ -2079,6 +2276,7 @@ void jit_uni_eltwise_injector_f32<isa>::register_table_entries() {
     if (need.log()) push_entries_of(log_consts);
     if (need.log()) push_entries_of(log_polynomial);
     if (need.log()) push_entries_of(log_predefined_values);
+    if (need.hardswish()) push_entries_of(hardswish_consts);
 
     // Now that we registered the entries, we set the offsets.  No
     // entries should be registered after this point.  This allows to
