@@ -43,12 +43,11 @@ struct jit_resampling_args_t {
     dim_t w; // fwd: ow  bwd: iw
 };
 
-struct jit_avx512_common_resampling_kernel : public c_compatible {
+struct jit_avx512_common_resampling_kernel : public jit_generator {
     jit_avx512_common_resampling_kernel(const resampling_pd_t *pd) : pd_(pd) {}
-    virtual ~jit_avx512_common_resampling_kernel() = default;
-
-    virtual status_t create_kernel() = 0;
-    virtual void operator()(const jit_resampling_args_t *args) = 0;
+    void operator()(const jit_resampling_args_t *args) {
+        jit_generator::operator()(args);
+    }
 
 protected:
     // Convert between vector register lengths.
@@ -73,8 +72,7 @@ protected:
 namespace {
 
 struct jit_avx512_common_resampling_t
-    : public jit_avx512_common_resampling_kernel,
-      public jit_generator {
+    : public jit_avx512_common_resampling_kernel {
     DECLARE_CPU_JIT_AUX_FUNCTIONS(jit_avx512_common_resampling)
 
     jit_avx512_common_resampling_t(const resampling_pd_t *pd)
@@ -101,12 +99,6 @@ struct jit_avx512_common_resampling_t
         stack_size_needed_ = 0;
     }
 
-    status_t create_kernel() override { return jit_generator::create_kernel(); }
-
-    void operator()(const jit_resampling_args_t *args) override {
-        return jit_generator::operator()(args);
-    }
-
 private:
     enum class rounding_mode { none, floor, ceil, rounding_mode_max };
 
@@ -845,94 +837,18 @@ private:
     bwd_counting_range_t oh;
     bwd_counting_range_t od;
 
-    dim_t stride_d_;
-    dim_t stride_h_;
-    dim_t stride_w_;
-    dim_t inner_stride_;
-    unsigned number_of_loops_;
-    size_t tail_mask_;
-    bool use_bf16_emulation_;
-    unsigned stack_size_needed_;
+    dim_t stride_d_ = 0;
+    dim_t stride_h_ = 0;
+    dim_t stride_w_ = 0;
+    dim_t inner_stride_ = 0;
+    unsigned number_of_loops_ = 0;
+    size_t tail_mask_ = 0;
+    bool use_bf16_emulation_ = false;
+    unsigned stack_size_needed_ = 0;
 };
 
 } // namespace
 
-template <data_type_t d_type>
-status_t jit_avx512_common_resampling_fwd_t<d_type>::pd_t::init(
-        engine_t *engine) {
-    using namespace format_tag;
-    using namespace data_type;
-    bool ok = mayiuse(avx512_common)
-            && IMPLICATION(d_type == bf16, mayiuse(avx512_core)) && is_fwd()
-            && !has_zero_dim_memory()
-            && utils::everyone_is(
-                    d_type, src_md()->data_type, dst_md()->data_type)
-            && set_default_params() == status::success
-            && attr()->has_default_values();
-    if (!ok) return status::unimplemented;
-
-    format_tag_t dat_tag = memory_desc_matches_one_of_tag(*src_md(), nCw8c,
-            nChw8c, nCdhw8c, nCw16c, nChw16c, nCdhw16c, nwc, nhwc, ndhwc);
-    if (!memory_desc_matches_tag(*dst_md(), dat_tag))
-        return status::unimplemented;
-
-    return status::success;
-}
-
-template <impl::data_type_t d_type>
-inline jit_avx512_common_resampling_fwd_t<
-        d_type>::jit_avx512_common_resampling_fwd_t(const pd_t *apd)
-    : primitive_t(apd) {}
-
-template <impl::data_type_t d_type>
-status_t jit_avx512_common_resampling_fwd_t<d_type>::init(engine_t *engine) {
-    CHECK(safe_ptr_assign(kernel_, new jit_avx512_common_resampling_t(pd())));
-    return kernel_->create_kernel();
-}
-
-template <impl::data_type_t d_type>
-jit_avx512_common_resampling_fwd_t<
-        d_type>::~jit_avx512_common_resampling_fwd_t()
-        = default;
-
-template <impl::data_type_t d_type>
-status_t jit_avx512_common_resampling_fwd_t<d_type>::execute(
-        const exec_ctx_t &ctx) const {
-    const auto src = CTX_IN_MEM(const data_t *, DNNL_ARG_SRC);
-    auto dst = CTX_OUT_MEM(data_t *, DNNL_ARG_DST);
-
-    const dim_t OD = pd()->OD();
-    const dim_t OH = pd()->OH();
-    const dim_t OW = pd()->OW();
-    const dim_t ID = pd()->ID();
-    const dim_t IH = pd()->IH();
-    const dim_t IW = pd()->IW();
-
-    const memory_desc_wrapper src_d(pd()->src_md());
-    const dim_t inner_stride = src_d.blocking_desc().strides[pd()->ndims() - 1];
-    const dim_t nsp_outer = src_d.nelems(true) / (ID * IH * IW * inner_stride);
-
-    parallel_nd(nsp_outer, OD, OH, OW,
-            [&](dim_t nsp, dim_t od, dim_t oh, dim_t ow) {
-                dim_t src_off = nsp * ID * IH * IW * inner_stride;
-                dim_t dst_off
-                        = (nsp * OD * OH * OW + od * OH * OW + oh * OW + ow)
-                        * inner_stride;
-                jit_resampling_args_t args;
-                args.src = src + src_off;
-                args.dst = dst + dst_off;
-                args.d = od;
-                args.h = oh;
-                args.w = ow;
-                (*kernel_)(&args);
-            });
-
-    return status_t();
-}
-
-template struct jit_avx512_common_resampling_fwd_t<data_type::bf16>;
-template struct jit_avx512_common_resampling_fwd_t<data_type::f32>;
-
 template <data_type_t d_type>
 status_t jit_avx512_common_resampling_bwd_t<d_type>::pd_t::init(
         engine_t *engine) {
@@ -1015,4 +931,4 @@ template struct jit_avx512_common_resampling_bwd_t<data_type::f32>;
 } // namespace x64
 } // namespace cpu
 } // namespace impl
-} // namespace dnnl
\ No newline at end of file
+} // namespace dnnl
