@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2020 Intel Corporation
+* Copyright 2016-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -21,6 +21,9 @@
 #include "common/type_helpers.hpp"
 #include "common/utils.hpp"
 
+#include "cpu/x64/injectors/injector_utils.hpp"
+#include "cpu/x64/injectors/jit_uni_binary_injector.hpp"
+#include "cpu/x64/injectors/jit_uni_eltwise_injector.hpp"
 #include "cpu/x64/jit_avx512_core_x8s8s32x_conv_kernel.hpp"
 
 #define GET_OFF(field) offsetof(jit_conv_call_s, field)
@@ -49,19 +52,33 @@ void pick_loop_order(jit_conv_conf_t &jcp, int nthr) {
 } // namespace
 
 template <typename Vmm>
-bool _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::maybe_eltwise(int position) {
-    using namespace primitive_kind;
-    const auto &p = attr_.post_ops_;
+_jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::_jit_avx512_core_x8s8s32x_fwd_kernel(
+        const jit_conv_conf_t &ajcp, const primitive_attr_t &attr,
+        const memory_desc_t &dst_md)
+    : jcp(ajcp), attr_(attr), postops_injector_(nullptr) {
+    if (jcp.with_eltwise || jcp.with_binary || jcp.with_sum) {
+        using namespace binary_injector;
+        static constexpr bool preserve_gpr = true;
+        static constexpr bool preserve_vmm = false;
+        static constexpr size_t helper_vmm_idx = 31;
+        const size_t oc_block_tail = jcp.oc_block % isa_simd_width_;
+        const size_t tail_size = oc_block_tail
+                ? oc_block_tail
+                : jcp.oc_without_padding % isa_simd_width_;
+        static constexpr bool use_exact_tail_scalar_bcast = false;
 
-    if (position == 0) {
-        /* eltwise before sum */
-        return p.contain(eltwise, 0);
-    } else if (position == 1) {
-        /* eltwise after sum */
-        return p.contain(sum, 0) && p.contain(eltwise, 1);
+        const rhs_arg_static_params_t rhs_arg_static_params {helper_vmm_idx,
+                r14, r15, preserve_gpr, preserve_vmm,
+                GET_OFF(post_ops_binary_rhs_arg_vec),
+                memory_desc_wrapper(dst_md), tail_size, postops_mask,
+                use_exact_tail_scalar_bcast};
+        const static_params_t static_params {
+                this->param1, rhs_arg_static_params};
+
+        postops_injector_ = utils::make_unique<
+                injector::jit_uni_postops_injector_t<avx512_core>>(
+                this, jcp.post_ops, static_params);
     }
-
-    return false;
 }
 
 template <typename Vmm>
@@ -110,11 +127,101 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::cvt2ps(data_type_t type_in,
     if (type_in != data_type::f32) vcvtdq2ps(vmm_in, vmm_in);
 }
 
+template <typename F>
+static void iterate(const int nb_oc_block, const int ur_w,
+        const bool last_oc_block_flag, const bool force_masking, const F &f) {
+    for (int k = 0; k < nb_oc_block; k++) {
+        const bool mask_flag
+                = force_masking || (last_oc_block_flag && k + 1 == nb_oc_block);
+        for (int j = 0; j < ur_w; j++)
+            f(mask_flag, k, j);
+    }
+}
+template <typename F>
+static void iterate(const int nb_oc_block, const int ur_w,
+        const bool last_oc_block_flag, const F &f) {
+    iterate(nb_oc_block, ur_w, last_oc_block_flag, false, f);
+}
+template <typename F>
+static void iterate(const int nb_oc_block, const int ur_w, const F &f) {
+    iterate(nb_oc_block, ur_w, false, false, f);
+}
+
 template <typename Vmm>
-void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::compute_eltwise(int ur_w) {
-    int nb_oc_block
-            = jcp.is_depthwise ? jcp.nb_ch_blocking : jcp.nb_oc_blocking;
-    eltwise_injector_->compute_vector_range(0, nb_oc_block * ur_w);
+void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::apply_sum(int ur_w,
+        bool last_oc_block_flag, const int nb_oc_block, const int oc_block,
+        const float *p_sum_scale) {
+    if (jcp.with_sum) {
+        const float sum_scale = *p_sum_scale;
+        const auto sum_injector_lam = [this, oc_block, sum_scale](
+                                              const bool mask_flag, const int k,
+                                              const int j) {
+            int aux_output_offset = jcp.typesize_out
+                    * (k * oc_block + j * jcp.oc_without_padding * jcp.ngroups);
+            auto addr = EVEX_compress_addr(reg_out, aux_output_offset);
+            Vmm vmm = vmm_out(j, k);
+            cvt2ps(jcp.dst_dt, vmm_prev_dst, addr, mask_flag);
+            if (sum_scale == 1.f)
+                vaddps(vmm, vmm_prev_dst);
+            else
+                vfmadd231ps(vmm, vmm_prev_dst, zword_b[reg_ptr_sum_scale]);
+        };
+        const auto sum_injector = [=]() {
+            iterate(nb_oc_block, ur_w, last_oc_block_flag, sum_injector_lam);
+        };
+        if (sum_scale != 1.f) mov(reg_ptr_sum_scale, (size_t)p_sum_scale);
+        postops_injector_->set_lambda_injector(
+                primitive_kind::sum, sum_injector);
+    }
+}
+
+template <typename Vmm>
+void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::apply_postops(int ur_w,
+        bool last_oc_block_flag, const int nb_oc_block, const int oc_block,
+        const float *p_sum_scale) {
+    if (jcp.with_eltwise || jcp.with_binary || jcp.with_sum) {
+        apply_sum(ur_w, last_oc_block_flag, nb_oc_block, oc_block, p_sum_scale);
+
+        injector_utils::vmm_index_set_t vmm_idxs;
+        if (jcp.with_binary) {
+            binary_injector::rhs_arg_dynamic_params_t rhs_arg_params;
+            const auto temp_offset_reg = this->r12;
+            const bool oc_blk_is_smaller_than_vmm = oc_block < isa_simd_width_;
+            iterate(nb_oc_block, ur_w, last_oc_block_flag,
+                    oc_blk_is_smaller_than_vmm,
+                    [&](const bool mask_flag, const int k, const int j) {
+                        const int aux_output_l_off = (k * oc_block
+                                + j * jcp.oc_without_padding * jcp.ngroups);
+                        const auto vmm_idx = vmm_out_idx(j, k);
+                        vmm_idxs.emplace(vmm_idx);
+
+                        rhs_arg_params.vmm_idx_to_oc_elem_off_addr.emplace(
+                                vmm_idx, ptr[param1 + GET_OFF(oc_l_off)]);
+                        rhs_arg_params.vmm_idx_to_oc_elem_off_val.emplace(
+                                vmm_idx, k * oc_block);
+                        rhs_arg_params.vmm_idx_to_out_off_oprnd.emplace(
+                                vmm_idx, temp_offset_reg);
+                        rhs_arg_params.vmm_idx_to_out_elem_off_val.emplace(
+                                vmm_idx, aux_output_l_off);
+                        if (mask_flag)
+                            rhs_arg_params.vmm_tail_idx_.emplace(vmm_idx);
+                    });
+
+            const injector_utils::register_preserve_guard_t register_guard(
+                    this, {temp_offset_reg});
+            mov(temp_offset_reg, reg_out);
+            sub(temp_offset_reg, ptr[param1 + GET_OFF(dst_orig)]);
+            shr(temp_offset_reg, std::log2(sizeof(float)));
+
+            postops_injector_->compute_vector_range(vmm_idxs, rhs_arg_params);
+        } else {
+            iterate(nb_oc_block, ur_w,
+                    [&](const bool, const int k, const int j) {
+                        vmm_idxs.emplace(vmm_out_idx(j, k));
+                    });
+            postops_injector_->compute_vector_range(vmm_idxs);
+        }
+    }
 }
 
 template <typename Vmm>
@@ -162,28 +269,31 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::store_output(
         }
         if (jcp.signed_input) {
             int comp_offset = sizeof(int32_t) * k * oc_block;
-            auto comp_addr = EVEX_compress_addr(reg_compensation, comp_offset);
-            cvt2ps(data_type::s32, vmm_comp, comp_addr, mask_flag);
+            Vmm vmm_comp_ = vmm_mask(vmm_comp, mask_flag);
+            vmovups(vmm_comp_,
+                    EVEX_compress_addr(reg_compensation, comp_offset));
         }
         if (jcp.src_zero_point) {
             // zero_point: conv(src_x8, wei_s8) - src_shift_s32 * compensation_s32
             int zp_offset = sizeof(int32_t) * k * oc_block;
-            vmovups(vmm_zp, EVEX_compress_addr(reg_zp_compensation, zp_offset));
-            vpmulld(vmm_zp, vmm_zp,
+            Vmm vmm_zp_ = vmm_mask(vmm_zp, mask_flag);
+            vmovups(vmm_zp_,
+                    EVEX_compress_addr(reg_zp_compensation, zp_offset));
+            vpmulld(vmm_zp_, vmm_zp_,
                     EVEX_compress_addr(
                             reg_src_zero_point, 0, jcp.zp_src_is_common));
-            // upscale to f32
-            Vmm vmm_ = vmm_mask(vmm_zp, mask_flag);
-            vcvtdq2ps(vmm_, vmm_);
         }
         /* add to zmm_accum: compensation, zero_point, bias and permute */
         for (int j = 0; j < ur_w; j++) {
             Vmm vmm = vmm_out(j, k);
             if (jcp.is_fast_depthwise)
                 vpermd(zmm_out(j, k), zmm_permute, zmm_out(j, k));
+            /* add comp in s32 to avoid loss of precision
+               when convert s32 to f32 in integer(2^24)
+               TODO: do the same to bias */
+            if (jcp.signed_input) vpaddd(vmm, vmm, vmm_comp);
+            if (jcp.src_zero_point) vpaddd(vmm, vmm, vmm_zp);
             vcvtdq2ps(vmm, vmm);
-            if (jcp.signed_input) vaddps(vmm, vmm, vmm_comp);
-            if (jcp.src_zero_point) vaddps(vmm, vmm, vmm_zp);
 
             if (jcp.with_bias) vaddps(vmm, vmm, vmm_bias);
 
@@ -193,28 +303,7 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::store_output(
         }
     }
 
-    /* Do post-ops */
-    if (p_sum_scale && *p_sum_scale != 1.f)
-        mov(reg_ptr_sum_scale, (size_t)p_sum_scale);
-    if (maybe_eltwise(0)) compute_eltwise(ur_w);
-    if (p_sum_scale) { // post_op: sum
-        for (int k = 0; k < nb_oc_block; k++) {
-            const bool mask_flag = last_oc_block_flag && k == nb_oc_block - 1;
-            for (int j = 0; j < ur_w; j++) {
-                int aux_output_offset = jcp.typesize_out
-                        * (k * oc_block
-                                + j * jcp.oc_without_padding * jcp.ngroups);
-                auto addr = EVEX_compress_addr(reg_out, aux_output_offset);
-                Vmm vmm = vmm_out(j, k);
-                cvt2ps(jcp.dst_dt, vmm_prev_dst, addr, mask_flag);
-                if (*p_sum_scale == 1.f)
-                    vaddps(vmm, vmm_prev_dst);
-                else
-                    vfmadd231ps(vmm, vmm_prev_dst, zword_b[reg_ptr_sum_scale]);
-            }
-        }
-    }
-    if (maybe_eltwise(1)) compute_eltwise(ur_w);
+    apply_postops(ur_w, last_oc_block_flag, nb_oc_block, oc_block, p_sum_scale);
 
     if (jcp.dst_zero_point) {
         mov(reg_dst_zero_point, ptr[param1 + GET_OFF(dst_zero_point)]);
@@ -487,9 +576,11 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::compute_ker(int ur_w, int pad_l,
         if (compute_kernel) {
             for (int ic = 0; ic < icb; ic++) {
                 if (h_padded) {
-                    /* fill padded area with shifted values */
-                    Vmm inp = vmm_inp(0, nb_oc_block);
-                    vmovups(inp, vmm_shift); // bcast(128)
+                    // fill padded area with shifted value in first iteration
+                    if (ic == 0) {
+                        Vmm inp = vmm_inp(0, nb_oc_block);
+                        vmovups(inp, vmm_shift); // bcast(128)
+                    }
                 } else {
                     for (int jj = _start; jj < _end; jj++) {
                         int aux_input_offset = input_offset(jj, ic, ki);
@@ -510,8 +601,9 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::compute_ker(int ur_w, int pad_l,
                                 vpaddb(vmm_inp(jj, nb_oc_block),
                                         vmm_inp(jj, nb_oc_block), vmm_shift);
                         } else {
-                            /* fill padded area with shifted values */
-                            if (jcp.signed_input) {
+                            // fill padded area with shifted value in
+                            // first iteration
+                            if (jcp.signed_input && ic == 0) {
                                 Vmm inp = vmm_inp(jj, nb_oc_block);
                                 vmovups(inp, vmm_shift);
                             }
@@ -760,16 +852,18 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::icb_loop(
     // End of IC Loop
     if (do_icb_loop) {
         int inp_step = jcp.ic_block;
-        int ker_step = jcp.kd * jcp.kh * jcp.kw * jcp.oc_block * jcp.ic_block;
+        const size_t ker_step = (size_t)jcp.kd * jcp.kh * jcp.kw * jcp.oc_block
+                * jcp.ic_block;
         add(reg_inp, jcp.typesize_in * inp_step);
-        add(reg_ker, jcp.typesize_in * ker_step);
+        safe_add(reg_ker, jcp.typesize_in * ker_step, reg_ker_long_offt);
 
         dec(reg_icb);
         cmp(reg_icb, 0);
         jg(icb_label, T_NEAR);
 
         sub(reg_inp, jcp.typesize_in * inp_step * jcp.nb_ic);
-        sub(reg_ker, jcp.typesize_in * ker_step * jcp.nb_ic);
+        safe_sub(reg_ker, jcp.typesize_in * ker_step * jcp.nb_ic,
+                reg_ker_long_offt);
     }
 
     if (jcp.ngroups % jcp.ch_block != 0 || jcp.oc_without_padding != jcp.oc) {
@@ -861,7 +955,14 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::generate() {
         Reg32 regw_tmp = reg_oi.cvt32();
         mov(regw_tmp, mask);
         kmovw(ktail_mask, regw_tmp);
-    }
+        kmovw(postops_mask, regw_tmp);
+    } else if (jcp.with_binary)
+        if (jcp.oc_block != isa_simd_width_) {
+            const int mask = (1 << jcp.oc_block) - 1;
+            const Reg32 regw_tmp = reg_oi.cvt32();
+            mov(regw_tmp, mask);
+            kmovw(postops_mask, regw_tmp);
+        }
     if (jcp.is_fast_depthwise) {
         // prepare mask register for blending weights
         mov(reg_scratch, 0x8888444422221111);
@@ -1151,7 +1252,7 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::generate() {
 
     postamble();
 
-    if (jcp.with_eltwise) eltwise_injector_->prepare_table();
+    if (jcp.with_eltwise) postops_injector_->prepare_table();
 
     if (jcp.is_fast_depthwise) {
         align(64);
@@ -1163,25 +1264,6 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::generate() {
     }
 }
 
-bool jit_avx512_core_x8s8s32x_fwd_kernel::post_ops_ok(
-        jit_conv_conf_t &jcp, const primitive_attr_t &attr) {
-    using namespace primitive_kind;
-    const auto &p = attr.post_ops_;
-
-    auto is_eltwise = [&](int idx) { return p.entry_[idx].is_eltwise(); };
-
-    switch (p.len()) {
-        case 0: return true;
-        case 1: return is_eltwise(0) || p.contain(sum, 0);
-        case 2:
-            return (p.contain(sum, 0) && is_eltwise(1))
-                    || (p.contain(sum, 1) && is_eltwise(0));
-        default: return false;
-    }
-
-    return false;
-}
-
 status_t jit_avx512_core_x8s8s32x_fwd_kernel::init_conf(jit_conv_conf_t &jcp,
         const convolution_desc_t &cd, memory_desc_t &src_md,
         memory_desc_t &weights_md, memory_desc_t &dst_md,
@@ -1285,12 +1367,24 @@ status_t jit_avx512_core_x8s8s32x_fwd_kernel::init_conf(jit_conv_conf_t &jcp,
             return status::unimplemented;
     }
 
-    if (!post_ops_ok(jcp, attr)) return status::unimplemented;
-
-    const auto &p = attr.post_ops_;
-    const int eltwise_ind = p.find(primitive_kind::eltwise);
+    const auto &post_ops = attr.post_ops_;
+    const int eltwise_ind = post_ops.find(primitive_kind::eltwise);
     jcp.with_eltwise = eltwise_ind != -1;
-    if (jcp.with_eltwise) jcp.eltwise = p.entry_[eltwise_ind].eltwise;
+
+    const int binary_ind = post_ops.find(primitive_kind::binary);
+    jcp.with_binary = binary_ind != -1;
+
+    const int sum_ind = post_ops.find(primitive_kind::sum);
+    jcp.with_sum = sum_ind != -1;
+
+    jcp.post_ops = post_ops;
+
+    using namespace injector;
+    static constexpr bool sum_at_pos_0_only = false;
+    static constexpr bool sum_requires_scale_one = false;
+    const bool post_ops_ok_ = post_ops_ok({avx512_core, {eltwise, binary, sum},
+            jcp.post_ops, &dst_d, sum_at_pos_0_only, sum_requires_scale_one});
+    if (!post_ops_ok_) return status::unimplemented;
 
     const auto zp = attr.zero_points_;
     jcp.dst_zero_point = !zp.has_default_values(DNNL_ARG_DST);
@@ -1467,28 +1561,69 @@ status_t jit_avx512_core_x8s8s32x_fwd_kernel::init_conf(jit_conv_conf_t &jcp,
     if (jcp.ow < jcp.ur_w) jcp.ur_w = jcp.ow;
     jcp.ur_w_tail = jcp.ow % jcp.ur_w;
 
-    jcp.ow_block = jcp.ow;
-    int base_work_amount = jcp.mb * jcp.nb_ch * jcp.od * jcp.oh
-            * (jcp.nb_oc / jcp.nb_oc_blocking_thr_chunk);
-    float best_thr_eff
-            = (float)base_work_amount / rnd_up(base_work_amount, jcp.nthr);
-    int max_nb_ow = div_up(jcp.ow, jcp.ur_w);
-    for (int nb_ow = 1; nb_ow <= max_nb_ow; nb_ow++) {
-        int ow_block
-                = nstl::min(rnd_up(div_up(jcp.ow, nb_ow), jcp.ur_w), jcp.ow);
-        if (ow_block < jcp.nb_oc_blocking_thr_chunk * jcp.oc_block
-                && best_thr_eff > 0.8f)
-            break;
-        if (div_up(jcp.ow, ow_block) != nb_ow) continue;
+    auto get_thr_eff = [=](int nb_ow, int nthr) {
+        int base_work_amount = jcp.mb * jcp.nb_ch * jcp.od * jcp.oh
+                * (jcp.nb_oc / jcp.nb_oc_blocking_thr_chunk);
         auto work_amount = base_work_amount * nb_ow;
-        float thr_eff = (float)work_amount / rnd_up(work_amount, jcp.nthr);
-        if (ow_block >= jcp.ur_w && thr_eff > 1.1f * best_thr_eff) {
-            jcp.ow_block = ow_block;
-            best_thr_eff = thr_eff;
+        return float(work_amount) / rnd_up(work_amount, nthr);
+    };
+
+    auto get_ow_block = [=](int ur_w, int nthr) {
+        int res_ow_block = jcp.ow;
+        float best_thr_eff = get_thr_eff(1, nthr);
+        float thr_eff;
+        int max_nb_ow = div_up(jcp.ow, ur_w);
+        for (int nb_ow = 1; nb_ow <= max_nb_ow; nb_ow++) {
+            int ow_block
+                    = nstl::min(rnd_up(div_up(jcp.ow, nb_ow), ur_w), jcp.ow);
+            if (ow_block < jcp.nb_oc_blocking_thr_chunk * jcp.oc_block
+                    && best_thr_eff > 0.8f)
+                break;
+            if (div_up(jcp.ow, ow_block) != nb_ow) continue;
+            thr_eff = get_thr_eff(nb_ow, nthr);
+            if (ow_block >= ur_w && thr_eff > 1.1f * best_thr_eff) {
+                res_ow_block = ow_block;
+                best_thr_eff = thr_eff;
+            }
+            if (best_thr_eff > 0.9f) break;
         }
-        if (best_thr_eff > 0.9f) break;
-    }
+        return res_ow_block;
+    };
+
+    jcp.ow_block = get_ow_block(jcp.ur_w, jcp.nthr);
     jcp.nb_ow = div_up(jcp.ow, jcp.ow_block);
+    float thr_eff = get_thr_eff(jcp.nb_ow, jcp.nthr);
+
+    /* adjust the thread decomposition
+     * to improve the thr_eff for small size problem
+     * the threshold L1_cache_size is empirical */
+    size_t wei_size
+            = sizeof(float) * jcp.ic * jcp.oc * jcp.kh * jcp.kw * jcp.kd;
+    size_t out_size
+            = jcp.mb * jcp.typesize_out * jcp.oc * jcp.oh * jcp.ow * jcp.od;
+    size_t inp_size
+            = jcp.mb * jcp.typesize_in * jcp.ic * jcp.ih * jcp.iw * jcp.id;
+    size_t total_size = jcp.ngroups * (wei_size + out_size + inp_size);
+    const unsigned int L1_cache_size = platform::get_per_core_cache_size(1);
+
+    if (thr_eff < 0.9f && jcp.ngroups < jcp.nthr
+            && (total_size < L1_cache_size)) {
+        int ow_block = jcp.ow_block;
+        float best_thr_eff = -1.0f;
+        float eff = -1.0f;
+        int end_nthr = with_groups ? jcp.ngroups : 1;
+        for (int nthr = jcp.nthr / 2; nthr > end_nthr; nthr--) {
+            ow_block = get_ow_block(jcp.ur_w, nthr);
+            eff = get_thr_eff(div_up(jcp.ow, ow_block), nthr);
+            if (eff > 1.1f * best_thr_eff) {
+                best_thr_eff = eff;
+                jcp.ow_block = ow_block;
+                jcp.nb_ow = div_up(jcp.ow, jcp.ow_block);
+                jcp.nthr = jcp.aligned_threads = nthr;
+                if (best_thr_eff > 0.9f) break;
+            }
+        }
+    }
 
     if (jcp.oc % jcp.oc_block != 0) return status::unimplemented;
 
