@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2020 Intel Corporation
+* Copyright 2016-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -41,10 +41,14 @@ using namespace nstl;
 template <data_type_t src_type, data_type_t dst_type>
 status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
         dst_type>::execute_forward_1d(const exec_ctx_t &ctx) const {
+    const auto &jcp = pd()->jcp_;
+
     auto src = CTX_IN_MEM(const src_data_t *, DNNL_ARG_SRC);
     auto weights = CTX_IN_MEM(const wei_data_t *, DNNL_ARG_WEIGHTS);
     auto bias = CTX_IN_MEM(const char *, DNNL_ARG_BIAS);
     auto dst = CTX_OUT_MEM(dst_data_t *, DNNL_ARG_DST);
+    const auto post_ops_binary_rhs_arg_vec
+            = binary_injector::prepare_binary_args(jcp.post_ops, ctx);
 
     DEFINE_ZERO_POINTS_BUFFER(src_zero_point, DNNL_ARG_SRC);
     DEFINE_ZERO_POINTS_BUFFER(dst_zero_point, DNNL_ARG_DST);
@@ -58,7 +62,6 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
             ? types::data_type_size(pd()->desc()->bias_desc.data_type)
             : 0;
 
-    const auto &jcp = pd()->jcp_;
     assert(jcp.nb_oc % jcp.nb_oc_blocking == 0);
     assert(jcp.nb_ch % jcp.nb_ch_blocking == 0);
 
@@ -77,14 +80,17 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
         oscales = local_scales;
     }
 
-    size_t offset = weights_d.size() - weights_d.additional_buffer_size();
+    size_t extra_data_offset
+            = weights_d.size() - weights_d.additional_buffer_size();
+    size_t ch_offset = jcp.is_depthwise ? jcp.nb_ch * jcp.ch_block
+                                        : jcp.ngroups * jcp.oc;
     auto w = const_cast<wei_data_t *>(weights);
     int32_t *compensation = (jcp.signed_input)
-            ? reinterpret_cast<int32_t *>(&w[offset])
+            ? reinterpret_cast<int32_t *>(&w[extra_data_offset])
             : nullptr;
     int32_t *zp_compensation = jcp.src_zero_point
-            ? reinterpret_cast<int32_t *>(&w[offset])
-                    + (jcp.signed_input ? jcp.ngroups * jcp.oc : 0)
+            ? reinterpret_cast<int32_t *>(&w[extra_data_offset])
+                    + (jcp.signed_input ? ch_offset : 0)
             : nullptr;
 
     int oc_chunks = jcp.nb_oc / jcp.nb_oc_blocking;
@@ -144,6 +150,9 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
             p.b_overflow = 0;
             p.owb = owb;
 
+            p.oc_l_off = (g * jcp.nb_oc + ocb) * jcp.oc_block;
+            p.post_ops_binary_rhs_arg_vec = post_ops_binary_rhs_arg_vec.data();
+            p.dst_orig = dst;
             (*kernel_)(&p);
 
             ++start;
@@ -174,10 +183,13 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
 template <data_type_t src_type, data_type_t dst_type>
 status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
         dst_type>::execute_forward_2d(const exec_ctx_t &ctx) const {
+    const auto &jcp = pd()->jcp_;
     auto src = CTX_IN_MEM(const src_data_t *, DNNL_ARG_SRC);
     auto weights = CTX_IN_MEM(const wei_data_t *, DNNL_ARG_WEIGHTS);
     auto bias = CTX_IN_MEM(const char *, DNNL_ARG_BIAS);
     auto dst = CTX_OUT_MEM(dst_data_t *, DNNL_ARG_DST);
+    const auto post_ops_binary_rhs_arg_vec
+            = binary_injector::prepare_binary_args(jcp.post_ops, ctx);
 
     DEFINE_ZERO_POINTS_BUFFER(src_zero_point, DNNL_ARG_SRC);
     DEFINE_ZERO_POINTS_BUFFER(dst_zero_point, DNNL_ARG_DST);
@@ -191,7 +203,6 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
             ? types::data_type_size(pd()->desc()->bias_desc.data_type)
             : 0;
 
-    const auto &jcp = pd()->jcp_;
     assert(jcp.ch_block == 1);
     assert(jcp.nb_ch_blocking == 1);
     assert(jcp.nb_oc % jcp.nb_oc_blocking == 0);
@@ -292,10 +303,9 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
                     int kh_padding = nstl::max(
                             0, jcp.kh - i_t_overflow - i_b_overflow);
 
-                    size_t wei_stride
-                            = (!(jcp.signed_input || jcp.src_zero_point))
-                            ? i_t_overflow * wht_h_stride
-                            : 0;
+                    size_t wei_stride = (jcp.signed_input || jcp.src_zero_point)
+                            ? 0
+                            : i_t_overflow * wht_h_stride;
                     p.src = src_w + i_t_overflow * dilate_h * src_h_stride;
                     p.dst = dst_w;
                     p.filt = wht_w + wei_stride;
@@ -315,6 +325,10 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
                     p.b_overflow = i_b_overflow;
                     p.owb = owb;
 
+                    p.oc_l_off = (g * jcp.nb_oc + ocb) * jcp.oc_block;
+                    p.post_ops_binary_rhs_arg_vec
+                            = post_ops_binary_rhs_arg_vec.data();
+                    p.dst_orig = dst;
                     (*kernel_)(&p);
 
                     src_w += src_h_stride * jcp.stride_h;
@@ -345,10 +359,13 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
 template <data_type_t src_type, data_type_t dst_type>
 status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
         dst_type>::execute_forward_2d_dw(const exec_ctx_t &ctx) const {
+    const auto &jcp = pd()->jcp_;
     auto src = CTX_IN_MEM(const src_data_t *, DNNL_ARG_SRC);
     auto weights = CTX_IN_MEM(const wei_data_t *, DNNL_ARG_WEIGHTS);
     auto bias = CTX_IN_MEM(const char *, DNNL_ARG_BIAS);
     auto dst = CTX_OUT_MEM(dst_data_t *, DNNL_ARG_DST);
+    const auto post_ops_binary_rhs_arg_vec
+            = binary_injector::prepare_binary_args(jcp.post_ops, ctx);
 
     DEFINE_ZERO_POINTS_BUFFER(src_zero_point, DNNL_ARG_SRC);
     DEFINE_ZERO_POINTS_BUFFER(dst_zero_point, DNNL_ARG_DST);
@@ -362,7 +379,6 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
             ? types::data_type_size(pd()->desc()->bias_desc.data_type)
             : 0;
 
-    const auto &jcp = pd()->jcp_;
     assert(jcp.ic_block == 1);
     assert(jcp.oc_block == 1);
     assert(jcp.nb_ic == 1);
@@ -454,6 +470,11 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
                 p.b_overflow = i_b_overflow;
                 p.owb = owb;
 
+                p.oc_l_off = g * jcp.oc;
+                p.post_ops_binary_rhs_arg_vec
+                        = post_ops_binary_rhs_arg_vec.data();
+                p.dst_orig = dst;
+
                 (*kernel_)(&p);
             });
     return status::success;
@@ -462,10 +483,13 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
 template <data_type_t src_type, data_type_t dst_type>
 status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
         dst_type>::execute_forward_3d(const exec_ctx_t &ctx) const {
+    const auto &jcp = pd()->jcp_;
     auto src = CTX_IN_MEM(const src_data_t *, DNNL_ARG_SRC);
     auto weights = CTX_IN_MEM(const wei_data_t *, DNNL_ARG_WEIGHTS);
     auto bias = CTX_IN_MEM(const char *, DNNL_ARG_BIAS);
     auto dst = CTX_OUT_MEM(dst_data_t *, DNNL_ARG_DST);
+    const auto post_ops_binary_rhs_arg_vec
+            = binary_injector::prepare_binary_args(jcp.post_ops, ctx);
 
     DEFINE_ZERO_POINTS_BUFFER(src_zero_point, DNNL_ARG_SRC);
     DEFINE_ZERO_POINTS_BUFFER(dst_zero_point, DNNL_ARG_DST);
@@ -479,7 +503,6 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
             ? types::data_type_size(pd()->desc()->bias_desc.data_type)
             : 0;
 
-    const auto &jcp = pd()->jcp_;
     assert(jcp.ch_block == 1);
     assert(jcp.nb_ch_blocking == 1);
     assert(jcp.nb_oc % jcp.nb_oc_blocking == 0);
@@ -599,13 +622,12 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
                     int kh_padding = nstl::max(
                             0, jcp.kh - i_t_overflow - i_b_overflow);
 
-                    size_t wei_stride
-                            = !(jcp.signed_input || jcp.src_zero_point)
-                            ? wht_h_stride
-                            : 0;
+                    size_t wei_stride = (jcp.signed_input || jcp.src_zero_point)
+                            ? 0
+                            : wht_h_stride * i_t_overflow;
                     p.src = src_w + i_t_overflow * dilate_h * src_h_stride;
                     p.dst = dst_w;
-                    p.filt = wht_w + i_t_overflow * wei_stride;
+                    p.filt = wht_w + wei_stride;
                     p.bias = bias_w;
                     p.compensation = compensation_w;
                     p.zp_compensation = jcp.src_zero_point
@@ -625,7 +647,12 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
                     p.back_overflow = d_back_overflow;
                     p.owb = owb;
 
+                    p.oc_l_off = (g * jcp.nb_oc + ocb) * jcp.oc_block;
+                    p.post_ops_binary_rhs_arg_vec
+                            = post_ops_binary_rhs_arg_vec.data();
+                    p.dst_orig = dst;
                     (*kernel_)(&p);
+
                     src_w += src_h_stride * jcp.stride_h;
                     dst_w += dst_h_stride;
                 }
