@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2020 Intel Corporation
+* Copyright 2019-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -34,17 +34,18 @@ using namespace dnnl::impl::utils;
 template <cpu_isa_t isa, data_type_t src_type, data_type_t dst_type>
 void jit_uni_dw_convolution_fwd_t<isa, src_type, dst_type>::execute_forward(
         const exec_ctx_t &ctx) const {
+    const auto &jcp = pd()->jcp_;
     auto src = CTX_IN_MEM(const data_t *, DNNL_ARG_SRC);
     auto weights = CTX_IN_MEM(const data_t *, DNNL_ARG_WEIGHTS);
     auto dst = CTX_OUT_MEM(dst_data_t *, DNNL_ARG_DST);
+    const auto post_ops_binary_rhs_arg_vec
+            = binary_injector::prepare_binary_args(jcp.post_ops, ctx);
 
     const memory_desc_wrapper src_d(pd()->src_md());
     const memory_desc_wrapper dst_d(pd()->dst_md());
     const memory_desc_wrapper weights_d(pd()->weights_md(0));
     const memory_desc_wrapper bias_d(pd()->weights_md(1));
 
-    const auto &jcp = pd()->jcp_;
-
     f32_data_t *bias = nullptr;
     if (pd()->desc()->bias_desc.data_type == data_type::bf16) {
         auto bias_in = CTX_IN_MEM(const bf16_data_t *, DNNL_ARG_BIAS);
@@ -129,19 +130,19 @@ void jit_uni_dw_convolution_fwd_t<isa, src_type, dst_type>::execute_forward(
 
             par_conv.kh_padding = (size_t)nstl::max(0, kh_padding);
 
-            if (is_src_layout_nxc) {
-                // maximize jit work along contiguous dimension
-                int work_rem = end - iwork;
-                par_conv.ch_blocks = ch + work_rem * ch_step >= jcp.nb_ch
-                        ? jcp.nb_ch - ch
-                        : work_rem * ch_step;
-                assert(jcp.loop_order == loop_nhwcg);
-            } else {
-                par_conv.ch_blocks
-                        = utils::this_block_size(ch, jcp.nb_ch, ch_step);
-                assert(jcp.loop_order != loop_nhwcg);
-            }
+            assert(IMPLICATION(
+                    jcp.loop_order == loop_nhwcg, is_src_layout_nxc));
+            // For is_src_layout_nxc maximize jit work along contiguous dim.
+            const int work_rem = end - iwork;
+            par_conv.load_work = utils::this_block_size(ch * jcp.ch_block,
+                    jcp.oc_without_padding,
+                    (is_src_layout_nxc ? work_rem * ch_step : ch_step)
+                            * jcp.ch_block);
 
+            par_conv.oc_l_off = ch * jcp.ch_block;
+            par_conv.post_ops_binary_rhs_arg_vec
+                    = post_ops_binary_rhs_arg_vec.data();
+            par_conv.dst_orig = dst;
             (*kernel_)(&par_conv);
 
             if (jcp.loop_order == loop_ngcw) {
@@ -155,7 +156,7 @@ void jit_uni_dw_convolution_fwd_t<isa, src_type, dst_type>::execute_forward(
         }
     });
 
-    if (pd()->wants_zero_pad_dst()) ctx.memory(DNNL_ARG_DST)->zero_pad(ctx);
+    if (pd()->wants_zero_pad_dst()) ctx.zero_pad_output(DNNL_ARG_DST);
 }
 
 template struct jit_uni_dw_convolution_fwd_t<avx512_core, data_type::bf16,
