@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2020 Intel Corporation
+* Copyright 2017-2021 Intel Corporation
 * Copyright 2018 YANDEX LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
@@ -15,12 +15,9 @@
 * limitations under the License.
 *******************************************************************************/
 
-#include "common/c_types_map.hpp"
-#include "common/nstl.hpp"
-#include "common/utils.hpp"
-#include "cpu/cpu_pooling_pd.hpp"
-
 #include "cpu/x64/jit_uni_pool_kernel.hpp"
+#include "cpu/cpu_pooling_pd.hpp"
+#include "cpu/x64/jit_avx512_core_bf16cvt.hpp"
 
 namespace dnnl {
 namespace impl {
@@ -32,34 +29,48 @@ using namespace alg_kind;
 
 #define GET_OFF(field) offsetof(jit_pool_call_s, field)
 
+static bcast_set_t get_supported_bcast_strategies() {
+    return {broadcasting_strategy_t::scalar, broadcasting_strategy_t::per_oc};
+}
+
 template <cpu_isa_t isa>
 jit_uni_pool_kernel<isa>::~jit_uni_pool_kernel() = default;
 
 template <cpu_isa_t isa>
 jit_uni_pool_kernel<isa>::jit_uni_pool_kernel(
         const jit_pool_conf_t &ajpp, const memory_desc_t *dst_md)
-    : jpp(ajpp), bf16_emu_(nullptr) {
+    : jit_generator(nullptr, MAX_CODE_SIZE, true, isa)
+    , jpp(ajpp)
+    , bf16_emu_(nullptr) {
     if (jpp.is_bf16 && !isa_has_bf16(jpp.isa))
         bf16_emu_ = utils::make_unique<bf16_emulation_t>(this,
                 bf16_emu_reserv_1, bf16_emu_reserv_2, bf16_emu_reserv_3,
                 bf16_emu_reserv_4, bf16_emu_reserv_5);
 
     if (jpp.with_postops) {
-        static constexpr bool use_per_oc_spatial_strategy = false;
+        static constexpr bool preserve_gpr = true;
+        static constexpr bool preserve_vmm = true;
+        static constexpr bool use_exact_tail_scalar_bcast = false;
+        static constexpr int sse41_single_block_size
+                = cpu_isa_traits<sse41>::vlen / sizeof(float);
+        size_t postop_tail = static_cast<size_t>(jpp.c_tail);
+        const bool high_half_block_empty = isa == sse41
+                && static_cast<size_t>(jpp.c_tail) > sse41_single_block_size;
+        if (high_half_block_empty) postop_tail -= sse41_single_block_size;
+
+        const binary_injector::rhs_arg_static_params_t rhs_sp {
+                static_cast<std::size_t>(this->xmm4.getIdx()), this->rax,
+                this->rdx, preserve_gpr, preserve_vmm,
+                GET_OFF(post_ops_binary_rhs_arg_vec),
+                memory_desc_wrapper(*dst_md), postop_tail, k_c_tail_mask,
+                use_exact_tail_scalar_bcast};
+
+        const binary_injector::static_params_t bsp {
+                reg_param, get_supported_bcast_strategies(), rhs_sp};
 
         postops_injector_
                 = utils::make_unique<injector::jit_uni_postops_injector_t<isa>>(
-                        this, jpp.post_ops,
-                        binary_injector::static_params_t(reg_param,
-                                use_per_oc_spatial_strategy,
-                                binary_injector::rhs_arg_static_params_t {
-                                        static_cast<std::size_t>(
-                                                this->xmm4.getIdx()),
-                                        this->rax, this->rdx,
-                                        true /*preserve gpr*/,
-                                        true /*preserve vmm*/,
-                                        GET_OFF(post_ops_binary_rhs_arg_vec),
-                                        memory_desc_wrapper(*dst_md)}));
+                        this, jpp.post_ops, bsp);
     }
 }
 
@@ -140,12 +151,14 @@ status_t jit_uni_pool_kernel<isa>::init_conf(jit_pool_conf_t &jpp,
         // plain output
         jpp.is_bf16 = false;
         jpp.dt_size = types::data_type_size(data_type::f32);
-        jpp.tag_kind = jptg_ncsp;
+        jpp.tag_kind = jit_memory_tag_kind_t::ncsp;
     } else {
         jpp.is_bf16 = (src_d.data_type() == data_type::bf16
                 && dst_d.data_type() == data_type::bf16);
         jpp.dt_size = types::data_type_size(src_d.data_type());
-        jpp.tag_kind = (fmt_tag == nspc_fmt_tag) ? jptg_nspc : jptg_blocked;
+        jpp.tag_kind = (fmt_tag == nspc_fmt_tag)
+                ? jit_memory_tag_kind_t::nspc
+                : jit_memory_tag_kind_t::blocked;
     }
 
     jpp.isa = (jpp.is_bf16 && mayiuse(avx512_core_bf16)) ? avx512_core_bf16
@@ -157,13 +170,14 @@ status_t jit_uni_pool_kernel<isa>::init_conf(jit_pool_conf_t &jpp,
                     pooling_avg_include_padding, pooling_avg_exclude_padding);
     if (!args_ok) return status::unimplemented;
 
-    jpp.c = jpp.tag_kind == jptg_blocked
+    jpp.c = jpp.tag_kind == jit_memory_tag_kind_t::blocked
             ? utils::rnd_up(jpp.c_without_padding, jpp.c_block)
             : jpp.c_without_padding;
-    if (jpp.tag_kind == jptg_blocked) assert(src_d.padded_dims()[1] == jpp.c);
+    if (jpp.tag_kind == jit_memory_tag_kind_t::blocked)
+        assert(src_d.padded_dims()[1] == jpp.c);
     jpp.nb_c = utils::div_up(jpp.c, jpp.c_block);
     jpp.c_tail = jpp.c_without_padding % jpp.c_block;
-    jpp.is_c_padded = jpp.tag_kind == jptg_blocked
+    jpp.is_c_padded = jpp.tag_kind == jit_memory_tag_kind_t::blocked
             && src_d.padded_dims()[1] != jpp.c_without_padding;
 
     jpp.stride_d = (ndims == 5) ? pd.strides[0] : 1;
@@ -220,7 +234,7 @@ status_t jit_uni_pool_kernel<isa>::init_conf(jit_pool_conf_t &jpp,
     }
 
     // select jpp.ur_bc
-    if (jpp.tag_kind == jptg_nspc) {
+    if (jpp.tag_kind == jit_memory_tag_kind_t::nspc) {
         auto min_ur_w = nstl::max(1, utils::div_up(jpp.l_pad, jpp.stride_w));
         int min_ur_w1 = utils::div_up(right_pad, jpp.stride_w);
         if (min_ur_w < min_ur_w1) { min_ur_w = min_ur_w1; }
@@ -265,7 +279,7 @@ status_t jit_uni_pool_kernel<isa>::init_conf(jit_pool_conf_t &jpp,
     // scratchpad for c_block slice of input and/or output
     using namespace memory_tracking::names;
     const int nscr = nstl::min(dnnl_get_max_threads(), jpp.mb * jpp.nb_c);
-    if (jpp.tag_kind == jptg_ncsp) {
+    if (jpp.tag_kind == jit_memory_tag_kind_t::ncsp) {
         scratchpad.book(key_pool_src_plain2blocked_cvt,
                 jpp.c_block * jpp.id * jpp.ih * jpp.iw * nscr, jpp.dt_size);
         scratchpad.book(key_pool_dst_plain2blocked_cvt,
@@ -288,7 +302,7 @@ static int reg_ind(int shift, int bc, int j, int ur_bc, int ur_w) noexcept {
 
 template <cpu_isa_t isa>
 inline void jit_uni_pool_kernel<isa>::prepare_tail_mask() {
-    if (isa >= avx512_common) {
+    if (is_superset(isa, avx512_common)) {
         size_t c_tail_mask = (1ULL << jpp.c_tail) - 1ULL;
         mov(tmp_gpr.cvt32(), c_tail_mask);
         kmovw(k_c_tail_mask, tmp_gpr.cvt32());
@@ -402,8 +416,13 @@ bool jit_uni_pool_kernel<isa>::post_ops_ok(jit_pool_conf_t &jpp,
     if (!jpp.is_backward) {
         for (const auto &entry : entries) {
             if (entry.is_eltwise()) {
-                jpp.with_eltwise = true;
+                const auto alg = entry.eltwise.alg;
+                jpp.with_eltwise = eltwise_injector::is_supported(isa, alg);
             } else if (entry.is_binary()) {
+                if (isa != avx512_core
+                        && entry.binary.src1_desc.data_type == data_type::bf16)
+                    return false;
+
                 jpp.with_binary = true;
             } else
                 return false;
@@ -412,15 +431,13 @@ bool jit_uni_pool_kernel<isa>::post_ops_ok(jit_pool_conf_t &jpp,
         jpp.with_postops = jpp.with_eltwise || jpp.with_binary;
     }
 
-    if (jpp.with_eltwise && isa == avx) return false;
-
-    return binary_injector::binary_args_broadcast_supported(post_ops, dst_d)
-            && binary_injector::binary_args_tail_supported(
-                    post_ops, dst_d, cpu_isa_traits<isa>::vlen);
+    return binary_injector::binary_args_broadcast_supported(
+            post_ops, dst_d, get_supported_bcast_strategies());
 }
 
 template <cpu_isa_t isa>
-void jit_uni_pool_kernel<isa>::apply_postops(int ur_bc, int ur_w, int c_block) {
+void jit_uni_pool_kernel<isa>::apply_postops(int ur_bc, int ur_w, int c_block,
+        const std::function<bool(int, bool)> &is_tail_predicate) {
     binary_injector::rhs_arg_dynamic_params_t rhs_arg_params;
     const int end_idx = vmm_idx_upper_bound() + 1;
     const int start_idx = end_idx - (ur_bc * ur_w);
@@ -428,9 +445,11 @@ void jit_uni_pool_kernel<isa>::apply_postops(int ur_bc, int ur_w, int c_block) {
             = isa == sse41 && disable_postops_when_sse_high_half_processed_;
 
     if (jpp.with_binary && !sse41_postops_disabled) {
+
         static constexpr int sse41_simd_w
                 = cpu_isa_traits<sse41>::vlen / sizeof(float);
         const int sse_elem_off = sse_high_half ? sse41_simd_w : 0;
+
         for (int jj = 0; jj < ur_w; jj++) {
             for (int bci = 0; bci < ur_bc; bci++) {
                 const auto vmm_idx
@@ -439,6 +458,11 @@ void jit_uni_pool_kernel<isa>::apply_postops(int ur_bc, int ur_w, int c_block) {
                         vmm_idx, ptr[reg_param + GET_OFF(c_elem_off)]);
                 rhs_arg_params.vmm_idx_to_oc_elem_off_val.emplace(
                         vmm_idx, bci * c_block + sse_elem_off);
+                if (is_tail_predicate
+                        && is_tail_predicate(
+                                bci, true /*process_with_postops*/)) {
+                    rhs_arg_params.vmm_tail_idx_.emplace(vmm_idx);
+                }
             }
         }
     }
@@ -483,11 +507,13 @@ inline void jit_uni_pool_kernel<isa>::avg_step(int ur_w, int ur_bc, int pad_l,
     auto stride_w = jpp.stride_w;
     auto c_block = jpp.c_block;
     auto dt_size = jpp.dt_size;
-    const int c_off = (jpp.tag_kind == jptg_nspc) ? jpp.c : c_block;
+    const int c_off
+            = (jpp.tag_kind == jit_memory_tag_kind_t::nspc) ? jpp.c : c_block;
     Label kd_label, kh_label;
 
-    const auto is_tail_processing = [&](int bc) {
-        if (isa == sse41 && !jpp.is_c_padded) {
+    const auto is_tail_processing = [&](int bc,
+                                            bool process_with_postops = false) {
+        if (isa == sse41 && (!jpp.is_c_padded || process_with_postops)) {
             return with_c_tail_proccessing && bc == (ur_bc - 1)
                     && ((jpp.c_tail > (jpp.c_block / 2) && sse_high_half)
                             || (jpp.c_tail < (jpp.c_block / 2)
@@ -597,7 +623,8 @@ inline void jit_uni_pool_kernel<isa>::avg_step(int ur_w, int ur_bc, int pad_l,
             }
         }
 
-        if (jpp.with_postops) apply_postops(ur_bc, ur_w, c_block);
+        if (jpp.with_postops)
+            apply_postops(ur_bc, ur_w, c_block, is_tail_processing);
 
         for (int jj = 0; jj < ur_w; jj++) {
             for (int bci = 0; bci < ur_bc; bci++) {
@@ -627,11 +654,12 @@ inline void jit_uni_pool_kernel<isa>::max_step_fwd(int ur_w, int ur_bc,
     int kw = jpp.kw;
     int stride_w = jpp.stride_w;
     int c_block = jpp.c_block;
-    const int c_off = (jpp.tag_kind == jptg_nspc) ? jpp.c : c_block;
+    const int c_off
+            = (jpp.tag_kind == jit_memory_tag_kind_t::nspc) ? jpp.c : c_block;
     Label kd_label, kh_label;
 
-    auto is_tail_processing = [&](int bc) {
-        if (isa == sse41 && !jpp.is_c_padded) {
+    auto is_tail_processing = [&](int bc, bool process_with_postops = false) {
+        if (isa == sse41 && (!jpp.is_c_padded || process_with_postops)) {
             return with_c_tail_proccessing && bc == (ur_bc - 1)
                     && ((jpp.c_tail > (jpp.c_block / 2) && sse_high_half)
                             || (jpp.c_tail < (jpp.c_block / 2)
@@ -750,7 +778,8 @@ inline void jit_uni_pool_kernel<isa>::max_step_fwd(int ur_w, int ur_bc,
     if (with_c_tail_proccessing && jpp.is_c_padded && isa == sse41)
         mov(tmp_gpr, 0); // needed zero to fill padded tail
 
-    if (jpp.with_postops) apply_postops(ur_bc, ur_w, c_block);
+    if (jpp.with_postops)
+        apply_postops(ur_bc, ur_w, c_block, is_tail_processing);
 
     for_(int jj = 0; jj < ur_w; jj++)
     for (int bci = 0; bci < ur_bc; bci++) {
@@ -879,7 +908,8 @@ inline void jit_uni_pool_kernel<isa>::max_step_bwd(int ur_w, int ur_bc,
     int kw = jpp.kw;
     int stride_w = jpp.stride_w;
     int c_block = jpp.c_block;
-    const int c_off = (jpp.tag_kind == jptg_nspc) ? jpp.c : c_block;
+    const int c_off
+            = (jpp.tag_kind == jit_memory_tag_kind_t::nspc) ? jpp.c : c_block;
     Label kd_label, kh_label;
 
     const auto is_tail_processing = [&](int bc) {
@@ -1077,7 +1107,9 @@ inline void jit_uni_pool_kernel<isa>::max_step_bwd(int ur_w, int ur_bc,
 template <cpu_isa_t isa>
 void jit_uni_pool_kernel<isa>::zero_diff_src(
         int ur_bc, bool with_c_tail_proccessing) {
-    const int c_off = (jpp.tag_kind == jptg_nspc) ? jpp.c : jpp.c_block;
+    const int c_off = (jpp.tag_kind == jit_memory_tag_kind_t::nspc)
+            ? jpp.c
+            : jpp.c_block;
 
     Label l_skip, l_ih_loop, l_id_loop;
 
@@ -1163,7 +1195,8 @@ void jit_uni_pool_kernel<isa>::generate() {
     int c_block = jpp.c_block;
     int stride_w = jpp.stride_w;
     int l_pad = jpp.l_pad;
-    const int c_off = (jpp.tag_kind == jptg_nspc) ? jpp.c : c_block;
+    const int c_off
+            = (jpp.tag_kind == jit_memory_tag_kind_t::nspc) ? jpp.c : c_block;
 
     int vlen = cpu_isa_traits<isa>::vlen;
 
@@ -1216,7 +1249,7 @@ void jit_uni_pool_kernel<isa>::generate() {
                  * postops arg1 tensor (nchw format) in per_oc bcast strategy.
                  */
                 disable_postops_when_sse_high_half_processed_
-                        = jpp.tag_kind == jptg_blocked;
+                        = jpp.tag_kind == jit_memory_tag_kind_t::blocked;
             }
             sse_high_half = true;
             step_high_half(ur_w, ur_bc, lpad, rpad, with_c_tail_proccessing);
