@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2020 Intel Corporation
+* Copyright 2019-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -91,10 +91,12 @@ struct jit_softmax_base_t : public jit_generator {
     Vmm vsum = Vmm(isa == avx512_common ? 30 : 14);
     Vmm vmax = Vmm(isa == avx512_common ? 31 : 15);
     Vmm vsbr = vsum; // must be not equal to vmax
+    Vmm vzeropad = Vmm(isa == avx512_common ? 21 : 11);
 
     bool is_bf16_ = false;
     bool is_softmax_ = pd_->is_softmax();
     bool is_logsoftmax_ = pd_->is_logsoftmax();
+    bool axis_is_blocked_;
 
     size_t data_type_size_ = 0;
     size_t simd_w_ = 0;
@@ -112,6 +114,8 @@ struct jit_softmax_base_t : public jit_generator {
         n_loops_ = axis_simd_full_ / unroll_regs_;
         loop_tail_ = axis_simd_full_ - n_loops_ * unroll_regs_;
         axis_stride_ = compute_axis_stride();
+        axis_is_blocked_ = (pd_->src_md(0)->padded_dims[pd_->axis()]
+                != pd_->src_md(0)->dims[pd_->axis()]);
     }
 
     size_t compute_axis_stride() {
@@ -251,7 +255,9 @@ struct jit_softmax_base_t : public jit_generator {
     }
 
     jit_softmax_base_t(const softmax_pd_t *pd)
-        : pd_(pd), data_d_(pd_->dst_md()) {
+        : jit_generator(nullptr, MAX_CODE_SIZE, true, isa)
+        , pd_(pd)
+        , data_d_(pd_->dst_md()) {
         is_bf16_ = data_d_.data_type() == data_type::bf16;
         data_type_size_ = is_bf16_ ? sizeof(bfloat16_t) : sizeof(float);
         simd_w_ = vlen / sizeof(float); // bf16 works on ymms
@@ -276,15 +282,29 @@ struct jit_softmax_t<avx512_common> : public jit_softmax_base_t<avx512_common> {
 
     void store(const Address &addr, const Vmm &vmm, bool tail = false) {
         auto effective_addr = addr;
-        if (tail) effective_addr = addr | tail_opmask;
+        Vmm src_vmm = vmm;
+
+        if (tail) {
+            if (axis_is_blocked_) {
+                src_vmm = vzeropad | tail_opmask;
+                uni_vxorps(vzeropad, vzeropad, vzeropad);
+                uni_vmovups(src_vmm, vmm);
+                src_vmm = vzeropad;
+                effective_addr = addr;
+            } else {
+                effective_addr = addr | tail_opmask;
+            }
+        }
+
         if (is_bf16_) {
             if (bf16_emu_)
-                bf16_emu_->vcvtneps2bf16(bf16_cvt_ymm, vmm);
+                bf16_emu_->vcvtneps2bf16(bf16_cvt_ymm, src_vmm);
             else
-                vcvtneps2bf16(bf16_cvt_ymm, vmm);
+                vcvtneps2bf16(bf16_cvt_ymm, src_vmm);
             vmovdqu16(effective_addr, bf16_cvt_ymm);
-        } else
-            uni_vmovups(effective_addr, vmm);
+        } else {
+            uni_vmovups(effective_addr, src_vmm);
+        }
     };
 
     void load(const Vmm &vmm, const Address &addr, bool tail = false) {
@@ -533,8 +553,15 @@ struct jit_softmax_t<avx2> : public jit_softmax_base_t<avx2> {
                         uni_vmulps(vreg_tmp_src, vreg_tmp_src, vsum);
                     if (is_logsoftmax_)
                         uni_vsubps(vreg_tmp_src, vreg_tmp_src, vsum);
-                    uni_vmovups_tail(dst_ptr(axis_stride_ * i), tail_vmask,
-                            vreg_tmp_src);
+
+                    if (axis_is_blocked_) {
+                        uni_vxorps(vzeropad, vzeropad, vzeropad);
+                        vpblendvb(vzeropad, vzeropad, vreg_tmp_src, tail_vmask);
+                        uni_vmovups(dst_ptr(axis_stride_ * i), vzeropad);
+                    } else {
+                        uni_vmovups_tail(dst_ptr(axis_stride_ * i), tail_vmask,
+                                vreg_tmp_src);
+                    }
                 }
             }
         });
