@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020 Intel Corporation
+* Copyright 2020-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -34,8 +34,7 @@ status_t gen9_softmax_fwd_t::execute_generic(const exec_ctx_t &ctx) const {
 
     auto nd_range = compute::nd_range_t(pd()->gws, pd()->lws);
 
-    status_t status = parallel_for(ctx, nd_range, kernel_, arg_list);
-    return status;
+    return parallel_for(ctx, nd_range, kernel_, arg_list);
 }
 
 } // namespace ocl
