@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2020 Intel Corporation
+* Copyright 2019-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -241,8 +241,11 @@ enum conv_version_t {
     ver_unused,
     ver_1stconv,
     ver_16mb16c,
+    ver_32mb32c,
+    ver_32c,
     ver_8ow16c,
     ver_nhwc,
+    ver_nchw,
     ver_mb_block,
     ver_ow_block
 };
@@ -261,7 +264,7 @@ struct conv_conf_t {
     int stride_d, stride_h, stride_w;
     int dilate_d, dilate_h, dilate_w;
 
-    int sp_block;
+    int sp_block, sp;
     int od_block, oh_block, ow_block;
     int id_block, ih_block, iw_block;
     int oc_block, ic_block, nchunk;
@@ -270,7 +273,8 @@ struct conv_conf_t {
     int icb;
     int ocb;
     int osp_chunk, mb_chunk, mb_block, slm_ic;
-    size_t wei_slm_size, src_slm_size;
+    int iw_tail;
+    size_t wei_slm_size, src_slm_size, dst_slm_size;
     int sub_group_size;
     size_t gws_d[3], lws_d[3];
     compute::dispatch_t dispatch;
@@ -281,6 +285,8 @@ struct conv_conf_t {
 
     bool is_depthwise;
     bool is_nhwc;
+    bool reorder_wei = false;
+    bool reorder_bias = false;
     int ver;
     format_tag_t src_tag, dst_tag, wei_tag;
     bool is_nchw;
@@ -296,9 +302,11 @@ struct conv_conf_t {
     int wino_oc;
     int wino_ic_block;
     int wino_oc_block;
+    int vect_size;
     size_t U_gws_d[3], U_lws_d[3];
     size_t V_gws_d[3], V_lws_d[3];
     size_t M_gws_d[3], M_lws_d[3];
+    bool is_fused;
 
     data_type_t src_data_type;
     data_type_t weights_data_type;
@@ -311,6 +319,8 @@ struct conv_conf_t {
 struct pool_conf_t {
     int ndims;
     int mb, c;
+    int mb_padded;
+    int c_padded;
     int id, ih, iw, od, oh, ow;
     int stride_d, stride_h, stride_w;
     int kd, kh, kw;
@@ -332,6 +342,20 @@ struct pool_conf_t {
     memory_desc_info_t dst_md_info;
 };
 
+// Prelu
+struct prelu_conf_t {
+    bool is_forward;
+    bool reduce_diff_weights;
+    compute::dispatch_t dispatch;
+
+    attr_info_t attr_info;
+    memory_desc_info_t src_md_info;
+    memory_desc_info_t wei_md_info;
+    memory_desc_info_t dst_md_info;
+    memory_desc_info_t diff_src_md_info;
+    memory_desc_info_t diff_wei_md_info;
+};
+
 // Inner Product
 struct inner_product_conf_t {
     int ndims;
@@ -463,6 +487,9 @@ struct bnorm_conf_t {
     bool fuse_norm_relu, calculate_stats, calculate_diff_stats;
     bool diff_scaleshift;
     float relu_negative_slope, eps;
+    int sub_group_size;
+    bool vectorize_calc_stats;
+    bool skip_reduce_stat;
 
     compute::dispatch_t dispatch_calc_stat;
     compute::dispatch_t dispatch_reduce_stat;
@@ -484,7 +511,10 @@ struct lnorm_conf_t {
     bool use_scaleshift;
     bool calculate_stats;
     bool save_stats;
+    bool vectorize_calc_stats;
     float eps;
+    int sub_group_size;
+    int vect_dt_n;
 
     compute::dispatch_t dispatch_scaleshift;
     compute::dispatch_t dispatch;
@@ -503,6 +533,13 @@ struct binary_conf_t {
     bool is_max;
     bool is_min;
     bool is_div;
+    bool is_sub;
+    bool is_ge;
+    bool is_gt;
+    bool is_le;
+    bool is_lt;
+    bool is_eq;
+    bool is_ne;
     bool is_tensor_op;
     compute::dispatch_t dispatch;
     int dim0[MAX_NDIMS];
@@ -519,12 +556,52 @@ struct binary_conf_t {
     attr_info_t attr_info;
 };
 
+// Reduction
+struct reduction_conf_t {
+    int ndims, power, div;
+    float eps;
+    dim_t src_dims[MAX_NDIMS], reduce_dims[MAX_NDIMS], dst_dims[MAX_NDIMS];
+    bool is_reduction_dim[MAX_NDIMS];
+
+    int initial_hwd_dim, initial_hwd_chunk_size;
+    int final_hwd_dim, final_hwd_chunk_size;
+    int initial_c_chunks, final_c_dim, final_c_chunk_size;
+    int initial_n_chunk_size, initial_n_chunks;
+    int final_n_dim, final_n_chunk_size;
+
+    bool skip_final_phase;
+    int c_block_size, n_block_size;
+    int vector_size;
+    int sub_group_size;
+    data_type_t src_type, dst_type;
+    alg_kind_t alg;
+    compute::dispatch_t dispatch;
+    compute::dispatch_t finilize_dispatch;
+    memory_desc_info_t src_md_info, dst_md_info;
+    offsets_t off;
+    attr_info_t attr_info;
+};
+
 // Reorder
+enum reorder_kernel_t {
+    reorder_reference,
+    dense_vector,
+    unroll_16b,
+    unroll_16b16c,
+    unroll_16a16b,
+    plain_to_ABcd4axb,
+    vectorize_last_dim,
+    plain_to_ABxx8ayb,
+    plain_xFxE_to_abcdef,
+    transpose8x8,
+    transpose16x16,
+    reorder_nchw,
+    unaligned_sizes
+};
 struct reorder_conf_t {
-    bool do_reorder, with_group, has_padding;
+    bool has_padding;
     bool scale_quant, with_sum_ab, with_sum_a;
-    bool use_ref_impl, use_dense_vect;
-    bool vectorize_last_dim;
+    reorder_kernel_t implementation;
     int ndims;
     size_t nelems;
 
@@ -554,6 +631,7 @@ struct concat_conf_t {
 // Elementwise
 struct eltwise_conf_t {
     int ndims;
+    int vector_size;
     bool with_zero_padding;
     data_type_t data_type;
     alg_kind_t alg;
@@ -569,15 +647,11 @@ struct eltwise_conf_t {
 struct shuffle_conf_t {
     data_type_t data_type;
     int axis;
-    int axis_size;
-    int group_size;
     int transpose_row;
     int transpose_col;
-    size_t outer_size;
-    size_t inner_size;
-    size_t dim;
-    int ndims;
-    size_t gws_d[3];
+    compute::dispatch_t dispatch;
+    memory_desc_info_t src_md_info;
+    memory_desc_info_t dst_md_info;
 };
 
 inline void set_default_pool_conf(pool_conf_t &conf,
@@ -595,6 +669,8 @@ inline void set_default_pool_conf(pool_conf_t &conf,
     conf.mb = src_dims[0];
 
     conf.c = src_dims[1];
+    conf.mb_padded = src_mdw.padded_dims()[0];
+    conf.c_padded = src_mdw.padded_dims()[1];
     conf.id = (ndims == 5) ? src_dims[2] : 1;
     conf.ih = (ndims == 3) ? 1 : src_dims[ndims - 2];
     conf.iw = src_dims[ndims - 1];
@@ -814,6 +890,13 @@ inline void def_binary_alg_kinds(compute::kernel_ctx_t &kernel_ctx) {
     kernel_ctx.define_int("BINARY_MIN", alg_kind::binary_min);
     kernel_ctx.define_int("BINARY_MAX", alg_kind::binary_max);
     kernel_ctx.define_int("BINARY_DIV", alg_kind::binary_div);
+    kernel_ctx.define_int("BINARY_SUB", alg_kind::binary_sub);
+    kernel_ctx.define_int("BINARY_GE", alg_kind::binary_ge);
+    kernel_ctx.define_int("BINARY_GT", alg_kind::binary_gt);
+    kernel_ctx.define_int("BINARY_LE", alg_kind::binary_le);
+    kernel_ctx.define_int("BINARY_LT", alg_kind::binary_lt);
+    kernel_ctx.define_int("BINARY_EQ", alg_kind::binary_eq);
+    kernel_ctx.define_int("BINARY_NE", alg_kind::binary_ne);
 }
 
 inline void def_eltwise_alg_kinds(compute::kernel_ctx_t &kernel_ctx) {
@@ -821,6 +904,8 @@ inline void def_eltwise_alg_kinds(compute::kernel_ctx_t &kernel_ctx) {
     kernel_ctx.define_int("LINEAR", alg_kind::eltwise_linear);
     kernel_ctx.define_int("BOUNDED_RELU", alg_kind::eltwise_bounded_relu);
     kernel_ctx.define_int("SOFT_RELU", alg_kind::eltwise_soft_relu);
+    kernel_ctx.define_int("LOGSIGMOID", alg_kind::eltwise_logsigmoid);
+    kernel_ctx.define_int("MISH", alg_kind::eltwise_mish);
     kernel_ctx.define_int("LOGISTIC", alg_kind::eltwise_logistic);
     kernel_ctx.define_int("TANH", alg_kind::eltwise_tanh);
     kernel_ctx.define_int("ELU", alg_kind::eltwise_elu);
@@ -832,9 +917,11 @@ inline void def_eltwise_alg_kinds(compute::kernel_ctx_t &kernel_ctx) {
     kernel_ctx.define_int("SWISH", alg_kind::eltwise_swish);
     kernel_ctx.define_int("LOG", alg_kind::eltwise_log);
     kernel_ctx.define_int("CLIP", alg_kind::eltwise_clip);
+    kernel_ctx.define_int("CLIP_V2", alg_kind::eltwise_clip_v2);
     kernel_ctx.define_int("POW", alg_kind::eltwise_pow);
     kernel_ctx.define_int("GELU_ERF", alg_kind::eltwise_gelu_erf);
     kernel_ctx.define_int("ROUND", alg_kind::eltwise_round);
+    kernel_ctx.define_int("HARDSWISH", alg_kind::eltwise_hardswish);
 
     kernel_ctx.define_int("RELU_DST", alg_kind::eltwise_relu_use_dst_for_bwd);
     kernel_ctx.define_int(
@@ -843,10 +930,12 @@ inline void def_eltwise_alg_kinds(compute::kernel_ctx_t &kernel_ctx) {
     kernel_ctx.define_int("ELU_DST", alg_kind::eltwise_elu_use_dst_for_bwd);
     kernel_ctx.define_int("SQRT_DST", alg_kind::eltwise_sqrt_use_dst_for_bwd);
     kernel_ctx.define_int("EXP_DST", alg_kind::eltwise_exp_use_dst_for_bwd);
+    kernel_ctx.define_int(
+            "CLIP_V2_DST", alg_kind::eltwise_clip_v2_use_dst_for_bwd);
 }
 
-inline bool post_ops_with_binary_ok(
-        const primitive_attr_t *attr, const data_type_t dst_dt) {
+inline bool post_ops_with_binary_ok(const primitive_attr_t *attr,
+        const data_type_t dst_dt, const int max_ndims_supported = 2) {
     const auto &p = attr->post_ops_;
 
     auto is_eltwise = [&](int idx) { return p.entry_[idx].is_eltwise(false); };
@@ -856,7 +945,16 @@ inline bool post_ops_with_binary_ok(
     bool is_po_ok = true;
     for (int po_idx = 0; po_idx < p.len(); ++po_idx) {
         is_po_ok &= is_eltwise(po_idx) | is_sum(po_idx) | is_binary(po_idx);
-
+        if (is_binary(po_idx)) {
+            const auto &bin_desc = p.entry_[po_idx].binary.src1_desc;
+            if (bin_desc.ndims > max_ndims_supported) {
+                // accept descriptor if unsupported dims are equal to 1.
+                for (int dim_idx = max_ndims_supported;
+                        dim_idx < bin_desc.ndims; ++dim_idx) {
+                    if (bin_desc.dims[dim_idx] != 1) is_po_ok = false;
+                }
+            }
+        }
         if (is_sum(po_idx)) {
             if (p.entry_[po_idx].sum.dt != dnnl_data_type_undef
                     && types::data_type_size(p.entry_[po_idx].sum.dt)
