@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2020 Intel Corporation
+* Copyright 2017-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -24,7 +24,7 @@
 #include <utility>
 #include <vector>
 
-#include "mkldnn.h"
+#include "dnnl.h"
 
 #include "common.hpp"
 
@@ -172,8 +172,10 @@ const char *skip_reason2str(skip_reason_t skip_reason) {
     CASE(CASE_NOT_SUPPORTED);
     CASE(DATA_TYPE_NOT_SUPPORTED);
     CASE(INVALID_CASE);
+    CASE(KNOWN_LIMITATION);
     CASE(NOT_ENOUGH_RAM);
     CASE(SKIP_IMPL_HIT);
+    CASE(SKIP_START);
 #undef CASE
     return "SKIP_UNKNOWN";
 }
@@ -187,6 +189,10 @@ void parse_result(
         case UNTESTED:
             if (!(bench_mode & CORR)) {
                 want_perf_report = true;
+                if (status == FAIL)
+                    bs.failed++;
+                else
+                    bs.passed++;
                 break;
             }
         case FAILED:
