@@ -1,5 +1,6 @@
 /*******************************************************************************
-* Copyright 2017-2020 Intel Corporation
+* Copyright 2017-2021 Intel Corporation
+* Copyright 2021 FUJITSU LIMITED
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -14,6 +15,8 @@
 * limitations under the License.
 *******************************************************************************/
 
+#include <cstring>
+
 #include <float.h>
 #include <math.h>
 #include <stdio.h>
@@ -21,8 +24,12 @@
 
 #include "oneapi/dnnl/dnnl.h"
 
+#if DNNL_X64
+#include "tests/cpu_x64_isa_common.hpp"
+#endif
 #include "tests/test_thread.hpp"
 
+#include "compare.hpp"
 #include "dnnl_common.hpp"
 #include "dnnl_memory.hpp"
 #include "norm.hpp"
@@ -133,14 +140,13 @@ inline int compare_dat(const prb_t *prb, data_kind_t kind, dnn_mem_t &mem_dt,
 
     const char *skind = data_kind2str(kind);
 
-    int in = 0, below = 0, above = 0;
-    int in_ok = 0, below_ok = 0, above_ok = 0;
+    int in = 0, not_a_num = 0, below = 0, above = 0;
+    int in_ok = 0, not_a_num_ok = 0, below_ok = 0, above_ok = 0;
     int non_zero = 0;
 
     // Update trh with the largest value from all eltwise post-ops
     const auto &po = prb->attr.post_ops;
     bool has_eltwise = po.eltwise_index() != -1;
-    const bool has_binary = po.binary_index() != -1;
     using pk_t = attr_t::post_ops_t::kind_t;
 
     int sum_ind = po.find(pk_t::SUM);
@@ -164,24 +170,31 @@ inline int compare_dat(const prb_t *prb, data_kind_t kind, dnn_mem_t &mem_dt,
         const float rel_diff = diff / (fabsf(fp) > FLT_MIN ? fabsf(fp) : 1);
 
         bool ok = true;
-        if (fp < f_min) {
+        if (std::isnan(fp0) && is_integral_dt(f_dt)) {
+            // XXX: if reference fp0 value is nan, allow to return anything from
+            // the library for integral target data types.
+            not_a_num += 1;
+            not_a_num_ok += true;
+        } else if (fp0 < f_min) {
             diff_norm.update(f_min, dt);
             ok = dt == f_min;
-            if (!ok && has_eltwise)
-                ok = eltwise::check_extreme_values(fp, dt, pk_t::ELTWISE_END);
-            if (!ok && has_binary)
-                ok = binary::check_extreme_values(fp, dt, pk_t::BINARY_END);
+            if (!ok) ok = compare::compare_extreme_values(fp, dt);
             below += 1;
             below_ok += ok;
-        } else if (fp > f_max) {
+        } else if (fp0 > f_max) {
             diff_norm.update(f_max, dt);
             ok = dt == f_max;
-            if (!ok && has_eltwise)
-                ok = eltwise::check_extreme_values(fp, dt, pk_t::ELTWISE_END);
-            if (!ok && has_binary)
-                ok = binary::check_extreme_values(fp, dt, pk_t::BINARY_END);
+            if (!ok) ok = compare::compare_extreme_values(fp, dt);
+            // Hack for (TODO: Intel only?) CPU saturation check.
+            if (is_cpu() && f_dt == dnnl_s32)
+                ok = dt >= BENCHDNN_S32_TO_F32_SAT_CONST && dt < f_max;
             above += 1;
             above_ok += ok;
+        } else if (fp == dt) {
+            // Quick check.
+            diff_norm.update(fp, dt);
+            in += 1;
+            in_ok += ok;
         } else {
             diff_norm.update(fp, dt);
             float trh = get_eps(prb, kind);
@@ -195,10 +208,7 @@ inline int compare_dat(const prb_t *prb, data_kind_t kind, dnn_mem_t &mem_dt,
                 }
             }
             ok = (fabs(fp) > 1e-5 ? rel_diff : diff) <= trh;
-            if (!ok && has_eltwise)
-                ok = eltwise::check_extreme_values(fp, dt, pk_t::ELTWISE_END);
-            if (!ok && has_binary)
-                ok = binary::check_extreme_values(fp, dt, pk_t::BINARY_END);
+            if (!ok) ok = compare::compare_extreme_values(fp, dt);
             in += 1;
             in_ok += ok;
         }
@@ -273,11 +283,12 @@ inline int compare_dat(const prb_t *prb, data_kind_t kind, dnn_mem_t &mem_dt,
         BENCHDNN_PRINT(0,
                 "@@@ [%s] %strust range:%.2f nz:%.2f "
                 "(level range:%.2f nz:%.2f). "
-                "in:%d (ok:%d) below:%d (ok:%d) above:%d (ok:%d) nz:%d "
-                "total:%lu\n",
+                "in:%d (ok:%d) nan:%d (ok:%d) below:%d (ok:%d) above:%d "
+                "(ok:%d) nz:%d total:%lu\n",
                 skind, final_compare ? "final: " : "", trust_rg, trust_nz,
-                trust_rg_level, trust_nz_level, in, in_ok, below, below_ok,
-                above, above_ok, non_zero, (unsigned long)res->total);
+                trust_rg_level, trust_nz_level, in, in_ok, not_a_num,
+                not_a_num_ok, below, below_ok, above, above_ok, non_zero,
+                (unsigned long)res->total);
     }
 
     if (no_trust) {
@@ -314,13 +325,14 @@ int compare_dst(const prb_t *prb, dnn_mem_t &mem_dt, dnn_mem_t &mem_fp,
 
 int fill_src(
         const prb_t *prb, dnn_mem_t &mem_dt, dnn_mem_t &mem_fp, res_t *res) {
-    const bool need_extra_mem = mem_dt.dt() != mem_fp.dt();
+    const bool check_reorder
+            = (bench_mode & CORR) && (mem_dt.dt() != mem_fp.dt());
     dnn_mem_t extra_mem;
-    if (need_extra_mem) {
+    if (check_reorder) {
         extra_mem
                 = dnn_mem_t(mem_dt.md_, dnnl_f32, tag::abx, get_test_engine());
     }
-    dnn_mem_t &mem_00 = need_extra_mem ? extra_mem : mem_fp;
+    dnn_mem_t &mem_00 = check_reorder ? extra_mem : mem_fp;
 
     const auto &c = prb->cfg[SRC];
     const int range = c.f_max - c.f_min + 1;
@@ -341,7 +353,7 @@ int fill_src(
             });
 
     SAFE(mem_dt.reorder(mem_00), WARN);
-    if (need_extra_mem) {
+    if (check_reorder) {
         SAFE(mem_fp.reorder(mem_dt), WARN);
         SAFE(compare_src(prb, mem_fp, mem_00, res), WARN);
     }
@@ -352,12 +364,31 @@ int fill_src(
 int fill_wei(
         const prb_t *prb, dnn_mem_t &mem_dt, dnn_mem_t &mem_fp, res_t *res) {
     const bool wino_s8 = prb->alg == WINO && prb->cfg[WEI].dt == dnnl_s8;
-    const bool s8_s8
-            = prb->cfg[WEI].dt == dnnl_s8 && prb->cfg[SRC].dt == dnnl_s8;
     const bool is_def_zp = prb->attr.zero_points.is_def(DNNL_ARG_SRC);
     const bool diff_data_type = mem_dt.dt() != mem_fp.dt();
-    const bool check_reorder
-            = diff_data_type && !wino_s8 && !s8_s8 && is_def_zp;
+
+    dnnl_data_type_t dt_check = dnnl_s8;
+#if DNNL_AARCH64
+    /* Note for x64:
+    Both data types of src and weight are s8, oneDNN addds 128 to one of the s8
+    input to make it of type u8 instead, as explained in
+    https://oneapi-src.github.io/oneDNN/dev_guide_int8_computations.html or
+    doc/advanced/int8_computations.md
+    It is because `VPDPBUSD` instruction uses the combination of s8 and u8 as
+    input.
+
+    Note for AArch64:
+    Because dot product instructions of AArch64 "SDOT" receives s8 input
+    for both src and weight, the addition (and its counterpart of subtraction)
+    is not required for AArch64.
+    */
+    if (res->impl_name.find("jit", 0) == 0) dt_check = dnnl_u8;
+#endif
+
+    const bool wei_x8x8
+            = prb->cfg[WEI].dt == dnnl_s8 && prb->cfg[SRC].dt == dt_check;
+    const bool check_reorder = (bench_mode & CORR) && diff_data_type && !wino_s8
+            && !wei_x8x8 && is_def_zp;
 
     dnn_mem_t extra_mem;
     if (check_reorder) {
@@ -387,17 +418,29 @@ int fill_wei(
         SAFE(mem_fp.reorder(mem_dt), WARN);
         SAFE(compare_wei(prb, mem_fp, mem_00, res), WARN);
     }
+    if ((wei_x8x8 || !is_def_zp) && is_cpu()) {
+        // Check that s8 -> s8_comp exists in the library since users may have
+        // already quantized data.
+        dnn_mem_t mem_fp_s8(mem_fp.md_, dnnl_s8, get_test_engine());
+        dnn_mem_t mem_dt_s8(mem_dt.md_, dnnl_s8, get_test_engine());
+        SAFE(mem_fp_s8.reorder(mem_fp), WARN);
+        SAFE(mem_dt_s8.reorder(mem_fp_s8), WARN);
+        SAFE(mem_dt.size() == mem_dt_s8.size() ? OK : FAIL, WARN);
+        int rc = std::memcmp((void *)mem_dt, (void *)mem_dt_s8, mem_dt.size());
+        SAFE(rc == 0 ? OK : FAIL, WARN);
+    }
 
     return OK;
 }
 
 int fill_bia(
         const prb_t *prb, dnn_mem_t &mem_dt, dnn_mem_t &mem_fp, res_t *res) {
-    const bool need_extra_mem = mem_dt.dt() != mem_fp.dt();
+    const bool check_reorder
+            = (bench_mode & CORR) && (mem_dt.dt() != mem_fp.dt());
     dnn_mem_t extra_mem;
-    if (need_extra_mem)
+    if (check_reorder)
         extra_mem = dnn_mem_t(mem_dt.md_, dnnl_f32, tag::x, get_test_engine());
-    dnn_mem_t &mem_00 = need_extra_mem ? extra_mem : mem_fp;
+    dnn_mem_t &mem_00 = check_reorder ? extra_mem : mem_fp;
 
     const size_t nelems = mem_00.nelems();
     if (nelems == 0) return OK;
@@ -415,7 +458,7 @@ int fill_bia(
     }
 
     SAFE(mem_dt.reorder(mem_00), WARN);
-    if (need_extra_mem) {
+    if (check_reorder) {
         SAFE(mem_fp.reorder(mem_dt), WARN);
         SAFE(compare_bia(prb, mem_fp, mem_00, res), WARN);
     }
@@ -425,14 +468,15 @@ int fill_bia(
 int fill_dst_with_params(const prb_t *prb, dnn_mem_t &mem_dt, dnn_mem_t &mem_fp,
         dnnl_data_type_t dt, double sparsity, int min, int max, int base,
         int step, res_t *res) {
-    const bool need_extra_mem = mem_dt.dt() != mem_fp.dt();
+    const bool check_reorder
+            = (bench_mode & CORR) && (mem_dt.dt() != mem_fp.dt());
     dnn_mem_t extra_mem;
-    if (need_extra_mem) {
+    if (check_reorder) {
         extra_mem
                 = dnn_mem_t(mem_dt.md_, dnnl_f32, tag::abx, get_test_engine());
     }
 
-    dnn_mem_t &mem_00 = need_extra_mem ? extra_mem : mem_fp;
+    dnn_mem_t &mem_00 = check_reorder ? extra_mem : mem_fp;
     const int range = max - min + 1;
 
     dnnl::impl::parallel_nd(prb->mb, prb->oc, prb->od, prb->oh, prb->ow,
@@ -447,7 +491,7 @@ int fill_dst_with_params(const prb_t *prb, dnn_mem_t &mem_dt, dnn_mem_t &mem_fp,
             });
 
     SAFE(mem_dt.reorder(mem_00), WARN);
-    if (need_extra_mem) {
+    if (check_reorder) {
         SAFE(mem_fp.reorder(mem_dt), WARN);
         SAFE(compare_dst(prb, mem_fp, mem_00, res), WARN);
     }
@@ -619,13 +663,27 @@ void check_known_skipped_case(const prb_t *prb, res_t *res) {
             {prb->cfg[SRC].dt, prb->cfg[WEI].dt, prb->cfg[DST].dt}, prb->dir,
             res);
     if (res->state == SKIPPED) return;
+    // GPU only case
+    bool is_f32_wei = prb->cfg[WEI].dt == dnnl_f32;
+    bool is_f32_src = prb->cfg[SRC].dt == dnnl_f32;
+    bool is_int8_dst = prb->cfg[DST].dt == dnnl_s8;
+    const bool f32_s8_conv = is_f32_src && is_f32_wei && is_int8_dst;
+
+    if (is_cpu() && f32_s8_conv) {
+        res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+        return;
+    }
 
     // Winograd implementation limitations.
     if (prb->alg == WINO) {
-        if (engine_tgt_kind == dnnl_cpu) {
+        if (is_cpu()) {
+#ifdef DNNL_X64
             static auto isa = dnnl_get_effective_cpu_isa();
-            static bool has_avx512_common = isa >= dnnl_cpu_isa_avx512_mic;
-            static bool has_avx512_bw = isa >= dnnl_cpu_isa_avx512_core;
+            static bool has_avx512_bw
+                    = dnnl::is_superset(isa, dnnl_cpu_isa_avx512_core);
+            static bool has_avx512_common = has_avx512_bw
+                    || dnnl::is_superset(isa, dnnl_cpu_isa_avx512_mic);
+
             bool is_int8 = prb->cfg[WEI].dt == dnnl_s8;
 
             bool pad_ok_f32 = prb->pw <= 1 && prb->ph <= 1 && prb->pw_r <= 1
@@ -669,13 +727,28 @@ void check_known_skipped_case(const prb_t *prb, res_t *res) {
                 res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
                 return;
             }
-        } else if (engine_tgt_kind == dnnl_gpu) {
+#else
+            res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+            return;
+#endif
+        } else if (is_gpu()) {
             bool shape_ok = prb->ndims == 4 && prb->g == 1 && prb->kh == 3
                     && prb->kw == 3 && prb->sh == 1 && prb->sw == 1
-                    && prb->dh == 0 && prb->dw == 0;
+                    && prb->dh == 0 && prb->dw == 0 && prb->pw < prb->kw
+                    && prb->pw_r < prb->kw && prb->ph < prb->kh
+                    && prb->ph_r < prb->kh;
             if (!shape_ok) {
                 res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
             }
+
+            const auto stag = normalize_tag(prb->stag, prb->ndims);
+            const bool stag_is_axb
+                    = stag == normalize_tag(tag::axb, prb->ndims);
+            bool is_axb_ok = (prb->ic % 16 == 0) && (prb->oc % 16 == 0);
+            if (stag_is_axb && !is_axb_ok) {
+                res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+            }
+
             return;
         } else {
             assert(!"Unknown Engine");
@@ -683,6 +756,53 @@ void check_known_skipped_case(const prb_t *prb, res_t *res) {
             return;
         }
     }
+
+    if (is_nvidia_gpu()) {
+        const int64_t ID = prb->id, IH = prb->ih, IW = prb->iw;
+        const int64_t OD = prb->od, OH = prb->oh, OW = prb->ow;
+        const int64_t KD = prb->kd, KH = prb->kh, KW = prb->kw;
+        const int64_t SD = prb->sd, SH = prb->sh, SW = prb->sw;
+        const int64_t PD = prb->pd, PH = prb->ph, PW = prb->pw;
+        const int64_t PD_R = prb->pd_r, PH_R = prb->ph_r, PW_R = prb->pw_r;
+        const bool pad_ok = PD >= PD_R && PH >= PH_R && PW >= PW_R;
+        // copy-pasted from str2desc, dilation is not supported for Nvidia
+        const auto compute_out
+                = [](int64_t i, int64_t k, int64_t s, int64_t p) {
+                      return (i - k + 2 * p) / s + 1;
+                  };
+        const bool out_ok = OD == compute_out(ID, KD, SD, PD)
+                && OH == compute_out(IH, KH, SH, PH)
+                && OW == compute_out(IW, KW, SW, PW);
+
+        const auto &po = prb->attr.post_ops;
+        bool post_ops_ok = true;
+        for (int i = 0; i < po.len(); ++i) {
+            const auto &e = po.entry[i];
+            if (e.is_sum_kind())
+                continue;
+            else if (e.is_eltwise_kind())
+                post_ops_ok = post_ops_ok && is_nvidia_eltwise_ok(prb->dir, e);
+            else if (e.is_binary_kind() || e.is_convolution_kind())
+                post_ops_ok = false;
+            else
+                assert(!"unknown post-op type");
+        }
+
+        const auto dtag = normalize_tag(prb->dtag, prb->ndims);
+        const bool dtag_is_axb = dtag == normalize_tag(tag::axb, prb->ndims);
+        const bool tag_ok = !((prb->dir & FLAG_BWD) && dtag_is_axb);
+
+        const bool is_f16_src = prb->cfg[SRC].dt == dnnl_f16;
+        const bool is_f16_wei = prb->cfg[WEI].dt == dnnl_f16;
+        const bool f16_s8_conv = is_f16_src && is_f16_wei && is_int8_dst;
+
+        // TODO: specified wtag (even for supported formats) is not working?
+        if (!pad_ok || !out_ok || !post_ops_ok || !tag_ok || f32_s8_conv
+                || f16_s8_conv) {
+            res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+            return;
+        }
+    }
 }
 
 int doit(const prb_t *prb, res_t *res) {
@@ -707,7 +827,7 @@ int doit(const prb_t *prb, res_t *res) {
     const_dnnl_primitive_desc_t const_pd;
     DNN_SAFE(dnnl_primitive_get_primitive_desc(c, &const_pd), CRIT);
 
-    if (dnn_mem_t::check_mem_size(const_pd) != OK) {
+    if (check_mem_size(const_pd) != OK) {
         DNN_SAFE_V(dnnl_primitive_destroy(c));
         return res->state = SKIPPED, res->reason = NOT_ENOUGH_RAM, OK;
     }
@@ -748,8 +868,8 @@ int doit(const prb_t *prb, res_t *res) {
     // testing time
     dnnl_primitive_t c_ref {};
 
-    if (bench_mode & CORR && engine_tgt_kind == dnnl_gpu && fast_ref_gpu
-            && // TODO: temporary disable cpu as ref for testcases with binary post-ops
+    if (bench_mode & CORR && is_gpu() && fast_ref_gpu &&
+            // TODO: temporary disable cpu as ref for testcases with binary post-ops
             prb->attr.post_ops.binary_index() == -1) {
         dnnl_primitive_desc_t cpd_ref = nullptr;
         SAFE(init_pd_custom(get_cpu_engine(), prb, cpd_ref, nullptr, fp, fp, fp,
