@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2020 Intel Corporation
+* Copyright 2017-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -109,14 +109,16 @@ dir_t str2dir(const char *str) {
 }
 
 dnnl_prop_kind_t prop2prop_kind(const dir_t dir) {
-    if (dir == FWD_D) return dnnl_forward;
+    if (dir == FWD_D) return dnnl_forward_training;
+    if (dir == FWD_I) return dnnl_forward_inference;
     if (dir == BWD_DW) return dnnl_backward;
     assert(!"unknown dir");
     return dnnl_prop_kind_undef;
 }
 
 const char *prop2str(dnnl_prop_kind_t prop) {
-    if (prop == dnnl_forward) return "FWD_D";
+    if (prop == dnnl_forward_training) return "FWD_D";
+    if (prop == dnnl_forward_inference) return "FWD_I";
     if (prop == dnnl_backward) return "BWD_DW";
     assert(!"unknown prop_kind");
     return "unknown prop_kind";
@@ -163,6 +165,7 @@ policy_t attr_t::str2policy(const std::string &str) {
     CASE(PER_DIM_0);
     CASE(PER_DIM_1);
     CASE(PER_DIM_01);
+    CASE(PER_TENSOR);
 #undef CASE
     assert(!"unknown attr_t::policy_t policy");
     return POLICY_TOTAL;
@@ -174,6 +177,7 @@ const char *attr_t::policy2str(policy_t policy) {
     if (policy == PER_DIM_0) return "per_dim_0";
     if (policy == PER_DIM_1) return "per_dim_1";
     if (policy == PER_DIM_01) return "per_dim_01";
+    if (policy == PER_TENSOR) return "per_tensor";
     assert(!"unknown attr_t::policy_t policy");
     return "unknown attr_t::policy_t policy";
 }
@@ -184,6 +188,7 @@ int attr_t::get_default_mask(policy_t policy) {
         case PER_OC:
         case PER_DIM_1: return (1 << 1);
         case PER_DIM_01: return (1 << 0) + (1 << 1);
+        case PER_TENSOR: return (1 << DNNL_MAX_NDIMS) - 1;
         case COMMON: return 0;
         default: SAFE_V(FAIL); return 0;
     }
@@ -299,17 +304,23 @@ static po_table_entry_t kind_table[] = {
         {pk_t::BRELU, "bounded_relu", dnnl_eltwise_bounded_relu},
         {pk_t::BRELU, "brelu", dnnl_eltwise_bounded_relu},
         {pk_t::CLIP, "clip", dnnl_eltwise_clip},
+        {pk_t::CLIP_V2, "clip_v2", dnnl_eltwise_clip_v2},
+        {pk_t::CLIP_V2_DST, "clip_v2_dst",
+                dnnl_eltwise_clip_v2_use_dst_for_bwd},
         {pk_t::ELU, "elu", dnnl_eltwise_elu},
         {pk_t::ELU_DST, "elu_dst", dnnl_eltwise_elu_use_dst_for_bwd},
         {pk_t::EXP, "exp", dnnl_eltwise_exp},
         {pk_t::EXP_DST, "exp_dst", dnnl_eltwise_exp_use_dst_for_bwd},
         {pk_t::GELU_ERF, "gelu_erf", dnnl_eltwise_gelu_erf},
         {pk_t::GELU_TANH, "gelu_tanh", dnnl_eltwise_gelu_tanh},
+        {pk_t::HARDSWISH, "hardswish", dnnl_eltwise_hardswish},
         {pk_t::LINEAR, "linear", dnnl_eltwise_linear},
         {pk_t::LOG, "log", dnnl_eltwise_log},
         {pk_t::LOGISTIC, "logistic", dnnl_eltwise_logistic},
         {pk_t::LOGISTIC_DST, "logistic_dst",
                 dnnl_eltwise_logistic_use_dst_for_bwd},
+        {pk_t::LOGSIGMOID, "logsigmoid", dnnl_eltwise_logsigmoid},
+        {pk_t::MISH, "mish", dnnl_eltwise_mish},
         {pk_t::POW, "pow", dnnl_eltwise_pow},
         {pk_t::RELU, "relu", dnnl_eltwise_relu},
         {pk_t::RELU_DST, "relu_dst", dnnl_eltwise_relu_use_dst_for_bwd},
@@ -326,10 +337,13 @@ static po_table_entry_t kind_table[] = {
         // binary
         {pk_t::BINARY_START, "binary_undef", dnnl_alg_kind_undef},
         {pk_t::ADD, "add", dnnl_binary_add},
-        {pk_t::DIV, "div", dnnl_binary_div},
+        {pk_t::DIV, "div", dnnl_binary_div}, {pk_t::EQ, "eq", dnnl_binary_eq},
+        {pk_t::GE, "ge", dnnl_binary_ge}, {pk_t::GT, "gt", dnnl_binary_gt},
+        {pk_t::LE, "le", dnnl_binary_le}, {pk_t::LT, "lt", dnnl_binary_lt},
         {pk_t::MAX, "max", dnnl_binary_max},
         {pk_t::MIN, "min", dnnl_binary_min},
-        {pk_t::MUL, "mul", dnnl_binary_mul},
+        {pk_t::MUL, "mul", dnnl_binary_mul}, {pk_t::NE, "ne", dnnl_binary_ne},
+        {pk_t::SUB, "sub", dnnl_binary_sub},
         {pk_t::BINARY_END, "binary_undef", dnnl_alg_kind_undef},
         // guard entry
         {pk_t::KIND_TOTAL, "kind_undef", dnnl_alg_kind_undef}};
@@ -379,12 +393,18 @@ std::vector<int> attr_t::post_ops_t::get_binary_po_masks() const {
 
 int attr_t::post_ops_t::from_str(const std::string &s) {
     *this = post_ops_t();
+
+    auto s_no_quotes = s;
+    // strip double quotes to allow command line style work from a batch file
+    if (s.front() == '\"' && s.back() == '\"')
+        s_no_quotes = s.substr(1, s.size() - 2);
+
     // "'" is mandatory as long as ";" is used as alg delimiter
-    if (s.front() != '\'' || s.back() != '\'') return FAIL;
-    if (s.size() == 2) return OK; // empty input
+    if (s_no_quotes.front() != '\'' || s_no_quotes.back() != '\'') return FAIL;
+    if (s_no_quotes.size() == 2) return OK; // empty input
 
     // strip quotes to simplify further logic
-    auto s_no_quotes = s.substr(1, s.size() - 2);
+    s_no_quotes = s_no_quotes.substr(1, s_no_quotes.size() - 2);
 
     // operate over substrings separated by ';' represeting a single post op
     // with further parsing of specific kind
@@ -655,8 +675,11 @@ std::ostream &operator<<(std::ostream &s, const attr_t &attr) {
 std::ostream &dump_global_params(std::ostream &s) {
     s << "--" << driver_name << " ";
     if (canonical) s << "--canonical=" << bool2str(canonical) << " ";
-    if (canonical || engine_tgt_kind != dnnl_cpu)
-        s << "--engine=" << engine_tgt_kind << " ";
+    if (canonical || engine_tgt_kind != dnnl_cpu) {
+        s << "--engine=" << engine_tgt_kind;
+        if (engine_index != 0) s << ":" << engine_index;
+        s << " ";
+    }
     if (canonical || fast_ref_gpu != true)
         s << "--fast-ref-gpu=" << bool2str(fast_ref_gpu) << " ";
     if (!skip_impl.empty()) s << "--skip-impl=" << skip_impl << " ";
@@ -664,6 +687,8 @@ std::ostream &dump_global_params(std::ostream &s) {
         s << "--mem-check=" << bool2str(mem_check) << " ";
     if (canonical || allow_enum_tags_only != true)
         s << "--allow-enum-tags-only=" << bool2str(allow_enum_tags_only) << " ";
+    if (canonical || hints.get() != isa_hints_t::none)
+        s << "--cpu-isa-hints=" << isa_hints_t::hints2str(hints) << " ";
 
     return s;
 }
@@ -724,6 +749,21 @@ int attr_args_t::prepare_binary_post_op_mds(
     return OK;
 }
 
+void attr_args_t::prepare_dw_post_op(const attr_t &attr,
+        dnnl_data_type_t wei_dt, dnnl_data_type_t bia_dt, const void *vals,
+        int64_t count, int mask) {
+    const int dw_idx = attr.post_ops.convolution_index();
+    if (dw_idx == -1) return;
+
+    const auto &dw = attr.post_ops.entry[dw_idx].convolution;
+    // insert output scale which applies in fused convolution
+    insert(DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_ATTR_OUTPUT_SCALES, vals, count,
+            mask, dw.oscale.runtime);
+
+    dw_entry.wei_dt = wei_dt;
+    dw_entry.bia_dt = bia_dt;
+}
+
 dnnl_primitive_attr_t create_dnnl_attr(
         const attr_t &attr, const attr_args_t &attr_args) {
     dnnl_primitive_attr_t dnnl_attr = nullptr;
@@ -784,6 +824,23 @@ dnnl_primitive_attr_t create_dnnl_attr(
             if (e.is_sum_kind()) {
                 DNN_SAFE_V(dnnl_post_ops_append_sum_v2(
                         ops, e.sum.scale, e.sum.dt));
+            } else if (e.is_convolution_kind()) {
+                const auto wei_dt = attr_args.get_dw_arg(DNNL_ARG_WEIGHTS);
+                const auto bia_dt = attr_args.get_dw_arg(DNNL_ARG_BIAS);
+
+                const auto &os_args = attr_args.get(
+                        DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_ATTR_OUTPUT_SCALES);
+                const auto scales = os_args.get_float_ptr();
+                const auto &policy = e.convolution.oscale.policy;
+                // API expects count=0 if output_scale was not set
+                const auto count = scales ? os_args.get_count(policy) : 0;
+                const auto mask = os_args.get_mask(policy);
+
+                const auto dnnl_post_ops_append_dw = e.convolution.stride == 1
+                        ? dnnl_post_ops_append_dw_k3s1p1
+                        : dnnl_post_ops_append_dw_k3s2p1;
+                DNN_SAFE_V(dnnl_post_ops_append_dw(ops, wei_dt, bia_dt,
+                        e.convolution.dst_dt, count, mask, scales));
             } else if (e.is_eltwise_kind()) {
                 DNN_SAFE_V(dnnl_post_ops_append_eltwise(ops, e.eltwise.scale,
                         e.eltwise.alg, e.eltwise.alpha, e.eltwise.beta));
@@ -1028,12 +1085,21 @@ std::string normalize_tag(const std::string &tag_, int ndims) {
     // Handle meta-tags (abx, axb, etc).
     auto pos = tag.find("x");
     if (pos != std::string::npos) {
-        // If ndims is unknown, arbitrarily use 3 dimensions.
-        int meta_ndims = (ndims == -1 ? 3 : ndims);
-        std::string cdef_tail;
-        for (int i = 0; i < meta_ndims - 2; i++)
-            cdef_tail += ('c' + i);
-        return trim_tag(tag.replace(pos, 1, cdef_tail), meta_ndims);
+        // Non-grouped tags will start `x` from `c`, but grouped will most of
+        // times start `x` from `d`.
+        char start_x = 'c';
+        for (char c = 'a' + DNNL_MAX_NDIMS - 1; c >= 'b'; c--) {
+            if (tag.find(c) != std::string::npos) {
+                start_x = c + 1;
+                break;
+            }
+        }
+        // Adjust ndims if they are not specified.
+        int meta_ndims = (ndims == -1 ? (start_x - 'a' + 1) : ndims);
+        std::string tail;
+        for (int i = 0; i < meta_ndims - (start_x - 'a'); i++)
+            tail += (start_x + i);
+        return trim_tag(tag.replace(pos, 1, tail), meta_ndims);
     }
 
     return map_tag_letters(tag);
@@ -1041,6 +1107,7 @@ std::string normalize_tag(const std::string &tag_, int ndims) {
 
 int check_tag(const std::string &tag_, bool check_enum_tags_only) {
     auto tag = normalize_tag(tag_);
+    if (tag == tag::undef || tag == tag::any) return OK;
     return check_abc_tag(tag, check_enum_tags_only);
 }
 
@@ -1053,9 +1120,10 @@ void maybe_oscale(const attr_t &attr, float &d, float *scales, int64_t oc) {
 
 void maybe_zero_point(const attr_t &attr, float &d, const int32_t *zero_points,
         int64_t c, int arg, bool opposite_zero_point) {
-    const auto &e = attr.zero_points.get(arg);
+    if (attr.zero_points.is_def()) return;
 
-    if (!attr.zero_points.is_def(arg)) {
+    const auto &e = attr.zero_points.get(arg);
+    if (!e.is_def()) {
         const int idx = e.policy == policy_t::COMMON ? 0 : c;
         const int zp_sign = opposite_zero_point ? -1 : 1;
         d -= zp_sign * zero_points[idx];
@@ -1064,10 +1132,10 @@ void maybe_zero_point(const attr_t &attr, float &d, const int32_t *zero_points,
 
 float compute_eltwise_fwd(
         pk_t kind, float src, float scale, float alpha, float beta) {
-    using namespace dnnl::impl::math;
-
     // don't compute on nan, propagate it
-    if (std::isnan(src)) return src;
+    if (std::isnan(src)) return NAN;
+
+    using namespace dnnl::impl::math;
 
     switch (kind) {
         case pk_t::RELU: return scale * relu_fwd(src, alpha);
@@ -1079,22 +1147,26 @@ float compute_eltwise_fwd(
         case pk_t::LINEAR: return scale * linear_fwd(src, alpha, beta);
         case pk_t::BRELU: return scale * bounded_relu_fwd(src, alpha);
         case pk_t::SRELU: return scale * soft_relu_fwd(src);
+        case pk_t::LOGSIGMOID: return scale * logsigmoid_fwd(src);
+        case pk_t::MISH: return scale * mish_fwd(src);
         case pk_t::LOGISTIC: return scale * logistic_fwd(src);
         case pk_t::EXP: return scale * exp_fwd(src);
         case pk_t::GELU_TANH: return scale * gelu_tanh_fwd(src);
         case pk_t::SWISH: return scale * swish_fwd(src, alpha);
         case pk_t::LOG: return scale * log_fwd(src);
         case pk_t::CLIP: return scale * clip_fwd(src, alpha, beta);
+        case pk_t::CLIP_V2: return scale * clip_v2_fwd(src, alpha, beta);
         case pk_t::POW: return scale * pow_fwd(src, alpha, beta);
         case pk_t::GELU_ERF: return scale * gelu_erf_fwd(src);
         case pk_t::ROUND: return scale * round_fwd(src);
-
+        case pk_t::HARDSWISH: return scale * hardswish_fwd(src);
         case pk_t::RELU_DST: return scale * relu_fwd(src, alpha);
         case pk_t::TANH_DST: return scale * tanh_fwd(src);
         case pk_t::ELU_DST: return scale * elu_fwd(src, alpha);
         case pk_t::SQRT_DST: return scale * sqrt_fwd(src);
         case pk_t::LOGISTIC_DST: return scale * logistic_fwd(src);
         case pk_t::EXP_DST: return scale * exp_fwd(src);
+        case pk_t::CLIP_V2_DST: return scale * clip_v2_fwd(src, alpha, beta);
 
         default: assert(!"unknown attr::post_ops::kind");
     };
@@ -1115,14 +1187,18 @@ float compute_eltwise_bwd(
         case pk_t::LINEAR: return linear_bwd(d_dst, src, alpha, beta);
         case pk_t::BRELU: return bounded_relu_bwd(d_dst, src, alpha);
         case pk_t::SRELU: return soft_relu_bwd(d_dst, src);
+        case pk_t::LOGSIGMOID: return logsigmoid_bwd(d_dst, src);
+        case pk_t::MISH: return mish_bwd(d_dst, src);
         case pk_t::LOGISTIC: return logistic_bwd(d_dst, src);
         case pk_t::EXP: return exp_bwd(d_dst, src);
         case pk_t::GELU_TANH: return gelu_tanh_bwd(d_dst, src);
         case pk_t::SWISH: return swish_bwd(d_dst, src, alpha);
         case pk_t::LOG: return log_bwd(d_dst, src);
         case pk_t::CLIP: return clip_bwd(d_dst, src, alpha, beta);
+        case pk_t::CLIP_V2: return clip_v2_bwd(d_dst, src, alpha, beta);
         case pk_t::POW: return pow_bwd(d_dst, src, alpha, beta);
         case pk_t::GELU_ERF: return gelu_erf_bwd(d_dst, src);
+        case pk_t::HARDSWISH: return hardswish_bwd(d_dst, src);
 
         case pk_t::RELU_DST: return relu_bwd_use_dst(d_dst, src, alpha);
         case pk_t::TANH_DST: return tanh_bwd_use_dst(d_dst, src);
@@ -1130,6 +1206,8 @@ float compute_eltwise_bwd(
         case pk_t::SQRT_DST: return sqrt_bwd_use_dst(d_dst, src);
         case pk_t::LOGISTIC_DST: return logistic_bwd_use_dst(d_dst, src);
         case pk_t::EXP_DST: return exp_bwd_use_dst(d_dst, src);
+        case pk_t::CLIP_V2_DST:
+            return clip_v2_bwd_use_dst(d_dst, src, alpha, beta);
 
         default: assert(!"unknown attr::post_ops::kind");
     }
@@ -1150,6 +1228,20 @@ float compute_binary(pk_t kind, float src0, float src1) {
         return MIN2(src0, src1);
     } else if (kind == pk_t::DIV) {
         return src0 / src1;
+    } else if (kind == pk_t::SUB) {
+        return src0 - src1;
+    } else if (kind == pk_t::GE) {
+        return src0 >= src1;
+    } else if (kind == pk_t::GT) {
+        return src0 > src1;
+    } else if (kind == pk_t::LE) {
+        return src0 <= src1;
+    } else if (kind == pk_t::LT) {
+        return src0 < src1;
+    } else if (kind == pk_t::EQ) {
+        return src0 == src1;
+    } else if (kind == pk_t::NE) {
+        return src0 != src1;
     } else {
         assert(!"operation not supported!");
     }
@@ -1181,37 +1273,38 @@ void maybe_post_ops(const attr_t &attr, float &val, float sum_val,
     }
 }
 
-engine_t::engine_t(dnnl_engine_kind_t engine_kind) {
-#ifdef DNNL_SYCL_DPCPP
+engine_t::engine_t(dnnl_engine_kind_t engine_kind) : is_owner_(true) {
+    size_t idx = engine_kind == dnnl_cpu ? 0 : engine_index;
+#ifdef DNNL_WITH_SYCL
     if (engine_kind == dnnl_cpu) {
         static dnnl_engine_t inst = nullptr;
-        if (!inst) DNN_SAFE_V(dnnl_engine_create(&inst, engine_kind, 0));
+        if (!inst) DNN_SAFE_V(dnnl_engine_create(&inst, engine_kind, idx));
         engine_ = inst;
     } else if (engine_kind == dnnl_gpu) {
         static dnnl_engine_t inst = nullptr;
-        if (!inst) DNN_SAFE_V(dnnl_engine_create(&inst, engine_kind, 0));
+        if (!inst) DNN_SAFE_V(dnnl_engine_create(&inst, engine_kind, idx));
         engine_ = inst;
     } else
         assert(!"unsupported engine_kind");
 #else
-    DNN_SAFE_V(dnnl_engine_create(&engine_, engine_kind, 0));
+    DNN_SAFE_V(dnnl_engine_create(&engine_, engine_kind, idx));
 #endif
 }
 
+engine_t::engine_t(dnnl_engine_t engine) : engine_(engine), is_owner_(false) {}
+
 engine_t::~engine_t() {
-#ifdef DNNL_SYCL_DPCPP
-    engine_ = NULL;
+    if (!is_owner_) return;
+#ifdef DNNL_WITH_SYCL
+    engine_ = nullptr;
 #else
     DNN_SAFE_V(dnnl_engine_destroy(engine_));
 #endif
 }
 
 stream_t::stream_t(dnnl_engine_t engine) {
-    dnnl_engine_kind_t engine_kind;
-    DNN_SAFE_V(dnnl_engine_get_kind(engine, &engine_kind));
-
 #if DNNL_CPU_THREADING_RUNTIME == DNNL_RUNTIME_THREADPOOL
-    if (engine_kind == dnnl_cpu) {
+    if (is_cpu(engine)) {
         SAFE_V(dnnl_threadpool_interop_stream_create(
                 &stream_, engine, dnnl::testing::get_threadpool()));
         return;
