@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2020 Intel Corporation
+* Copyright 2019-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -23,6 +23,7 @@
 
 #include "tests/test_thread.hpp"
 
+#include "compare.hpp"
 #include "dnnl_common.hpp"
 #include "dnnl_memory.hpp"
 
@@ -88,32 +89,6 @@ static int init_pd(dnnl_engine_t engine, const prb_t *prb,
     return OK;
 }
 
-// Check that on a given input specific alg may return NaN or inf.
-// Used in other drivers supporting eltwise post_ops.
-bool check_extreme_values(const float &a, const float &b, alg_t alg) {
-    switch (alg) {
-        case alg_t::EXP:
-        case alg_t::EXP_DST:
-        case alg_t::LOG:
-        case alg_t::POW:
-        case alg_t::SQRT:
-        case alg_t::SQRT_DST:
-        case alg_t::SQUARE:
-        // It is impossible to reliably test against reference in eltwise
-        // post-op chain since some algs may produce inf or NaN in the middle
-        // which is not expected for a standalone testing. Thus, when passing
-        // alg == ELTWISE_END, accept this fact. This alg to be used when
-        // comparing results with eltwise post-op chain.
-        case alg_t::ELTWISE_END:
-            if (std::isnan(a) && std::isnan(b)) return true;
-            if (std::isinf(a) && std::isinf(b)
-                    && std::signbit(a) == std::signbit(b))
-                return true;
-        default: break;
-    }
-    return false;
-}
-
 static bool check_abs_err(const prb_t *prb, const float &s, const float &trh) {
     const float approx_machine_eps = 2 * epsilon_dt(dnnl_f32);
     const float comp_err = approx_machine_eps / trh;
@@ -137,8 +112,11 @@ static bool check_abs_err(const prb_t *prb, const float &s, const float &trh) {
                     && fabsf(1.f + s * (1.f - v) * dg) <= 4.f * comp_err;
         }
         case alg_t::GELU_ERF: {
-            // catch catastrophic cancellation
-            // which occurs at large negative s
+            // Catch catastrophic cancellation
+            // which occurs at large negative s.
+            // Factor 2 (in bwd) is to account for the fact that error is
+            // accumulated for each summand (except the 1) when they
+            // are of the same order of magnitude.
             const float sqrt_2_over_2 = 0.707106769084930419921875f;
             const float two_over_sqrt_pi = 1.12837922573089599609375f;
             float v = s * sqrt_2_over_2;
@@ -147,7 +125,7 @@ static bool check_abs_err(const prb_t *prb, const float &s, const float &trh) {
             else
                 return fabsf(1.f + erff(v)
                                + v * two_over_sqrt_pi * expf(-v * v))
-                        <= comp_err;
+                        <= comp_err * 2;
         }
         case alg_t::TANH:
             // catch catastrophic cancellation, which occurs when err in tanh(s)
@@ -165,23 +143,30 @@ static bool check_abs_err(const prb_t *prb, const float &s, const float &trh) {
             // (10.f is magic scale for bf16)
             return (prb->dir & FLAG_FWD) && std::signbit(s)
                     && log1pf(expf(s)) <= 10.f * comp_err;
+        case alg_t::LOGSIGMOID:
+            // same situation like in SRELU
+            // in logsigmoid when s is positive
+            // results -> 0
+            return (prb->dir & FLAG_FWD) && !std::signbit(s)
+                    && log1pf(expf(-s)) <= 10.f * comp_err;
+        case alg_t::MISH:
+            // same situation like in SRELU
+            return (prb->dir & FLAG_FWD) && std::signbit(s)
+                    && s * tanh(log1pf(expf(s))) <= 10.f * comp_err;
         case alg_t::LOGISTIC:
             // when s >= 4, logistic(s) -> 0 rapidly, which leads to high
             // relative error of logistic(s) * (1 - logistic(s)) due to
             // catastrohic cancellation.
             return (prb->dir & FLAG_BWD) && !std::signbit(s)
                     && (1.f / (1.f + expf(s))) <= comp_err;
-        case alg_t::SWISH:
+        case alg_t::SWISH: {
             // catch cancellation happening when W(s) ~~ -1 in (1 + W(s))
             // formula part on backward.
+            const float alpha_s = prb->alpha * s;
             return (prb->dir & FLAG_BWD)
-                    && (prb->alpha * s
-                                    * (1.f
-                                            - 1.f
-                                                    / (1.f
-                                                            + expf(-prb->alpha
-                                                                    * s)))
+                    && (alpha_s * (1.f - 1.f / (1.f + expf(-alpha_s)))
                             <= comp_err);
+        }
         default: return false;
     }
 }
@@ -190,61 +175,40 @@ float get_eltwise_threshold(dnnl_data_type_t dt, alg_t alg, bool is_fwd) {
     // Tolerate only rounding error (1 ulp) for other than fp32 precisions.
     float trh = dt == dnnl_f32 ? 4e-6 : epsilon_dt(dt);
     // Tolerate bigger compute errors for complex algorithms.
-    bool alg_has_higher_tolerance = alg == alg_t::GELU_TANH || alg == alg_t::ELU
-            || alg == alg_t::SWISH || alg == alg_t::TANH || alg == alg_t::SRELU
-            || alg == alg_t::LOG || IMPLICATION(alg == alg_t::ELU_DST, is_fwd)
-            || IMPLICATION(alg == alg_t::TANH_DST, is_fwd);
+    const bool alg_has_higher_tolerance = alg == alg_t::GELU_TANH
+            || alg == alg_t::ELU || alg == alg_t::SWISH || alg == alg_t::TANH
+            || alg == alg_t::SRELU || alg == alg_t::LOGSIGMOID
+            || alg == alg_t::MISH || alg == alg_t::LOG
+            || ((alg == alg_t::ELU_DST || alg == alg_t::TANH_DST) && is_fwd);
     if (dt == dnnl_f32 && alg_has_higher_tolerance) trh = 4e-5;
     return trh;
 }
 
-static int compare(const prb_t *prb, const dnn_mem_t &mem_arg_fp,
-        const dnn_mem_t &mem_fp, const dnn_mem_t &mem_dt, res_t *res) {
-    const auto nelems = mem_dt.nelems();
-    if (nelems == 0) return res->state = PASSED, OK;
-
-    res->total = nelems;
-
-    const float trh
-            = get_eltwise_threshold(prb->dt, prb->alg, prb->dir & FLAG_FWD);
-
-    for (int64_t i = 0; i < nelems; i++) {
-        const float dt = mem_dt.get_elem(i);
-        const float src = mem_arg_fp.get_elem(i);
-        const float fp0 = mem_fp.get_elem(i);
-        const float fp = round_to_nearest_representable(prb->dt, fp0);
-
-        const float diff = fabsf(fp - dt);
-        const float rel_diff = diff / (fabsf(fp) > FLT_MIN ? fabsf(fp) : 1);
-
-        bool ok = (fabsf(fp) > 1e-5 ? rel_diff : diff) <= trh;
-
-        if (!ok) ok = check_extreme_values(fp, dt, prb->alg);
-
-        if (!ok && check_abs_err(prb, src, trh)) ok = diff <= trh;
-
-        res->errors += !ok;
-
-        const bool dump = false || (!ok && (res->errors < 10 || verbose >= 10))
-                || (verbose >= 50 && i < 30) || (verbose >= 99);
-        if (dump) {
-            std::stringstream ss;
-            dims_t dims_idx = off2dims_idx(prb->dims, i);
-            ss << dims_idx;
-            std::string ind_str = ss.str();
-
-            BENCHDNN_PRINT(0,
-                    "[%4ld][%s] src:% 9.6g fp0:% 9.6g fp:% 9.6g dt:% 9.6g "
-                    "diff:%8.3g rdiff:%8.3g\n",
-                    (long)i, ind_str.c_str(), src, fp0, fp, dt, diff, rel_diff);
-        }
+static float get_eltwise_zero_trust_percent(const prb_t *prb) {
+    float ztp = 60.f; // default for eltwise due to filling.
+    switch (prb->alg) {
+        case alg_t::LINEAR:
+            if (prb->alpha == 0) ztp = 100.f;
+            break;
+        case alg_t::BRELU:
+            if ((prb->alpha == 0) || (prb->dir & FLAG_BWD)) ztp = 100.f;
+            break;
+        case alg_t::CLIP:
+        case alg_t::CLIP_V2:
+        case alg_t::CLIP_V2_DST:
+            if ((prb->alpha == 0 && prb->beta == 0) || (prb->dir & FLAG_BWD))
+                ztp = 100.f;
+            break;
+        case alg_t::POW:
+            if (prb->alpha == 0 || ((prb->dir & FLAG_BWD) && prb->beta == 0))
+                ztp = 100.f;
+            break;
+        default: break;
     }
-
-    if (res->errors) res->state = FAILED;
-
-    if (res->state == UNTESTED) res->state = PASSED; /* optimism */
-
-    return res->state == FAILED ? FAIL : OK;
+    // Integral data types with small float values will produce most zeros.
+    // u8 with negative alpha will produce only zeros.
+    if (is_integral_dt(prb->dt)) ztp = 100.f;
+    return ztp;
 }
 
 int fill_data(const prb_t *prb, data_kind_t kind, dnn_mem_t &mem_dt,
@@ -278,8 +242,9 @@ int fill_data(const prb_t *prb, data_kind_t kind, dnn_mem_t &mem_dt,
         std::uniform_real_distribution<> fgen(0.f, 0.09f);
 
         for (int64_t idx = idx_start; idx < idx_end; ++idx) {
+            static constexpr int64_t num_of_generation_variants = 13;
             float value = FLT_MAX;
-            switch (idx % 8) {
+            switch (idx % num_of_generation_variants) {
                 case 0: value = (float)igen(msr); break; // [0-10] pos
                 case 1: value = -(float)igen(msr); break; // [0-10] neg
                 case 2: value = fgen(msr); break; // [0.-0.1) pos
@@ -288,6 +253,17 @@ int fill_data(const prb_t *prb, data_kind_t kind, dnn_mem_t &mem_dt,
                 case 5: value = -10 * (float)igen(msr); break; // [0-100] neg
                 case 6: value = 10.f * fgen(msr); break; // [0.-1.) pos
                 case 7: value = -10.f * fgen(msr); break; // [0.-1.) neg
+                case 8:
+                    value = 88.f + 10.f * fgen(msr);
+                    break; // values close to logf(FLT_MAX) for exp alg testing
+                case 9:
+                    value = 22.f + 10.f * fgen(msr);
+                    break; // values close to logf(FLT_MAX)/4.0 for bwd mish alg testing
+                case 10:
+                    value = 44.f + 10.f * fgen(msr);
+                    break; // values close to logf(FLT_MAX)/2.0 for fwd mish alg testing
+                case 11: value = prb->alpha; break; // `x = alpha` corner cases
+                case 12: value = prb->beta; break; // `x = beta` corner cases
             }
             value = round_to_nearest_representable(prb->dt, value);
 
@@ -310,7 +286,9 @@ void check_known_skipped_case(const prb_t *prb, res_t *res) {
 
     bool is_invalid = false;
     switch (prb->alg) {
-        case alg_t::CLIP: is_invalid = prb->beta < prb->alpha; break;
+        case alg_t::CLIP:
+        case alg_t::CLIP_V2:
+        case alg_t::CLIP_V2_DST: is_invalid = prb->beta < prb->alpha; break;
         case alg_t::BRELU:
         case alg_t::ELU_DST:
         case alg_t::RELU_DST: is_invalid = prb->alpha < 0; break;
@@ -323,6 +301,14 @@ void check_known_skipped_case(const prb_t *prb, res_t *res) {
         res->state = SKIPPED, res->reason = INVALID_CASE;
         return;
     }
+
+    if (is_nvidia_gpu()) {
+        if (!is_nvidia_eltwise_ok(prb->dir, prb->alg, prb->alpha)
+                || !prb->attr.post_ops.is_def()) {
+            res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+            return;
+        }
+    }
 }
 
 int doit(const prb_t *prb, res_t *res) {
@@ -338,7 +324,7 @@ int doit(const prb_t *prb, res_t *res) {
     const_dnnl_primitive_desc_t const_pd;
     DNN_SAFE(dnnl_primitive_get_primitive_desc(e, &const_pd), CRIT);
 
-    if (dnn_mem_t::check_mem_size(const_pd) != OK) {
+    if (check_mem_size(const_pd) != OK) {
         DNN_SAFE_V(dnnl_primitive_destroy(e));
         return res->state = SKIPPED, res->reason = NOT_ENOUGH_RAM, OK;
     }
@@ -348,19 +334,18 @@ int doit(const prb_t *prb, res_t *res) {
                 const_pd, dnnl_query_exec_arg_md, index);
     };
 
-    const auto &data_md = q(DNNL_ARG_SRC);
+    const bool is_fwd = prb->dir & FLAG_FWD;
+    const auto &src_md = q(DNNL_ARG_SRC);
+    const auto &dst_md = q(DNNL_ARG_DST);
+    const auto &data_md = !is_fwd && prb->use_dst() ? dst_md : src_md;
     const auto &scratchpad_md = q(DNNL_ARG_SCRATCHPAD);
-
-    const auto fp = dnnl_f32;
-    const auto tag = tag::abx;
-
     const auto &test_engine = get_test_engine();
 
-    dnn_mem_t src_fp(data_md, fp, tag, test_engine);
+    dnn_mem_t src_fp(data_md, dnnl_f32, tag::abx, test_engine);
     dnn_mem_t src_dt(data_md, test_engine);
 
     // we need src_fp for proper comparison, => no in-place reference
-    dnn_mem_t dst_fp(data_md, fp, tag, test_engine);
+    dnn_mem_t dst_fp(data_md, dnnl_f32, tag::abx, test_engine);
     dnn_mem_t placeholder_dst_dt;
     if (!prb->inplace) { placeholder_dst_dt = dnn_mem_t(data_md, test_engine); }
     dnn_mem_t &dst_dt = prb->inplace ? src_dt : placeholder_dst_dt;
@@ -378,6 +363,27 @@ int doit(const prb_t *prb, res_t *res) {
 
     args_t args;
 
+    dnn_mem_t &arg_fp = !is_fwd && prb->use_dst() ? dst_fp : src_fp;
+
+    // Shouldn't be defined inside since not available when `eltwise_add_check`
+    // is invoked due to removed from stack.
+    const float trh = get_eltwise_threshold(prb->dt, prb->alg, is_fwd);
+    compare::compare_t cmp;
+    if (bench_mode & CORR) {
+        cmp.set_threshold(trh);
+        cmp.set_zero_trust_percent(get_eltwise_zero_trust_percent(prb));
+
+        const auto eltwise_add_check = [&](int64_t i, float got, float diff) {
+            // Some algorithms require absolute value comparison for inputs
+            // where catastrophic cancellation may happen.
+            const float src = arg_fp.get_elem(i);
+            if (check_abs_err(prb, src, trh)) return diff <= trh;
+            if (prb->attr.post_ops.binary_index() != -1) return diff <= trh;
+            return false;
+        };
+        cmp.set_driver_check_function(eltwise_add_check);
+    }
+
     if (prb->dir & FLAG_FWD) {
         args.set(DNNL_ARG_SRC, src_dt);
         args.set(DNNL_ARG_DST, dst_dt);
@@ -388,13 +394,13 @@ int doit(const prb_t *prb, res_t *res) {
 
         if (bench_mode & CORR) {
             compute_ref_fwd(prb, src_fp, binary_po_fp, dst_fp);
-            dnn_mem_t dst(dst_dt, fp, tag, test_engine);
-            SAFE(compare(prb, src_fp, dst_fp, dst, res), WARN);
+            SAFE(cmp.compare(dst_fp, dst_dt, prb->attr, res), WARN);
         }
     } else {
         const auto &d_data_md = q(DNNL_ARG_DIFF_DST);
 
-        dnn_mem_t d_dst_fp = dnn_mem_t(d_data_md, fp, tag, test_engine);
+        dnn_mem_t d_dst_fp
+                = dnn_mem_t(d_data_md, dnnl_f32, tag::abx, test_engine);
         d_dst_dt = dnn_mem_t(d_data_md, test_engine);
 
         dnn_mem_t &d_src_fp = d_dst_fp; // in-place reference
@@ -416,7 +422,7 @@ int doit(const prb_t *prb, res_t *res) {
             // make dst_fp of same values as for bf16, otherwise there are high
             // relative and absolute errors due to initial difference in source
             // values which become worse particularly when (1 - x) is used.
-            SAFE(dst_fp.reorder(dst_dt), WARN);
+            if (dst_dt.dt() != dst_fp.dt()) SAFE(dst_fp.reorder(dst_dt), WARN);
             args.set(DNNL_ARG_DST, dst_dt);
         } else {
             args.set(DNNL_ARG_SRC, src_dt);
@@ -424,10 +430,8 @@ int doit(const prb_t *prb, res_t *res) {
         SAFE(execute_and_wait(e, args), WARN);
 
         if (bench_mode & CORR) {
-            dnn_mem_t &arg_fp = prb->use_dst() ? dst_fp : src_fp;
             compute_ref_bwd(prb, arg_fp, d_dst_fp, d_src_fp);
-            dnn_mem_t d_src(d_src_dt, fp, tag, test_engine);
-            SAFE(compare(prb, arg_fp, d_src_fp, d_src, res), WARN);
+            SAFE(cmp.compare(d_src_fp, d_src_dt, prb->attr, res), WARN);
         }
     }
 
