@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2020 Intel Corporation
+* Copyright 2017-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -14,11 +14,13 @@
 * limitations under the License.
 *******************************************************************************/
 
-#include <cmath>
+#include <cstring>
+
 #include <stdlib.h>
 
 #include "oneapi/dnnl/dnnl.h"
 
+#include "compare.hpp"
 #include "dnn_types.hpp"
 #include "dnnl_common.hpp"
 #include "dnnl_memory.hpp"
@@ -40,11 +42,38 @@ int get_n_scales(const prb_t *prb) {
     }
 }
 
-int fill_memory(const prb_t *prb, data_kind_t kind, dnn_mem_t &mem) {
+// Filling for integers is different due to problematic int -> float conversion.
+// And it doesn't require many different points to be tested.
+int fill_memory_int(const prb_t *prb, data_kind_t kind, dnn_mem_t &mem) {
     const dt_conf_t c_src = prb->conf_in;
-    const auto dt = c_src->dt;
-    const int range = c_src->range;
-    const int max = c_src->min + range - 1;
+    const auto dt = mem.dt();
+    const int max = c_src->max;
+    const auto nelems = mem.nelems();
+
+    for (int64_t idx = 0; idx < nelems; ++idx) {
+        const int gen[4] = {
+                (int)max, /* saturate to max of output data type */
+                (int)c_src->min, /* saturate to min of output data type */
+                (int)0,
+                (int)16,
+        };
+
+        const int rng = kind == SRC ? (idx % 4) : ((idx * 5 / 4) % 4);
+        const int value = gen[rng];
+        switch (dt) {
+            case dnnl_s32: ((int *)mem)[idx] = value; break;
+            case dnnl_s8: ((int8_t *)mem)[idx] = (int8_t)value; break;
+            case dnnl_u8: ((uint8_t *)mem)[idx] = (uint8_t)value; break;
+            default: assert(!"not expected"); break;
+        }
+    }
+
+    return OK;
+}
+
+int fill_memory_fp(const prb_t *prb, data_kind_t kind, dnn_mem_t &mem) {
+    const dt_conf_t c_src = prb->conf_in;
+    const int max = c_src->max;
     const int scale_mask = attr_t::get_default_mask(prb->attr.oscale.policy);
 
     const auto nelems = mem.nelems();
@@ -58,18 +87,23 @@ int fill_memory(const prb_t *prb, data_kind_t kind, dnn_mem_t &mem) {
                 (float)c_src->min, /* saturate to min of output data type */
                 (float)1.6 / scale, /* rounding check */
                 (float)0.2 / scale, /* saturate to 0 */
-                (float)1.0,
+                (float)1.0 / scale, /* exact multiplication check */
                 (float)2.0,
                 (float)scale,
         };
 
         const int rng = kind == SRC ? (idx % 7) : ((idx * 8 / 7) % 7);
-        mem.set_elem(idx, round_to_nearest_representable(dt, gen[rng]));
+        mem.set_elem(idx, gen[rng]);
     }
 
     return OK;
 }
 
+int fill_memory(const prb_t *prb, data_kind_t kind, dnn_mem_t &mem) {
+    if (is_integral_dt(mem.dt())) return fill_memory_int(prb, kind, mem);
+    return fill_memory_fp(prb, kind, mem);
+}
+
 int fill_memory_extra(const prb_t *prb, dnnl_memory_extra_desc_t &extra) {
     extra.flags = dnnl_memory_extra_flag_none;
 
@@ -110,7 +144,10 @@ int ref_reorder(const prb_t *prb, dnn_mem_t &dst, const dnn_mem_t &src) {
 
         const int64_t scale_idx = dst.get_scale_idx(idx, scale_mask);
         const float alpha = prb->scales[scale_idx];
-        const float value = alpha * s + beta * d + dst_zero_point;
+        float value = alpha * s + beta * d + dst_zero_point;
+        value = maybe_saturate(dst_dt, value);
+        if (dst_dt == dnnl_s32 && value >= (float)INT_MAX)
+            value = BENCHDNN_S32_TO_F32_SAT_CONST;
 
         dst.set_elem(idx, round_to_nearest_representable(dst_dt, value));
     }
@@ -125,7 +162,7 @@ int compare_bootstrap(dnn_mem_t &mem_ref, dnn_mem_t &mem_got, res_t *res) {
     if (size_ref == 0) return res->state = PASSED, OK;
 
     if (size_ref == mem_got.size())
-        ok = !memcmp((void *)mem_ref, (void *)mem_got, size_ref);
+        ok = !std::memcmp((void *)mem_ref, (void *)mem_got, size_ref);
 
     res->errors = !ok;
     res->state = ok ? PASSED : FAILED;
@@ -134,100 +171,6 @@ int compare_bootstrap(dnn_mem_t &mem_ref, dnn_mem_t &mem_got, res_t *res) {
     return res->state == FAILED ? FAIL : OK;
 }
 
-static int compare(const prb_t *prb, const dnn_mem_t &mem_ref,
-        const dnn_mem_t &mem_got, res_t *res) {
-    const auto nelems = mem_got.nelems();
-    if (nelems == 0) return res->state = PASSED, OK;
-
-    res->total = nelems;
-
-    int64_t inf_p = 0, inf_n = 0, zeros = 0, reg = 0;
-
-    const auto dt_out = mem_ref.dt();
-    const size_t width = mem_ref.sizeof_dt() * 8;
-    const float dt_out_min
-            = dt_out == dnnl_u8 ? 0.f : -(float)(1l << (width - 1));
-    const float dt_out_max
-            = dt_out == dnnl_u8 ? 255.f : (float)((1l << (width - 1)) - 1);
-    const float tolerance = (dt_out == dnnl_bf16)
-            ? 4e-3 // due to bf16 truncation (7th mantissa bit -> 1/129)
-            : 0.;
-
-    for (int64_t i = 0; i < nelems; i++) {
-        const float dt = mem_got.get_elem(i);
-        const float fp = mem_ref.get_elem(i);
-
-        if (fp == dt_out_max)
-            inf_p++;
-        else if (fp == dt_out_min)
-            inf_n++;
-        else if (fp == 0.0)
-            zeros++;
-        else
-            reg++;
-
-        const float diff = fabsf(fp - dt);
-        const float rel_diff = diff / (fabsf(fp) > FLT_MIN ? fabsf(fp) : 1);
-        bool ok = rel_diff <= tolerance;
-
-        // f32->f16 results in inf for FLT_MAX input
-        if (!ok) ok = std::isinf(fp) && std::isinf(dt);
-
-        res->errors += !ok;
-
-        const bool dump = false || (!ok && (res->errors < 10 || verbose >= 10))
-                || (verbose >= 50 && i < 30) || (verbose >= 99);
-        if (dump) {
-            std::stringstream ss;
-            dims_t dims_idx = off2dims_idx(prb->reorder.dims, i);
-            ss << dims_idx;
-            std::string ind_str = ss.str();
-
-            BENCHDNN_PRINT(0,
-                    "[%4ld][%s] fp:% 12.6g dt:% 12.6g diff:%8.3g rdiff:%8.3g\n",
-                    (long)i, ind_str.c_str(), fp, dt, diff, rel_diff);
-        }
-    }
-
-    if (res->errors) res->state = FAILED;
-
-    if (res->state == UNTESTED) res->state = PASSED; /* optimism */
-
-    if (res->state != FAILED) {
-        float max_scale = prb->scales[0];
-        for (int i = 1; i < get_n_scales(prb); ++i)
-            max_scale = MAX2(max_scale, prb->scales[i]);
-
-        dt_conf_t c_src = prb->conf_in;
-        dt_conf_t c_dst = prb->conf_out;
-        const int c_src_max = c_src->min + c_src->range - 1;
-        const int c_dst_max = c_dst->min + c_dst->range - 1;
-
-        bool check_int_overflow = (dt_out != dnnl_f32 && dt_out != dnnl_f16
-                && dt_out != dnnl_bf16);
-        bool check_inf_p = (check_int_overflow && dt_out != dnnl_s32)
-                && (c_src_max * max_scale > c_dst_max);
-        bool check_inf_n = (check_int_overflow && dt_out != dnnl_s32)
-                && (c_src->min * max_scale < c_dst->min);
-        bool check_zeros = (check_int_overflow)
-                && (dt_out_min != 0 && dt_out_max != 0)
-                && IMPLICATION(prb->src_zp, prb->src_zp[0] == 0)
-                && IMPLICATION(prb->dst_zp, prb->dst_zp[0] == 0)
-                && prb->attr.post_ops.find(attr_t::post_ops_t::kind_t::SUM)
-                        == -1;
-
-        bool mistrusted = (check_inf_p && inf_p == 0)
-                || (check_inf_n && inf_n == 0) || (check_zeros && zeros == 0);
-
-        bool expect_regular = max_scale < 2e9 || dt_out == dnnl_f32;
-        if (expect_regular) mistrusted = mistrusted || reg == 0;
-
-        if (mistrusted) res->state = MISTRUSTED;
-    }
-
-    return res->state == FAILED ? FAIL : OK;
-}
-
 static int init_pd(dnnl_engine_t engine, const prb_t *prb,
         dnnl_primitive_desc_t &rpd, res_t *res, dir_t dir,
         const_dnnl_primitive_desc_t hint) {
@@ -249,7 +192,7 @@ static int init_pd(dnnl_engine_t engine, const prb_t *prb,
     dst_d.extra = dst_md_extra;
 
     dnnl_engine_t src_engine = engine, dst_engine = engine;
-    if (engine_tgt_kind == dnnl_gpu) {
+    if (is_gpu()) {
         switch (prb->cross_engine) {
             case CPU2GPU: src_engine = get_cpu_engine(); break;
             case GPU2CPU: dst_engine = get_cpu_engine(); break;
@@ -271,7 +214,15 @@ static int init_pd(dnnl_engine_t engine, const prb_t *prb,
     SAFE(init_status, WARN);
 
     res->impl_name = query_impl_info(rpd);
-    BENCHDNN_PRINT(5, "oneDNN implementation: %s\n", res->impl_name.c_str());
+    if (maybe_skip(res->impl_name)) {
+        BENCHDNN_PRINT(2, "SKIPPED: oneDNN implementation: %s\n",
+                res->impl_name.c_str());
+        DNN_SAFE(dnnl_primitive_desc_destroy(rpd), WARN);
+        return res->state = SKIPPED, res->reason = SKIP_IMPL_HIT, OK;
+    } else {
+        BENCHDNN_PRINT(
+                5, "oneDNN implementation: %s\n", res->impl_name.c_str());
+    }
 
     return OK;
 }
@@ -300,7 +251,7 @@ void check_known_skipped_case(const prb_t *prb, res_t *res) {
     }
 
     // bf16 reorder on cpu supports only bf16/f32 src_dt/dst_dt
-    if (engine_tgt_kind == dnnl_cpu
+    if (is_cpu()
             && (!IMPLICATION(sdt == dnnl_bf16,
                         ddt == dnnl_f32 || ddt == dnnl_bf16 || ddt == dnnl_s8
                                 || ddt == dnnl_u8)
@@ -311,7 +262,15 @@ void check_known_skipped_case(const prb_t *prb, res_t *res) {
         return;
     }
 
-    if (engine_tgt_kind == dnnl_gpu) {
+    if (is_nvidia_gpu()) {
+        const bool oscale_ok = prb->attr.oscale.policy == policy_t::COMMON;
+        if (!oscale_ok) {
+            res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+            return;
+        }
+    }
+
+    if (is_gpu()) {
         // GPU does not support run-time dims and zero-points
         if (prb->runtime_dim_mask != 0 || !prb->attr.zero_points.is_def()
                 || prb->attr.oscale.runtime) {
@@ -360,7 +319,7 @@ int doit(const prb_t *prb, res_t *res) {
     const_dnnl_primitive_desc_t const_pd;
     DNN_SAFE(dnnl_primitive_get_primitive_desc(rp, &const_pd), CRIT);
 
-    if (dnn_mem_t::check_mem_size(const_pd) != OK) {
+    if (check_mem_size(const_pd) != OK) {
         DNN_SAFE_V(dnnl_primitive_destroy(rp));
         return res->state = SKIPPED, res->reason = NOT_ENOUGH_RAM, OK;
     }
@@ -413,7 +372,9 @@ int doit(const prb_t *prb, res_t *res) {
     /* Step 5: execute necessary reorders */
     SAFE(src_dt_in_fmt_in.reorder(src_dt_in_fmt_ref), WARN);
 
-    if (prb->attr.post_ops.find(attr_t::post_ops_t::kind_t::SUM) >= 0) {
+    const bool has_sum
+            = prb->attr.post_ops.find(attr_t::post_ops_t::kind_t::SUM) >= 0;
+    if (has_sum) {
         SAFE(fill_memory(prb, DST, dst_dt_out_fmt_ref), WARN);
         SAFE(dst_dt_out_fmt_out.reorder(dst_dt_out_fmt_ref), WARN);
     }
@@ -462,9 +423,37 @@ int doit(const prb_t *prb, res_t *res) {
             SAFE(ref_reorder(prb, dst_dt_out_fmt_ref, src_dt_in_fmt_ref), WARN);
 
             /* Step 5c: compare benchdnn and oneDNN output */
-            dnn_mem_t dst_dt_out(dst_md, dst_dt, tag, dst_engine);
-            SAFE(dst_dt_out.reorder(dst_dt_out_fmt_out), WARN);
-            SAFE(compare(prb, dst_dt_out_fmt_ref, dst_dt_out, res), WARN);
+            compare::compare_t cmp;
+            const bool has_s32 = src_md.data_type == dnnl_s32
+                    || dst_md.data_type == dnnl_s32;
+            const bool has_s8 = src_md.data_type == dnnl_s8
+                    || dst_md.data_type == dnnl_s8;
+            const bool has_u8 = src_md.data_type == dnnl_u8
+                    || dst_md.data_type == dnnl_u8;
+            if (has_u8)
+                cmp.set_zero_trust_percent(58.f); // 4/7 inputs becomes 0
+            else if (has_s32 || has_s8)
+                cmp.set_zero_trust_percent(43.f); // 3/7 inputs becomes 0
+
+            // A hack to avoid false-positive result from f32->s32 conversion
+            // in case of sum post-op on GPU happening when two max_dt values
+            // are summed together.
+            const auto reorder_add_check
+                    = [&](int64_t i, float got, float diff) {
+                          if (has_sum && dst_dt == dnnl_s32
+                                  && got == max_dt(dnnl_s32) && is_gpu()) {
+                              // 128.f = float(INT_MAX)
+                              //                - BENCHDNN_S32_TO_F32_SAT_CONST;
+                              return diff == 128.f;
+                          }
+                          return false;
+                      };
+            cmp.set_driver_check_function(reorder_add_check);
+
+            // TODO: enable additional checks for border values validity.
+            SAFE(cmp.compare(dst_dt_out_fmt_ref, dst_dt_out_fmt_out, prb->attr,
+                         res, dst_engine),
+                    WARN);
         }
     }
 
