@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2020 Intel Corporation
+* Copyright 2016-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -32,7 +32,6 @@
 #define collapse(x)
 #endif
 
-#include "dnnl_test_macros.hpp"
 #include "oneapi/dnnl/dnnl.hpp"
 #include "oneapi/dnnl/dnnl_debug.h"
 
@@ -40,14 +39,17 @@
 #include "oneapi/dnnl/dnnl_threadpool.hpp"
 #endif
 
-#if DNNL_GPU_RUNTIME == DNNL_RUNTIME_OCL || DNNL_WITH_SYCL
+#if DNNL_GPU_RUNTIME == DNNL_RUNTIME_OCL || defined(DNNL_WITH_SYCL)
 #include "dnnl_test_common_ocl.hpp"
 #endif
 
-#if DNNL_WITH_SYCL
+#ifdef DNNL_WITH_SYCL
 #include "oneapi/dnnl/dnnl_sycl.hpp"
 #endif
 
+// Don't move it higher than library public headers
+#include "dnnl_test_macros.hpp"
+
 #include "src/common/bfloat16.hpp"
 #include "src/common/float16.hpp"
 #include "src/common/memory_desc_wrapper.hpp"
@@ -76,6 +78,13 @@ using dnnl::impl::f16_support::float16_t;
     } while (0)
 #endif
 
+// XXX: Using EXPECT_NE in 'if' statement raises a warning when GCC compiler is
+// used: suggest explicit braces to avoid ambiguous 'else'
+#define GTEST_EXPECT_NE(val1, val2) \
+    do { \
+        EXPECT_NE(val1, val2); \
+    } while (0)
+
 using memory = dnnl::memory;
 
 bool is_current_test_failed();
@@ -85,6 +94,27 @@ dnnl::engine::kind get_test_engine_kind();
 dnnl::engine get_test_engine();
 #endif
 
+inline int get_vendor_id(const std::string &vendor) {
+    if (vendor == "nvidia") {
+        return 0x10DE;
+    } else if (vendor == "intel") {
+        return 0x8086;
+    } else {
+        return -1;
+    }
+}
+
+inline bool is_nvidia_gpu(const dnnl::engine &eng) {
+#ifdef DNNL_WITH_SYCL
+    const int nvidia_vendor_id = get_vendor_id("nvidia");
+    const auto device = dnnl::sycl_interop::get_device(eng);
+    const auto eng_vendor_id
+            = device.get_info<cl::sycl::info::device::vendor_id>();
+    return eng_vendor_id == nvidia_vendor_id;
+#endif
+    return false;
+}
+
 inline bool unsupported_data_type(memory::data_type dt, dnnl::engine eng) {
     dnnl::engine::kind kind = eng.get_kind();
 
@@ -92,7 +122,16 @@ inline bool unsupported_data_type(memory::data_type dt, dnnl::engine eng) {
     if (kind == dnnl::engine::kind::cpu)
         supported = dnnl::impl::cpu::platform::has_data_type_support(
                 memory::convert_to_c(dt));
-
+#ifdef DNNL_SYCL_CUDA
+    if (is_nvidia_gpu(eng)) {
+        switch (dt) {
+            case memory::data_type::f32: return false;
+            case memory::data_type::f16: return false;
+            case memory::data_type::s8: return false;
+            default: return true;
+        }
+    }
+#endif
     return !supported;
 }
 
@@ -521,7 +560,7 @@ bool catch_expected_failures(const F &f, bool expect_to_fail,
                 // Print unimplemented but do not treat as error
                 std::cout << "[  UNIMPL  ] "
                           << "Implementation not found" << std::endl;
-                reset_malloc_counter();
+                reset_failed_malloc_counter();
                 return true;
             } else if (test_out_of_memory()
                     && (e.status == dnnl_out_of_memory
@@ -533,7 +572,7 @@ bool catch_expected_failures(const F &f, bool expect_to_fail,
                 // gemm_pack_storage_shell_t ctor makes it unable to use the
                 // reference RNN impl, and the iterator produces an
                 // `dnnl_unimplemented` error.
-                increment_malloc_counter();
+                increment_failed_malloc_counter();
                 return catch_expected_failures(f, expect_to_fail,
                         expected_status, ignore_unimplemented);
             } else {
@@ -548,7 +587,7 @@ bool catch_expected_failures(const F &f, bool expect_to_fail,
         // Return normally if the failure is expected. Reset failed malloc
         // counter to zero before performing a new test.
         if (expect_to_fail) {
-            reset_malloc_counter();
+            reset_failed_malloc_counter();
             return true;
         }
     }
@@ -562,7 +601,7 @@ bool catch_expected_failures(const F &f, bool expect_to_fail,
     }
 
     // Reset failed malloc counter to zero before performing a new test.
-    reset_malloc_counter();
+    reset_failed_malloc_counter();
     return false;
 }
 
@@ -767,18 +806,10 @@ void test_fwd_pd_attr_zp(
 template <typename op_desc_t, typename pd_t>
 void test_fwd_pd_attr_scales(
         const op_desc_t &op_desc, const engine &eng, bool supports_scales) {
-    using dt_t = memory::data_type;
-
     dnnl::primitive_attr attr_scales;
     attr_scales.set_scales(DNNL_ARG_SRC, 0, {2.f});
 
-    pd_t new_pd(op_desc, eng);
-    const auto dt = new_pd.src_desc().data.data_type;
-    const bool dt_is_integral
-            = dt == dt_t::s32 || dt == dt_t::s8 || dt == dt_t::u8;
-
-    // Scales are supposed to work on int8 data type only.
-    if (supports_scales && dt_is_integral) {
+    if (supports_scales) { // Currently only used with binary ops
         EXPECT_NO_THROW(pd_t pd(op_desc, attr_scales, eng));
 
         // Check oscale and scales don't work together
