@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2020 Intel Corporation
+* Copyright 2018-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -32,8 +32,13 @@
 #include "oneapi/dnnl/dnnl_sycl.hpp"
 #endif
 
+#if DNNL_CPU_RUNTIME == DNNL_RUNTIME_THREADPOOL
+#include "oneapi/dnnl/dnnl_threadpool.hpp"
+#include "tests/test_thread.hpp"
+#endif
+
 #if DNNL_X64
-#include "src/cpu/x64/cpu_isa_traits.hpp"
+#include "tests/cpu_x64_isa_common.hpp"
 #endif
 
 #include <cstdint>
@@ -56,136 +61,6 @@
     CPU_INST_TEST_CASE_( \
             CONCAT_WITH_UNDERSCORE(str, TEST_CASE_NAME_PREFIX), __VA_ARGS__)
 
-#if DNNL_GPU_RUNTIME == DNNL_RUNTIME_OCL
-
-// Declare OpenCL GEMM interfaces for testing
-extern "C" {
-dnnl_status_t dnnl_ocl_sgemm(cl_command_queue queue, char transa, char transb,
-        dnnl_dim_t m, dnnl_dim_t n, dnnl_dim_t k, cl_float alpha, cl_mem a,
-        dnnl_dim_t offset_a, dnnl_dim_t lda, cl_mem b, dnnl_dim_t offset_b,
-        dnnl_dim_t ldb, cl_float beta, cl_mem c, dnnl_dim_t offset_c,
-        dnnl_dim_t ldc);
-
-dnnl_status_t dnnl_ocl_hgemm(cl_command_queue queue, char transa, char transb,
-        dnnl_dim_t m, dnnl_dim_t n, dnnl_dim_t k, cl_float alpha, cl_mem a,
-        dnnl_dim_t offset_a, dnnl_dim_t lda, cl_mem b, dnnl_dim_t offset_b,
-        dnnl_dim_t ldb, cl_float beta, cl_mem c, dnnl_dim_t offset_c,
-        dnnl_dim_t ldc);
-
-dnnl_status_t dnnl_ocl_gemm_f16f16f32(cl_command_queue queue, char transa,
-        char transb, dnnl_dim_t m, dnnl_dim_t n, dnnl_dim_t k, cl_float alpha,
-        cl_mem a, dnnl_dim_t offset_a, dnnl_dim_t lda, cl_mem b,
-        dnnl_dim_t offset_b, dnnl_dim_t ldb, cl_float beta, cl_mem c,
-        dnnl_dim_t offset_c, dnnl_dim_t ldc);
-
-dnnl_status_t dnnl_ocl_gemm_bf16bf16f32(cl_command_queue queue, char transa,
-        char transb, dnnl_dim_t m, dnnl_dim_t n, dnnl_dim_t k, cl_float alpha,
-        cl_mem a, dnnl_dim_t offset_a, dnnl_dim_t lda, cl_mem b,
-        dnnl_dim_t offset_b, dnnl_dim_t ldb, cl_float beta, cl_mem c,
-        dnnl_dim_t offset_c, dnnl_dim_t ldc);
-
-dnnl_status_t dnnl_ocl_gemm_bf16bf16bf16(cl_command_queue queue, char transa,
-        char transb, dnnl_dim_t m, dnnl_dim_t n, dnnl_dim_t k, cl_float alpha,
-        cl_mem a, dnnl_dim_t offset_a, dnnl_dim_t lda, cl_mem b,
-        dnnl_dim_t offset_b, dnnl_dim_t ldb, cl_float beta, cl_mem c,
-        dnnl_dim_t offset_c, dnnl_dim_t ldc);
-
-dnnl_status_t dnnl_ocl_gemm_s8s8s32(cl_command_queue queue, char transa,
-        char transb, char offsetc, dnnl_dim_t m, dnnl_dim_t n, dnnl_dim_t k,
-        cl_float alpha, cl_mem a, dnnl_dim_t offset_a, dnnl_dim_t lda,
-        int8_t ao, cl_mem b, dnnl_dim_t offset_b, dnnl_dim_t ldb, int8_t bo,
-        cl_float beta, cl_mem c, dnnl_dim_t offset_c, dnnl_dim_t ldc, cl_mem co,
-        dnnl_dim_t offset_co);
-dnnl_status_t dnnl_ocl_gemm_u8s8s32(cl_command_queue queue, char transa,
-        char transb, char offsetc, dnnl_dim_t m, dnnl_dim_t n, dnnl_dim_t k,
-        cl_float alpha, cl_mem a, dnnl_dim_t offset_a, dnnl_dim_t lda,
-        uint8_t ao, cl_mem b, dnnl_dim_t offset_b, dnnl_dim_t ldb, int8_t bo,
-        cl_float beta, cl_mem c, dnnl_dim_t offset_c, dnnl_dim_t ldc, cl_mem co,
-        dnnl_dim_t offset_co);
-dnnl_status_t dnnl_ocl_gemm_s8u8s32(cl_command_queue queue, char transa,
-        char transb, char offsetc, dnnl_dim_t m, dnnl_dim_t n, dnnl_dim_t k,
-        cl_float alpha, cl_mem a, dnnl_dim_t offset_a, dnnl_dim_t lda,
-        int8_t ao, cl_mem b, dnnl_dim_t offset_b, dnnl_dim_t ldb, uint8_t bo,
-        cl_float beta, cl_mem c, dnnl_dim_t offset_c, dnnl_dim_t ldc, cl_mem co,
-        dnnl_dim_t offset_co);
-dnnl_status_t dnnl_ocl_gemm_u8u8s32(cl_command_queue queue, char transa,
-        char transb, char offsetc, dnnl_dim_t m, dnnl_dim_t n, dnnl_dim_t k,
-        cl_float alpha, cl_mem a, dnnl_dim_t offset_a, dnnl_dim_t lda,
-        uint8_t ao, cl_mem b, dnnl_dim_t offset_b, dnnl_dim_t ldb, uint8_t bo,
-        cl_float beta, cl_mem c, dnnl_dim_t offset_c, dnnl_dim_t ldc, cl_mem co,
-        dnnl_dim_t offset_co);
-}
-#endif
-
-#if DNNL_WITH_SYCL
-
-// Declare SYCL GEMM interfaces for testing
-namespace dnnl {
-void DNNL_API gemm(cl::sycl::queue &queue, char transa, char transb,
-        memory::dim m, memory::dim n, memory::dim k, float alpha,
-        cl::sycl::buffer<float, 1> &a, memory::dim offset_a, memory::dim lda,
-        cl::sycl::buffer<float, 1> &b, memory::dim offset_b, memory::dim ldb,
-        float beta, cl::sycl::buffer<float, 1> &c, memory::dim offset_c,
-        memory::dim ldc);
-
-void DNNL_API gemm(cl::sycl::queue &queue, char transa, char transb,
-        memory::dim m, memory::dim n, memory::dim k, float alpha,
-        cl::sycl::buffer<cl::sycl::half, 1> &a, memory::dim offset_a,
-        memory::dim lda, cl::sycl::buffer<cl::sycl::half, 1> &b,
-        memory::dim offset_b, memory::dim ldb, float beta,
-        cl::sycl::buffer<cl::sycl::half, 1> &c, memory::dim offset_c,
-        memory::dim ldc);
-
-void DNNL_API gemm(cl::sycl::queue &queue, char transa, char transb,
-        memory::dim m, memory::dim n, memory::dim k, float alpha,
-        cl::sycl::buffer<cl::sycl::half, 1> &a, memory::dim offset_a,
-        memory::dim lda, cl::sycl::buffer<cl::sycl::half, 1> &b,
-        memory::dim offset_b, memory::dim ldb, float beta,
-        cl::sycl::buffer<float, 1> &c, memory::dim offset_c, memory::dim ldc);
-
-void DNNL_API gemm_bf16bf16bf16(cl::sycl::queue &queue, char transa,
-        char transb, memory::dim m, memory::dim n, memory::dim k, float alpha,
-        cl::sycl::buffer<uint16_t, 1> &a, memory::dim offset_a, memory::dim lda,
-        cl::sycl::buffer<uint16_t, 1> &b, memory::dim offset_b, memory::dim ldb,
-        float beta, cl::sycl::buffer<uint16_t, 1> &c, memory::dim offset_c,
-        memory::dim ldc);
-
-void DNNL_API gemm_bf16bf16f32(cl::sycl::queue &queue, char transa, char transb,
-        memory::dim m, memory::dim n, memory::dim k, float alpha,
-        cl::sycl::buffer<uint16_t, 1> &a, memory::dim offset_a, memory::dim lda,
-        cl::sycl::buffer<uint16_t, 1> &b, memory::dim offset_b, memory::dim ldb,
-        float beta, cl::sycl::buffer<float, 1> &c, memory::dim offset_c,
-        memory::dim ldc);
-
-void DNNL_API gemm(cl::sycl::queue &queue, char transa, char transb,
-        memory::dim m, memory::dim n, memory::dim k, float alpha,
-        const float *a, memory::dim lda, const float *b, memory::dim ldb,
-        float beta, float *c, memory::dim ldc);
-
-void DNNL_API gemm(cl::sycl::queue &queue, char transa, char transb,
-        memory::dim m, memory::dim n, memory::dim k, float alpha,
-        const cl::sycl::half *a, memory::dim lda, const cl::sycl::half *b,
-        memory::dim ldb, float beta, cl::sycl::half *c, memory::dim ldc);
-
-void DNNL_API gemm(cl::sycl::queue &queue, char transa, char transb,
-        memory::dim m, memory::dim n, memory::dim k, float alpha,
-        const cl::sycl::half *a, memory::dim lda, const cl::sycl::half *b,
-        memory::dim ldb, float beta, float *c, memory::dim ldc);
-
-void DNNL_API gemm_bf16bf16bf16(cl::sycl::queue &queue, char transa,
-        char transb, memory::dim m, memory::dim n, memory::dim k, float alpha,
-        const uint16_t *a, memory::dim lda, const uint16_t *b, memory::dim ldb,
-        float beta, uint16_t *c, memory::dim ldc);
-
-void DNNL_API gemm_bf16bf16f32(cl::sycl::queue &queue, char transa, char transb,
-        memory::dim m, memory::dim n, memory::dim k, float alpha,
-        const uint16_t *a, memory::dim lda, const uint16_t *b, memory::dim ldb,
-        float beta, float *c, memory::dim ldc);
-
-} // namespace dnnl
-
-#endif
-
 // Declare bfloat16 GEMM interfaces for testing
 extern "C" {
 dnnl_status_t dnnl_gemm_bf16bf16f32(char transa, char transb, dnnl_dim_t M,
@@ -271,7 +146,7 @@ inline test_params make_test_params_pack(
     return params;
 }
 
-#if defined(DNNL_SYCL_DPCPP)
+#if defined(DNNL_WTIH_SYCL)
 bool is_memory_kind_buffer(const test_memory &mem) {
     return sycl_interop::get_memory_kind(mem.get())
             == sycl_interop::memory_kind::buffer;
@@ -633,51 +508,6 @@ struct dnnl_gemm<float16_t, float16_t, float16_t> {
     static dnnl_status_t call(const test_params &p, const test_memory &a_mem,
             const test_memory &b_mem, const test_memory &c_mem,
             const test_memory &) {
-        engine eng = a_mem.get().get_engine();
-        stream s(eng);
-#if DNNL_GPU_RUNTIME == DNNL_RUNTIME_OCL
-        if (get_test_engine_kind() == engine::kind::gpu) {
-            cl_command_queue q = ocl_interop::get_command_queue(s);
-            auto status = dnnl_ocl_hgemm(q, p.transA, p.transB, p.M, p.N, p.K,
-                    p.alpha, ocl_interop::get_mem_object(a_mem.get()), p.off.a,
-                    p.lda, ocl_interop::get_mem_object(b_mem.get()), p.off.b,
-                    p.ldb, p.beta, ocl_interop::get_mem_object(c_mem.get()),
-                    p.off.c, p.ldc);
-            s.wait();
-            return status;
-        }
-#elif DNNL_GPU_RUNTIME == DNNL_RUNTIME_SYCL
-        if (get_test_engine_kind() == engine::kind::gpu) {
-            cl::sycl::queue sycl_queue = sycl_interop::get_queue(s);
-            if (is_memory_kind_buffer(a_mem)) {
-                // Test buffer API
-                assert(is_memory_kind_buffer(b_mem));
-                assert(is_memory_kind_buffer(c_mem));
-                auto a = sycl_interop::get_buffer<cl::sycl::half>(a_mem.get());
-                auto b = sycl_interop::get_buffer<cl::sycl::half>(b_mem.get());
-                auto c = sycl_interop::get_buffer<cl::sycl::half>(c_mem.get());
-                dnnl::gemm(sycl_queue, p.transA, p.transB, p.M, p.N, p.K,
-                        p.alpha, a, p.off.a, p.lda, b, p.off.b, p.ldb, p.beta,
-                        c, p.off.c, p.ldc);
-            } else {
-                // Test USM API
-                assert(!is_memory_kind_buffer(a_mem));
-                assert(!is_memory_kind_buffer(b_mem));
-                assert(!is_memory_kind_buffer(c_mem));
-
-                auto a = static_cast<cl::sycl::half *>(
-                        a_mem.get().get_data_handle());
-                auto b = static_cast<cl::sycl::half *>(
-                        b_mem.get().get_data_handle());
-                auto c = static_cast<cl::sycl::half *>(
-                        c_mem.get().get_data_handle());
-                dnnl::gemm(sycl_queue, p.transA, p.transB, p.M, p.N, p.K,
-                        p.alpha, a, p.lda, b, p.ldb, p.beta, c, p.ldc);
-            }
-            s.wait();
-            return dnnl_success;
-        }
-#endif
         throw error(dnnl_runtime_error, "unknown gemm");
     }
 };
@@ -759,58 +589,19 @@ struct dnnl_gemm<float, float, float> {
         if (p.pack_params.pack_a || p.pack_params.pack_b)
             return call_packed(p, a_mem, b_mem, c_mem);
 
-        engine eng = a_mem.get().get_engine();
-        stream s(eng);
+        auto A = map_memory<float>(a_mem);
+        auto B = map_memory<float>(b_mem);
+        auto C = map_memory<float>(c_mem);
 
-#if DNNL_GPU_RUNTIME == DNNL_RUNTIME_OCL
-        if (get_test_engine_kind() == engine::kind::gpu) {
-            cl_command_queue q = ocl_interop::get_command_queue(s);
-            auto status = dnnl_ocl_sgemm(q, p.transA, p.transB, p.M, p.N, p.K,
-                    p.alpha, ocl_interop::get_mem_object(a_mem.get()), p.off.a,
-                    p.lda, ocl_interop::get_mem_object(b_mem.get()), p.off.b,
-                    p.ldb, p.beta, ocl_interop::get_mem_object(c_mem.get()),
-                    p.off.c, p.ldc);
-            s.wait();
-            return status;
-        }
-#elif DNNL_GPU_RUNTIME == DNNL_RUNTIME_SYCL
-        if (get_test_engine_kind() == engine::kind::gpu) {
-            cl::sycl::queue sycl_queue = sycl_interop::get_queue(s);
-            if (is_memory_kind_buffer(a_mem)) {
-                // Test buffer API
-                assert(is_memory_kind_buffer(b_mem));
-                assert(is_memory_kind_buffer(c_mem));
-                auto a = sycl_interop::get_buffer<float>(a_mem.get());
-                auto b = sycl_interop::get_buffer<float>(b_mem.get());
-                auto c = sycl_interop::get_buffer<float>(c_mem.get());
-                dnnl::gemm(sycl_queue, p.transA, p.transB, p.M, p.N, p.K,
-                        p.alpha, a, p.off.a, p.lda, b, p.off.b, p.ldb, p.beta,
-                        c, p.off.c, p.ldc);
-            } else {
-                // Test USM API
-                assert(!is_memory_kind_buffer(a_mem));
-                assert(!is_memory_kind_buffer(b_mem));
-                assert(!is_memory_kind_buffer(c_mem));
-                auto a = static_cast<float *>(a_mem.get().get_data_handle());
-                auto b = static_cast<float *>(b_mem.get().get_data_handle());
-                auto c = static_cast<float *>(c_mem.get().get_data_handle());
-                dnnl::gemm(sycl_queue, p.transA, p.transB, p.M, p.N, p.K,
-                        p.alpha, a, p.lda, b, p.ldb, p.beta, c, p.ldc);
-            }
-            s.wait();
-            return dnnl_success;
-        }
+#if DNNL_CPU_RUNTIME == DNNL_RUNTIME_THREADPOOL
+        static auto *st = dnnl::testing::get_threadpool();
+        return static_cast<dnnl_status_t>(dnnl::threadpool_interop::sgemm(
+                p.transA, p.transB, p.M, p.N, p.K, p.alpha, A, p.lda, B, p.ldb,
+                p.beta, C, p.ldc, st));
+#else
+        return dnnl_sgemm(p.transA, p.transB, p.M, p.N, p.K, p.alpha, A, p.lda,
+                B, p.ldb, p.beta, C, p.ldc);
 #endif
-        if (get_test_engine_kind() == engine::kind::cpu) {
-            auto A = map_memory<float>(a_mem);
-            auto B = map_memory<float>(b_mem);
-            auto C = map_memory<float>(c_mem);
-
-            return dnnl_sgemm(p.transA, p.transB, p.M, p.N, p.K, p.alpha, A,
-                    p.lda, B, p.ldb, p.beta, C, p.ldc);
-        }
-
-        throw error(dnnl_runtime_error, "unknown gemm");
     }
 };
 
@@ -903,23 +694,6 @@ struct dnnl_gemm<int8_t, int8_t, int32_t> {
             const test_memory &b_mem, const test_memory &c_mem,
             const test_memory &oc_mem) {
 
-#if DNNL_GPU_RUNTIME == DNNL_RUNTIME_OCL
-        if (get_test_engine_kind() == engine::kind::gpu) {
-            engine eng = get_test_engine();
-            stream s(eng);
-            cl_command_queue q = ocl_interop::get_command_queue(s);
-            auto status = dnnl_ocl_gemm_s8s8s32(q, p.transA, p.transB,
-                    p.igemm_params.offsetc, p.M, p.N, p.K, p.alpha,
-                    ocl_interop::get_mem_object(a_mem.get()), p.off.a, p.lda,
-                    p.igemm_params.oa(),
-                    ocl_interop::get_mem_object(b_mem.get()), p.off.b, p.ldb,
-                    p.igemm_params.ob(), p.beta,
-                    ocl_interop::get_mem_object(c_mem.get()), p.off.c, p.ldc,
-                    ocl_interop::get_mem_object(oc_mem.get()), p.off.co);
-            s.wait();
-            return status;
-        }
-#endif
         if (p.pack_params.pack_a || p.pack_params.pack_b)
             return call_packed(p, a_mem, b_mem, c_mem, oc_mem);
 
@@ -940,23 +714,6 @@ struct dnnl_gemm<int8_t, uint8_t, int32_t> {
     static dnnl_status_t call(const test_params &p, const test_memory &a_mem,
             const test_memory &b_mem, const test_memory &c_mem,
             const test_memory &oc_mem) {
-#if DNNL_GPU_RUNTIME == DNNL_RUNTIME_OCL
-        if (get_test_engine_kind() == engine::kind::gpu) {
-            engine eng = get_test_engine();
-            stream s(eng);
-            cl_command_queue q2 = ocl_interop::get_command_queue(s);
-            auto status = dnnl_ocl_gemm_s8u8s32(q2, p.transA, p.transB,
-                    p.igemm_params.offsetc, p.M, p.N, p.K, p.alpha,
-                    ocl_interop::get_mem_object(a_mem.get()), p.off.a, p.lda,
-                    p.igemm_params.oa(),
-                    ocl_interop::get_mem_object(b_mem.get()), p.off.b, p.ldb,
-                    (uint8_t)p.igemm_params.ob(), p.beta,
-                    ocl_interop::get_mem_object(c_mem.get()), p.off.c, p.ldc,
-                    ocl_interop::get_mem_object(oc_mem.get()), p.off.co);
-            s.wait();
-            return status;
-        }
-#endif
         throw error(dnnl_runtime_error, "unknown gemm");
     }
 };
@@ -967,23 +724,6 @@ struct dnnl_gemm<uint8_t, uint8_t, int32_t> {
             const test_memory &b_mem, const test_memory &c_mem,
             const test_memory &oc_mem) {
 
-#if DNNL_GPU_RUNTIME == DNNL_RUNTIME_OCL
-        if (get_test_engine_kind() == engine::kind::gpu) {
-            engine eng = get_test_engine();
-            stream s(eng);
-            cl_command_queue q = ocl_interop::get_command_queue(s);
-            auto status = dnnl_ocl_gemm_u8u8s32(q, p.transA, p.transB,
-                    p.igemm_params.offsetc, p.M, p.N, p.K, p.alpha,
-                    ocl_interop::get_mem_object(a_mem.get()), p.off.a, p.lda,
-                    (uint8_t)p.igemm_params.oa(),
-                    ocl_interop::get_mem_object(b_mem.get()), p.off.b, p.ldb,
-                    (uint8_t)p.igemm_params.ob(), p.beta,
-                    ocl_interop::get_mem_object(c_mem.get()), p.off.c, p.ldc,
-                    ocl_interop::get_mem_object(oc_mem.get()), p.off.co);
-            s.wait();
-            return status;
-        }
-#endif
         throw error(dnnl_runtime_error, "unknown gemm");
     }
 };
@@ -1078,24 +818,6 @@ struct dnnl_gemm<uint8_t, int8_t, int32_t> {
             const test_memory &oc_mem) {
         assert(p.igemm_params.oa() >= 0);
 
-#if DNNL_GPU_RUNTIME == DNNL_RUNTIME_OCL
-        if (get_test_engine_kind() == engine::kind::gpu) {
-            engine eng = get_test_engine();
-            stream s(eng);
-            cl_command_queue q = ocl_interop::get_command_queue(s);
-            auto status = dnnl_ocl_gemm_u8s8s32(q, p.transA, p.transB,
-                    p.igemm_params.offsetc, p.M, p.N, p.K, p.alpha,
-                    ocl_interop::get_mem_object(a_mem.get()), p.off.a, p.lda,
-                    p.igemm_params.oa(),
-                    ocl_interop::get_mem_object(b_mem.get()), p.off.b, p.ldb,
-                    p.igemm_params.ob(), p.beta,
-                    ocl_interop::get_mem_object(c_mem.get()), p.off.c, p.ldc,
-                    ocl_interop::get_mem_object(oc_mem.get()), p.off.co);
-            s.wait();
-            return status;
-        }
-#endif
-
         if (p.pack_params.pack_a || p.pack_params.pack_b)
             return call_packed(p, a_mem, b_mem, c_mem, oc_mem);
 
@@ -1117,49 +839,6 @@ struct dnnl_gemm<float16_t, float16_t, float> {
     static dnnl_status_t call(const test_params &p, const test_memory &a_mem,
             const test_memory &b_mem, const test_memory &c_mem,
             const test_memory &) {
-        engine eng = a_mem.get().get_engine();
-        stream s(eng);
-#if DNNL_GPU_RUNTIME == DNNL_RUNTIME_OCL
-        if (get_test_engine_kind() == engine::kind::gpu) {
-            cl_command_queue q = ocl_interop::get_command_queue(s);
-            auto status = dnnl_ocl_gemm_f16f16f32(q, p.transA, p.transB, p.M,
-                    p.N, p.K, p.alpha, ocl_interop::get_mem_object(a_mem.get()),
-                    p.off.a, p.lda, ocl_interop::get_mem_object(b_mem.get()),
-                    p.off.b, p.ldb, p.beta,
-                    ocl_interop::get_mem_object(c_mem.get()), p.off.c, p.ldc);
-            s.wait();
-            return status;
-        }
-#elif DNNL_GPU_RUNTIME == DNNL_RUNTIME_SYCL
-        if (get_test_engine_kind() == engine::kind::gpu) {
-            cl::sycl::queue sycl_queue = sycl_interop::get_queue(s);
-            if (is_memory_kind_buffer(a_mem)) {
-                // Test buffer API
-                assert(is_memory_kind_buffer(b_mem));
-                assert(is_memory_kind_buffer(c_mem));
-                auto a = sycl_interop::get_buffer<cl::sycl::half>(a_mem.get());
-                auto b = sycl_interop::get_buffer<cl::sycl::half>(b_mem.get());
-                auto c = sycl_interop::get_buffer<float>(c_mem.get());
-                dnnl::gemm(sycl_queue, p.transA, p.transB, p.M, p.N, p.K,
-                        p.alpha, a, p.off.a, p.lda, b, p.off.b, p.ldb, p.beta,
-                        c, p.off.c, p.ldc);
-            } else {
-                // Test USM API
-                assert(!is_memory_kind_buffer(a_mem));
-                assert(!is_memory_kind_buffer(b_mem));
-                assert(!is_memory_kind_buffer(c_mem));
-                auto a = static_cast<cl::sycl::half *>(
-                        a_mem.get().get_data_handle());
-                auto b = static_cast<cl::sycl::half *>(
-                        b_mem.get().get_data_handle());
-                auto c = static_cast<float *>(c_mem.get().get_data_handle());
-                dnnl::gemm(sycl_queue, p.transA, p.transB, p.M, p.N, p.K,
-                        p.alpha, a, p.lda, b, p.ldb, p.beta, c, p.ldc);
-            }
-            s.wait();
-            return dnnl_success;
-        }
-#endif
         return dnnl_unimplemented;
     }
 };
@@ -1237,47 +916,6 @@ struct dnnl_gemm<bfloat16_t, bfloat16_t, float> {
     static dnnl_status_t call(const test_params &p, const test_memory &a_mem,
             const test_memory &b_mem, const test_memory &c_mem,
             const test_memory &) {
-        engine eng = a_mem.get().get_engine();
-        stream s(eng);
-#if DNNL_GPU_RUNTIME == DNNL_RUNTIME_OCL
-        if (get_test_engine_kind() == engine::kind::gpu) {
-            cl_command_queue q = ocl_interop::get_command_queue(s);
-            auto status = dnnl_ocl_gemm_bf16bf16f32(q, p.transA, p.transB, p.M,
-                    p.N, p.K, p.alpha, ocl_interop::get_mem_object(a_mem.get()),
-                    p.off.a, p.lda, ocl_interop::get_mem_object(b_mem.get()),
-                    p.off.b, p.ldb, p.beta,
-                    ocl_interop::get_mem_object(c_mem.get()), p.off.c, p.ldc);
-            s.wait();
-            return status;
-        }
-#elif DNNL_GPU_RUNTIME == DNNL_RUNTIME_SYCL
-        if (get_test_engine_kind() == engine::kind::gpu) {
-            cl::sycl::queue sycl_queue = sycl_interop::get_queue(s);
-            if (is_memory_kind_buffer(a_mem)) {
-                // Test buffer API
-                assert(is_memory_kind_buffer(b_mem));
-                assert(is_memory_kind_buffer(c_mem));
-                auto a = sycl_interop::get_buffer<uint16_t>(a_mem.get());
-                auto b = sycl_interop::get_buffer<uint16_t>(b_mem.get());
-                auto c = sycl_interop::get_buffer<float>(c_mem.get());
-                dnnl::gemm_bf16bf16f32(sycl_queue, p.transA, p.transB, p.M, p.N,
-                        p.K, p.alpha, a, p.off.a, p.lda, b, p.off.b, p.ldb,
-                        p.beta, c, p.off.c, p.ldc);
-            } else {
-                // Test USM API
-                assert(!is_memory_kind_buffer(a_mem));
-                assert(!is_memory_kind_buffer(b_mem));
-                assert(!is_memory_kind_buffer(c_mem));
-                auto a = static_cast<uint16_t *>(a_mem.get().get_data_handle());
-                auto b = static_cast<uint16_t *>(b_mem.get().get_data_handle());
-                auto c = static_cast<float *>(c_mem.get().get_data_handle());
-                dnnl::gemm_bf16bf16f32(sycl_queue, p.transA, p.transB, p.M, p.N,
-                        p.K, p.alpha, a, p.lda, b, p.ldb, p.beta, c, p.ldc);
-            }
-            s.wait();
-            return dnnl_success;
-        }
-#endif
         if (p.pack_params.pack_a || p.pack_params.pack_b)
             return call_packed(p, a_mem, b_mem, c_mem);
 
@@ -1294,48 +932,6 @@ struct dnnl_gemm<bfloat16_t, bfloat16_t, bfloat16_t> {
     static dnnl_status_t call(const test_params &p, const test_memory &a_mem,
             const test_memory &b_mem, const test_memory &c_mem,
             const test_memory &) {
-        engine eng = a_mem.get().get_engine();
-        stream s(eng);
-#if DNNL_GPU_RUNTIME == DNNL_RUNTIME_OCL
-        if (get_test_engine_kind() == engine::kind::gpu) {
-            cl_command_queue q = ocl_interop::get_command_queue(s);
-            auto status = dnnl_ocl_gemm_bf16bf16bf16(q, p.transA, p.transB, p.M,
-                    p.N, p.K, p.alpha, ocl_interop::get_mem_object(a_mem.get()),
-                    p.off.a, p.lda, ocl_interop::get_mem_object(b_mem.get()),
-                    p.off.b, p.ldb, p.beta,
-                    ocl_interop::get_mem_object(c_mem.get()), p.off.c, p.ldc);
-            s.wait();
-            return status;
-        }
-#elif DNNL_GPU_RUNTIME == DNNL_RUNTIME_SYCL
-        if (get_test_engine_kind() == engine::kind::gpu) {
-            cl::sycl::queue sycl_queue = sycl_interop::get_queue(s);
-            if (is_memory_kind_buffer(a_mem)) {
-                // Test buffer API
-                assert(is_memory_kind_buffer(b_mem));
-                assert(is_memory_kind_buffer(c_mem));
-                auto a = sycl_interop::get_buffer<uint16_t>(a_mem.get());
-                auto b = sycl_interop::get_buffer<uint16_t>(b_mem.get());
-                auto c = sycl_interop::get_buffer<uint16_t>(c_mem.get());
-                dnnl::gemm_bf16bf16bf16(sycl_queue, p.transA, p.transB, p.M,
-                        p.N, p.K, p.alpha, a, p.off.a, p.lda, b, p.off.b, p.ldb,
-                        p.beta, c, p.off.c, p.ldc);
-            } else {
-                // Test USM API
-                assert(!is_memory_kind_buffer(a_mem));
-                assert(!is_memory_kind_buffer(b_mem));
-                assert(!is_memory_kind_buffer(c_mem));
-                auto a = static_cast<uint16_t *>(a_mem.get().get_data_handle());
-                auto b = static_cast<uint16_t *>(b_mem.get().get_data_handle());
-                auto c = static_cast<uint16_t *>(c_mem.get().get_data_handle());
-                dnnl::gemm_bf16bf16bf16(sycl_queue, p.transA, p.transB, p.M,
-                        p.N, p.K, p.alpha, a, p.lda, b, p.ldb, p.beta, c,
-                        p.ldc);
-            }
-            s.wait();
-            return dnnl_success;
-        }
-#endif
         return dnnl_unimplemented;
     }
 };
@@ -1393,6 +989,14 @@ protected:
     virtual void SetUp() {
         const auto &p = ::testing::TestWithParam<test_params>::GetParam();
 
+        SKIP_IF(get_test_engine_kind() == engine::kind::gpu,
+                "GPU GEMM not implemented.");
+
+#if DNNL_CPU_RUNTIME == DNNL_RUNTIME_SYCL
+        SKIP_IF(get_test_engine_kind() == engine::kind::cpu,
+                "SYCL CPU GEMM not implemented.");
+#endif
+
         bool zero_off = (p.off.a == 0 && p.off.b == 0 && p.off.c == 0);
         SKIP_IF(!zero_off && get_test_engine_kind() == engine::kind::cpu,
                 "CPU does not support non-zero offsets.");
@@ -1415,23 +1019,21 @@ protected:
                                 || data_traits<a_dt>::data_type
                                         == memory::data_type::s8),
                 "SYCL GPU int GEMM not implemented.");
+        SKIP_IF_CUDA(true, "Test not supported in CUDA backend");
 #endif
 
+#if DNNL_X64
         bool is_bf16bf16f32 = true
                 && data_traits<a_dt>::data_type == memory::data_type::bf16
                 && data_traits<b_dt>::data_type == memory::data_type::bf16
                 && data_traits<c_dt>::data_type == memory::data_type::f32;
 
-#if DNNL_X64
         SKIP_IF(is_bf16bf16f32 && get_test_engine_kind() == engine::kind::cpu
-                        && !impl::cpu::x64::mayiuse(
-                                impl::cpu::x64::avx512_core),
+                        && !dnnl::mayiuse(cpu_isa::avx512_core),
                 "Skip test for systems that do not support avx512_core.");
 #endif
 
         bool pack = (p.pack_params.pack_a || p.pack_params.pack_b);
-        SKIP_IF(get_test_engine_kind() == engine::kind::gpu && pack,
-                "GPU does not support packed GEMM.");
         SKIP_IF(!DNNL_X64 && pack,
                 "Packed GEMM does not support non-x64 CPUs.");
         SKIP_IF((p.alpha != 1.f || p.igemm_params.oa() != 0
