@@ -1,5 +1,6 @@
 /*******************************************************************************
-* Copyright 2017-2020 Intel Corporation
+* Copyright 2017-2021 Intel Corporation
+* Copyright 2020-2021 FUJITSU LIMITED
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -215,6 +216,81 @@ HANDLE_EXCEPTIONS_FOR_TEST_F(attr_test_t, TestScales) {
     ASSERT_EQ(scales[2], 3.);
 }
 
+HANDLE_EXCEPTIONS_FOR_TEST_F(attr_test_t, TestRNNDataQuantization) {
+    dnnl::primitive_attr attr;
+
+    float scale, shift;
+
+    // default scale and shift
+    attr.get_rnn_data_qparams(scale, shift);
+    ASSERT_EQ(scale, 1.f);
+    ASSERT_EQ(shift, 0.f);
+
+    // non-default
+    attr.set_rnn_data_qparams(0.5f, 2.f);
+    attr.get_rnn_data_qparams(scale, shift);
+    ASSERT_EQ(scale, 0.5f);
+    ASSERT_EQ(shift, 2.f);
+}
+
+HANDLE_EXCEPTIONS_FOR_TEST_F(attr_test_t, TestRNNWeightsQuantization) {
+    dnnl::primitive_attr attr;
+
+    int scales_mask;
+    std::vector<float> scales;
+
+    // default scale and shift
+    attr.get_rnn_weights_qparams(scales_mask, scales);
+    ASSERT_EQ(scales_mask, 0);
+    ASSERT_EQ(scales.size(), 1U);
+    ASSERT_EQ(scales[0], 1.f);
+
+    // single non-default scales
+    attr.set_rnn_weights_qparams(0, {2.f});
+    attr.get_rnn_weights_qparams(scales_mask, scales);
+    ASSERT_EQ(scales_mask, 0);
+    ASSERT_EQ(scales.size(), 1U);
+    ASSERT_EQ(scales[0], 2.f);
+
+    // multiple scales
+    attr.set_rnn_weights_qparams(1 << 1, {1.f, 2.f, 4.f});
+    attr.get_rnn_weights_qparams(scales_mask, scales);
+    ASSERT_EQ(scales_mask, 1 << 1);
+    ASSERT_EQ(scales.size(), 3U);
+    ASSERT_EQ(scales[0], 1.f);
+    ASSERT_EQ(scales[1], 2.f);
+    ASSERT_EQ(scales[2], 4.f);
+}
+
+HANDLE_EXCEPTIONS_FOR_TEST_F(attr_test_t, TestRNNProjWeightsQuantization) {
+    dnnl::primitive_attr attr;
+
+    int scales_mask;
+    std::vector<float> scales;
+
+    // default scale and shift
+    attr.get_rnn_weights_projection_qparams(scales_mask, scales);
+    ASSERT_EQ(scales_mask, 0);
+    ASSERT_EQ(scales.size(), 1U);
+    ASSERT_EQ(scales[0], 1.f);
+
+    // single non-default scales
+    attr.set_rnn_weights_projection_qparams(0, {2.f});
+    attr.get_rnn_weights_projection_qparams(scales_mask, scales);
+    ASSERT_EQ(scales_mask, 0);
+    ASSERT_EQ(scales.size(), 1U);
+    ASSERT_EQ(scales[0], 2.f);
+
+    // multiple scales
+    attr.set_rnn_weights_projection_qparams(1 << 1, {1.f, 2.f, 4.f});
+    attr.get_rnn_weights_projection_qparams(scales_mask, scales);
+    ASSERT_EQ(scales_mask, 1 << 1);
+    ASSERT_EQ(scales.size(), 3U);
+    ASSERT_EQ(scales[0], 1.f);
+    ASSERT_EQ(scales[1], 2.f);
+    ASSERT_EQ(scales[2], 4.f);
+}
+
 TEST_F(attr_test_t, TestScalesExpectFailure) {
     dnnl::primitive_attr attr;
     const int unsupported_arg = DNNL_ARG_MEAN;
@@ -333,6 +409,9 @@ HANDLE_EXCEPTIONS_FOR_TEST_F(attr_test_t, DepthwiseFusion) {
     auto engine_kind = get_test_engine_kind();
     SKIP_IF(engine_kind != engine::kind::cpu,
             "Depthwise fusion is only supported on CPU engine");
+#if DNNL_AARCH64
+    SKIP_IF(true, "Depthwise fusion is not supported on AArch64 at this time");
+#endif
 
     engine e {engine_kind, 0};
 
@@ -383,6 +462,57 @@ HANDLE_EXCEPTIONS_FOR_TEST_F(attr_test_t, DepthwiseFusion) {
     }
 }
 
+HANDLE_EXCEPTIONS_FOR_TEST_F(attr_test_t, InnerProdPostops) {
+    auto engine_kind = get_test_engine_kind();
+#ifdef DNNL_SYCL_CUDA
+    SKIP_IF(engine_kind == engine::kind::gpu,
+            "Inner Product postops are not supported on CUDA backend");
+#endif
+    engine e {engine_kind, 0};
+
+    std::vector<memory::data_type> test_dts {
+            memory::data_type::f32, memory::data_type::s8};
+
+    if (!unsupported_data_type(memory::data_type::bf16))
+        test_dts.push_back(memory::data_type::bf16);
+
+    for (auto dt : test_dts) {
+        memory::desc src_md {{1024, 512, 64, 64}, dt, memory::format_tag::any};
+        memory::desc wht_md {{256, 512, 64, 64}, dt, memory::format_tag::any};
+        memory::desc bia_md {{256}, dt, memory::format_tag::any};
+        memory::desc dst_md {{1024, 256}, dt, memory::format_tag::any};
+
+        auto ip_desc = inner_product_forward::desc(
+                prop_kind::forward_training, src_md, wht_md, bia_md, dst_md);
+
+        std::string impl_info_no_postops;
+
+        auto pd = inner_product_forward::primitive_desc(ip_desc, e);
+        ASSERT_NO_THROW(impl_info_no_postops = pd.impl_info_str(););
+
+        dnnl::primitive_attr attr;
+        const float scale = 1.f;
+        const float alpha = 1.f;
+        const float beta = 1.f;
+        dnnl::post_ops ops;
+
+        ops.append_sum(1.0);
+
+        ops.append_eltwise(scale, algorithm::eltwise_relu, alpha, beta);
+
+        memory::desc src1_md({1}, data_type::f32, {1});
+        ops.append_binary(algorithm::binary_add, src1_md);
+
+        attr.set_post_ops(ops);
+
+        std::string impl_info_with_postops;
+
+        pd = inner_product_forward::primitive_desc(ip_desc, attr, e);
+        ASSERT_NO_THROW(impl_info_with_postops = pd.impl_info_str(););
+        ASSERT_EQ(impl_info_no_postops, impl_info_with_postops);
+    }
+}
+
 HANDLE_EXCEPTIONS_FOR_TEST_F(attr_test_t, TestGetAttr) {
     auto engine_kind = get_test_engine_kind();
     SKIP_IF(engine_kind != engine::kind::cpu,
