@@ -121,6 +121,8 @@ TEST_F(runtime_attr_test_t, TestConcat) {
 }
 
 TEST_F(runtime_attr_test_t, TestConv) {
+    // Datatype u8 is not supported in the Nvidia backend
+    SKIP_IF_CUDA(true, "Unsupported datatype for CUDA");
     memory::desc src_md {{1, 16, 7, 7}, data_type::u8, tag::any};
     memory::desc wei_md {{32, 16, 3, 3}, data_type::s8, tag::any};
     memory::desc dst_md {{1, 32, 7, 7}, data_type::s32, tag::any};
@@ -201,6 +203,8 @@ TEST_F(runtime_attr_test_t, TestEltwise) {
 }
 
 TEST_F(runtime_attr_test_t, TestInnerProduct) {
+    // Datatype u8 is not supported in the Nvidia backend
+    SKIP_IF_CUDA(true, "Unsupported datatype for CUDA");
     memory::desc src_md {{1, 16, 7, 7}, data_type::u8, tag::any};
     memory::desc wei_md {{32, 16, 7, 7}, data_type::s8, tag::any};
     memory::desc dst_md {{1, 32}, data_type::s32, tag::any};
@@ -222,6 +226,7 @@ TEST_F(runtime_attr_test_t, TestInnerProduct) {
 }
 
 TEST_F(runtime_attr_test_t, TestLNorm) {
+    SKIP_IF_CUDA(true, "Layer normalization primitive not supported for CUDA");
     for (auto dt : {data_type::f32}) {
         memory::desc md {{1, 16, 16}, dt, tag::abc};
         memory::desc stat_md {{1, 16}, data_type::f32, tag::ab};
@@ -317,6 +322,25 @@ TEST_F(runtime_attr_test_t, TestPool) {
     }
 }
 
+TEST_F(runtime_attr_test_t, TestPReLU) {
+    SKIP_IF_CUDA(true, "Unsupported primitive not supported for CUDA");
+    memory::desc data_md {{1, 16, 3, 3}, data_type::f32, tag::abcd};
+    memory::desc weights_md {{1, 16, 3, 3}, data_type::f32, tag::abcd};
+    prelu_forward::desc op_d(prop_kind::forward, data_md, weights_md);
+    CHECK_OK(prelu_forward::primitive_desc(op_d, eng));
+    CHECK_UNIMPL(prelu_forward::primitive_desc(
+            op_d, gen_attr_with_oscale(false), eng));
+    CHECK_UNIMPL(prelu_forward::primitive_desc(
+            op_d, gen_attr_with_oscale(true), eng));
+
+    for (auto arg : {DNNL_ARG_SRC, DNNL_ARG_DST}) {
+        CHECK_UNIMPL(prelu_forward::primitive_desc(
+                op_d, gen_attr_with_zp(false, arg), eng));
+        CHECK_UNIMPL(prelu_forward::primitive_desc(
+                op_d, gen_attr_with_zp(true, arg), eng));
+    }
+}
+
 CPU_TEST_F(runtime_attr_test_t, TestReorder) {
     memory::desc src_md {{1, 16, 8, 8}, data_type::s8, tag::abcd};
     memory::desc dst_md {{1, 16, 8, 8}, data_type::s8, tag::acdb};
@@ -335,6 +359,8 @@ CPU_TEST_F(runtime_attr_test_t, TestReorder) {
 }
 
 TEST_F(runtime_attr_test_t, TestRNN) {
+    SKIP_IF_CUDA(true, "RNN primitive not supported for CUDA");
+
 #if !DNNL_X64
     return;
 #endif
@@ -381,6 +407,7 @@ TEST_F(runtime_attr_test_t, TestRNN) {
 }
 
 TEST_F(runtime_attr_test_t, TestShuffle) {
+    SKIP_IF_CUDA(true, "Shuffle primitive not supported for CUDA");
     memory::desc md {{1, 16, 3, 3}, data_type::f32, tag::abcd};
     shuffle_forward::desc op_d(prop_kind::forward, md, 1, 4);
     CHECK_OK(shuffle_forward::primitive_desc(op_d, eng));
