@@ -96,12 +96,43 @@ class inner_product_test_bwd_data_t
 protected:
     void SetUp() override {
         auto p = ::testing::TestWithParam<inprod_test_params_t>::GetParam();
+        SKIP_IF_CUDA(!cuda_check_format_tags(p.diff_src_format,
+                             p.weights_format, p.diff_dst_format),
+                "Unsupported format tag");
+        SKIP_IF_CUDA(p.ndims > 5, "Unsupported number of dimensions");
         catch_expected_failures(
                 [=]() { Test(); }, p.expect_to_fail, p.expected_status);
     }
 
-    void Test() {
+    bool cuda_check_format_tags(memory::format_tag diff_src_format,
+            memory::format_tag wei_format, memory::format_tag diff_dst_format) {
+        bool diff_src_ok = diff_src_format == memory::format_tag::ncdhw
+                || diff_src_format == memory::format_tag::ndhwc
+                || diff_src_format == memory::format_tag::nchw
+                || diff_src_format == memory::format_tag::nhwc
+                || diff_src_format == memory::format_tag::ncw
+                || diff_src_format == memory::format_tag::nwc
+                || diff_src_format == memory::format_tag::nc
+                || diff_src_format == memory::format_tag::any;
+        bool wei_ok = wei_format == memory::format_tag::oidhw
+                || wei_format == memory::format_tag::odhwi
+                || wei_format == memory::format_tag::dhwio
+                || wei_format == memory::format_tag::oihw
+                || wei_format == memory::format_tag::hwio
+                || wei_format == memory::format_tag::ohwi
+                || wei_format == memory::format_tag::oiw
+                || wei_format == memory::format_tag::owi
+                || wei_format == memory::format_tag::wio
+                || wei_format == memory::format_tag::io
+                || wei_format == memory::format_tag::oi
+                || wei_format == memory::format_tag::any;
+        bool diff_dst_ok = diff_dst_format == memory::format_tag::any
+                || diff_dst_format == memory::format_tag::nc;
 
+        return diff_src_ok && wei_ok && diff_dst_ok;
+    }
+
+    void Test() {
         auto p = ::testing::TestWithParam<inprod_test_params_t>::GetParam();
         test_inner_product_descr_t ipd = p.test_ipd;
         bool has_spatial = ipd.kh > 1 || ipd.kw > 1;
@@ -281,27 +312,42 @@ INSTANTIATE_TEST_SUITE_P(TestInnerProductBackwardData, inner_product_test_float,
                 inprod_test_params_float {memory::format_tag::any,
                         memory::format_tag::any, memory::format_tag::any,
                         EXPAND_SIZES_2D(2, 1024, 48, 2, 2)},
-                inprod_test_params_float {memory::format_tag::nhwc,
-                        memory::format_tag::hwio, memory::format_tag::nc,
-                        EXPAND_SIZES_2D(2, 32, 48, 6, 6)},
+                inprod_test_params_float {memory::format_tag::nwc,
+                        memory::format_tag::oiw, memory::format_tag::nc,
+                        EXPAND_SIZES_1D(2, 32, 48, 6)},
                 inprod_test_params_float {memory::format_tag::nwc,
                         memory::format_tag::wio, memory::format_tag::nc,
                         EXPAND_SIZES_1D(2, 32, 48, 5)},
+                inprod_test_params_float {memory::format_tag::nwc,
+                        memory::format_tag::owi, memory::format_tag::nc,
+                        EXPAND_SIZES_1D(2, 32, 48, 5)},
                 inprod_test_params_float {memory::format_tag::ncw,
                         memory::format_tag::oiw, memory::format_tag::nc,
                         EXPAND_SIZES_1D(2, 32, 48, 5)},
                 inprod_test_params_float {memory::format_tag::ncw,
                         memory::format_tag::wio, memory::format_tag::nc,
                         EXPAND_SIZES_1D(2, 32, 48, 5)},
+                inprod_test_params_float {memory::format_tag::ncw,
+                        memory::format_tag::owi, memory::format_tag::nc,
+                        EXPAND_SIZES_1D(2, 32, 48, 5)},
                 inprod_test_params_float {memory::format_tag::nhwc,
                         memory::format_tag::hwio, memory::format_tag::nc,
                         EXPAND_SIZES_2D(2, 32, 48, 6, 6)},
                 inprod_test_params_float {memory::format_tag::nhwc,
                         memory::format_tag::oihw, memory::format_tag::nc,
                         EXPAND_SIZES_2D(2, 32, 48, 6, 6)},
+                inprod_test_params_float {memory::format_tag::nhwc,
+                        memory::format_tag::iohw, memory::format_tag::nc,
+                        EXPAND_SIZES_2D(2, 32, 48, 6, 6)},
                 inprod_test_params_float {memory::format_tag::nchw,
                         memory::format_tag::oihw, memory::format_tag::nc,
                         EXPAND_SIZES_2D(2, 32, 48, 6, 6)},
+                inprod_test_params_float {memory::format_tag::nchw,
+                        memory::format_tag::hwio, memory::format_tag::nc,
+                        EXPAND_SIZES_2D(2, 32, 48, 6, 6)},
+                inprod_test_params_float {memory::format_tag::nchw,
+                        memory::format_tag::ohwi, memory::format_tag::nc,
+                        EXPAND_SIZES_2D(2, 32, 48, 6, 6)},
                 inprod_test_params_float {memory::format_tag::nChw8c,
                         memory::format_tag::aBcd8b, memory::format_tag::nc,
                         EXPAND_SIZES_2D(2, 32, 48, 6, 6)},
@@ -339,6 +385,18 @@ INSTANTIATE_TEST_SUITE_P(TestInnerProductBackwardData3D,
                 inprod_test_params_float {memory::format_tag::ncdhw,
                         memory::format_tag::oidhw, memory::format_tag::nc,
                         EXPAND_SIZES_3D(2, 32, 48, 6, 6, 6)},
+                inprod_test_params_float {memory::format_tag::ncdhw,
+                        memory::format_tag::odhwi, memory::format_tag::nc,
+                        EXPAND_SIZES_3D(2, 32, 48, 6, 6, 6)},
+                inprod_test_params_float {memory::format_tag::ncdhw,
+                        memory::format_tag::dhwio, memory::format_tag::nc,
+                        EXPAND_SIZES_3D(2, 32, 48, 6, 6, 6)},
+                inprod_test_params_float {memory::format_tag::ndhwc,
+                        memory::format_tag::oidhw, memory::format_tag::nc,
+                        EXPAND_SIZES_3D(2, 16, 48, 3, 3, 3)},
+                inprod_test_params_float {memory::format_tag::ndhwc,
+                        memory::format_tag::odhwi, memory::format_tag::nc,
+                        EXPAND_SIZES_3D(2, 16, 48, 3, 3, 3)},
                 inprod_test_params_float {memory::format_tag::ndhwc,
                         memory::format_tag::dhwio, memory::format_tag::nc,
                         EXPAND_SIZES_3D(2, 16, 48, 3, 3, 3)}));
