@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2020 Intel Corporation
+* Copyright 2019-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -95,6 +95,26 @@ protected:
 
         SKIP_IF(unsupported_data_type(p.base.src.dt),
                 "Engine does not support this data type.");
+        SKIP_IF(unsupported_data_type(p.base.weights.dt),
+                "Engine does not support this data type.");
+        SKIP_IF(unsupported_data_type(p.base.dst.dt),
+                "Engine does not support this data type.");
+        SKIP_IF(unsupported_data_type(p.base.bia_dt),
+                "Engine does not support this data type.");
+        SKIP_IF(get_test_engine_kind() == engine::kind::gpu
+                        && ((p.attr.zero_points.src & P::PER_N)
+                                || (p.attr.zero_points.dst & P::PER_N)),
+                "Per dimensional zero points are not supported on GPU");
+        SKIP_IF(get_test_engine_kind() == engine::kind::cpu
+                        && p.base.src.tag == impl::format_tag::AB8a4b,
+                "Don't test blocked formats on CPU");
+
+        SKIP_IF_CUDA((p.attr.zero_points.src != 0 || p.attr.zero_points.dst != 0
+                             || p.attr.zero_points.weights != 0),
+                "Zero points not supported for CUDA");
+
+        SKIP_IF_CUDA((p.attr.scale_flags & P::MASK_MASK) == P::PER_N,
+                "Per dimensional scaling is not supported for CUDA");
 
         catch_expected_failures(
                 [=]() { Test(); }, p.expect_to_fail, p.expected_status, false);
@@ -152,6 +172,7 @@ protected:
                 scales_m = test::make_memory(
                         {{scale_size}, memory::data_type::f32, {1}}, eng);
                 auto s = map_memory<float>(scales_m);
+                GTEST_EXPECT_NE(s, nullptr);
                 for (memory::dim i = 0; i < scale_size; ++i)
                     s[i] = 2.f;
             } else {
@@ -194,6 +215,7 @@ protected:
                 zero_points_m = test::make_memory(
                         {{zero_points_size}, memory::data_type::s32, {1}}, eng);
                 auto z = map_memory<int32_t>(zero_points_m);
+                GTEST_EXPECT_NE(z, nullptr);
                 for (memory::dim i = 0; i < zero_points_size; ++i)
                     z[i] = (arg % 7) - 3;
             } else {
@@ -283,6 +305,7 @@ protected:
         auto set_to_zero = [](memory &m) {
             if (m) {
                 auto p = map_memory<char>(m);
+                GTEST_EXPECT_NE(p, nullptr);
                 memset(p, 0, m.get_desc().get_size());
             }
         };
@@ -373,10 +396,17 @@ static auto cases_ef = []() {
                      {{10, 20}, data_type::f32, tag::ab}, data_type::u8},
                     {}, true, dnnl_unimplemented});
     // XXX: disable assert in type_helpers.hpp: default_accum_data_type(...)
-    //cases.push_back({{{{10, 1}, data_type::u8, tag::ab}, {{1, 20}, data_type::u8, tag::ab},
+    // cases.push_back({{{{10, 1}, data_type::u8, tag::ab}, {{1, 20},
+    // data_type::u8, tag::ab},
     //                           {{10, 20}, data_type::u8, tag::ab}},
     //        {}, true, dnnl_unimplemented});
 
+    // unimplemented formats (GPU only)
+    cases.push_back({{{{16, 16}, data_type::f32, tag::AB8a4b},
+                             {{16, 16}, data_type::f32, tag::AB8a4b},
+                             {{16, 16}, data_type::f32, tag::AB8a4b}},
+            {}, true, dnnl_unimplemented});
+
     return ::testing::ValuesIn(cases);
 };
 INSTANTIATE_TEST_SUITE_P(EF, iface, cases_ef());
@@ -497,6 +527,13 @@ static auto cases_x8 = [](memory::data_type src_dt, memory::data_type dst_dt) {
                                     P::ZERO_POINTS | P::DST | P::COMMON
                                             | P::RUNTIME}}});
 
+    // per_dim_1 zero points + runtime
+    cases.push_back({{{{10, 2}, src_dt, tag::ba},
+                             {{2, 20}, data_type::s8, tag::ab},
+                             {{10, 20}, dst_dt, tag::ab}, data_type::f32},
+            {P::SCALES | P::COMMON | P::RUNTIME,
+                    {P::ZERO_POINTS | P::SRC | P::PER_N | P::RUNTIME, P::NONE,
+                            P::ZERO_POINTS | P::DST | P::PER_N | P::RUNTIME}}});
     // post-ops
     cases.push_back({{{{10, 1}, src_dt, tag::ab},
                              {{1, 20}, data_type::s8, tag::ab},
