@@ -215,8 +215,19 @@ private:
     stream strm;
 
 protected:
+    bool cuda_supported_format_tag(memory::format_tag tag) {
+        return impl::utils::one_of(
+                tag, dnnl_abc, dnnl_abcd, dnnl_acb, dnnl_acdb);
+    }
     void SetUp() override {
         p = ::testing::TestWithParam<decltype(p)>::GetParam();
+        SKIP_IF_CUDA(p.aalgorithm == algorithm::resampling_nearest,
+                "nearet algorithm is not supported for cudnn backend");
+        SKIP_IF_CUDA(p.ndims == 5,
+                "cudnn resampling backend does not support 5d tensor");
+        SKIP_IF_CUDA(!cuda_supported_format_tag(p.src_format),
+                "Unsupported format tag");
+
         catch_expected_failures(
                 [=]() { Test(); }, p.expect_to_fail, p.expected_status);
     }
@@ -368,6 +379,26 @@ INSTANTIATE_TEST_SUITE_P(TestResampleForwardPlainLinear, resampling_test_float,
                         EXPAND_SIZES_3D(
                                 1, 16, 5, 10, 1, 10, 5, 1, 2.f, 0.5f, 1.f)}));
 
+GPU_INSTANTIATE_TEST_SUITE_P(TestResamplePlainLinear, resampling_test_float,
+        ::testing::Values(
+                resampling_test_params_t {prop_kind::forward,
+                        algorithm::resampling_linear, memory::format_tag::ncw,
+                        EXPAND_SIZES_1D(1, 1, 5, 10, 2.f)},
+                resampling_test_params_t {prop_kind::forward,
+                        algorithm::resampling_linear, memory::format_tag::ncw,
+                        EXPAND_SIZES_1D(1, 1, 525, 5, 0.01f)},
+                resampling_test_params_t {prop_kind::forward,
+                        algorithm::resampling_linear, memory::format_tag::ncw,
+                        EXPAND_SIZES_1D(13, 10, 7, 13, 1.99f)},
+                resampling_test_params_t {prop_kind::forward,
+                        algorithm::resampling_linear, memory::format_tag::ncw,
+                        EXPAND_SIZES_1D(10, 16, 7, 13, 1.9f)},
+                resampling_test_params_t {prop_kind::forward,
+                        algorithm::resampling_linear, memory::format_tag::nchw,
+                        EXPAND_SIZES_2D(32, 10, 14, 7, 29, 5, 2.1f, 0.72f)},
+                resampling_test_params_t {prop_kind::forward,
+                        algorithm::resampling_linear, memory::format_tag::nhwc,
+                        EXPAND_SIZES_2D(2, 14, 5, 5, 2, 3, 0.5f, 0.6f)}));
 INSTANTIATE_TEST_SUITE_P(TestResampleForwardBlockedLinear,
         resampling_test_float,
         ::testing::Values(
